SystemOrganization addCategory: #'XML-Parser'!
SystemOrganization addCategory: 'XML-Parser-Clients'!
SystemOrganization addCategory: 'XML-Parser-DOM'!
SystemOrganization addCategory: 'XML-Parser-DTD'!
SystemOrganization addCategory: 'XML-Parser-Exceptions'!
SystemOrganization addCategory: 'XML-Parser-HTML'!
SystemOrganization addCategory: 'XML-Parser-Namespaces'!
SystemOrganization addCategory: 'XML-Parser-Validation'!
SystemOrganization addCategory: 'XML-Parser-Validation-DFA'!

Error subclass: #XMLParserException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLDOMException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLFileIOException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLHTTPException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLLimitException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLNamespaceException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLValidationException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLWellFormednessException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!String methodsFor: '*xml-parser' stamp: 'JAAyer 4/14/2010 21:26'!
asXMLString
	^ XMLString string: self! !

!String methodsFor: '*xml-parser' stamp: 'monty 12/9/2013 05:06'!
splitQualifiedNameInto: aTwoArgumentBlock
	"Split the name into namespace prefix and local name (the block arguments).
	(Optimized code; profile when modifying.)"
	| i |

	i := self size - 1.
	[i > 1]
		whileTrue: [
			((self at: i) == $:)
				ifTrue: [
					^ aTwoArgumentBlock
						value: (self copyFrom: 1 to: i - 1)
						value: (self copyFrom: i + 1 to: self size)].
			i := i - 1].
	^ aTwoArgumentBlock
		value: ''
		value: self! !

Collection subclass: #XMLNFAStateSet
	instanceVariableNames: 'set isAccepting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAStateSet commentStamp: 'monty 12/28/2013 19:40' prior: 0!
This class is a set of NFA states. It wraps an IdentitySet!

!XMLNFAStateSet methodsFor: 'comparing' stamp: 'monty 12/28/2013 18:50'!
= anObject
	^ anObject == self
		or: [anObject class == self class
			and: [anObject set = set]]! !

!XMLNFAStateSet methodsFor: 'adding' stamp: 'monty 12/28/2013 18:52'!
add: aState
	isAccepting
		ifFalse: [isAccepting := aState isAccepting].

	^ set add: aState.! !

!XMLNFAStateSet methodsFor: 'enumerating' stamp: 'monty 12/28/2013 18:47'!
do: aBlock
	set do: aBlock! !

!XMLNFAStateSet methodsFor: 'comparing' stamp: 'monty 12/28/2013 18:50'!
hash
	^ set hash! !

!XMLNFAStateSet methodsFor: 'testing' stamp: 'monty 12/28/2013 18:48'!
includes: aState
	^ set includes: aState! !

!XMLNFAStateSet methodsFor: 'initialization' stamp: 'monty 12/28/2013 18:52'!
initialize
	super initialize.

	set := IdentitySet new.
	isAccepting := false.! !

!XMLNFAStateSet methodsFor: 'testing' stamp: 'monty 12/28/2013 18:53'!
isAccepting
	^ isAccepting! !

!XMLNFAStateSet methodsFor: 'removing' stamp: 'monty 12/28/2013 18:46'!
remove: aState ifAbsent: anExceptionBlock
	set
		remove: aState
		ifAbsent: [^ anExceptionBlock value].

	isAccepting
		ifTrue: [isAccepting := set anySatisfy: [:each | each isAccepting]].

	^ aState.! !

!XMLNFAStateSet methodsFor: 'accessing' stamp: 'monty 12/28/2013 18:50'!
set
	^ set! !

!XMLNFAStateSet methodsFor: 'accessing' stamp: 'monty 12/28/2013 19:01'!
size
	^ set size! !

SequenceableCollection subclass: #XMLOrderedList
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLOrderedList commentStamp: '<historical>' prior: 0!
This class functions as an OrderedCollection that will not raise exceptions when attempts are made to access absent objects. This means messages like #at:, #after:, and #first will return nil when the requested object cannot be retrieved. Messages that modify the collection based on the presence or absence of other objects such as #add:after: or #at:put: will still raise exceptions as usual.

!

XMLOrderedList subclass: #XMLObservableList
	instanceVariableNames: 'notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLObservableList commentStamp: '<historical>' prior: 0!
This class is an is an ordered list that sends notifications to observers when objects are added to it or removed from it. Subclasses should override the #beforeAdding:, #afterAdding: and #afterRemoving: messages to send custom notifications.!

XMLObservableList subclass: #XMLAttributeList
	instanceVariableNames: 'attributesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttributeList commentStamp: '<historical>' prior: 0!
This class represents an ordered list of attribute nodes that also keeps a dictionary mapping attribute names to nodes for faster lookups. You usually shouldn't have to use this class directly.!

!XMLAttributeList methodsFor: 'adding' stamp: 'JAAyer 10/15/2010 12:41'!
addName: aName value: aValue
	^ self addLast:
		(XMLAttribute
			name: aName
			value: aValue)! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 12/6/2010 21:25'!
afterAdding: anObject
	anObject addObserver: self.
	self attributesByName
		at: anObject name ifPresent: [self removeAttributeNodeAt: anObject name];
		at: anObject name put: anObject.

	self notifyOf: #addedAttribute: with: anObject.! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
afterRemoving: anAttribute
	anAttribute removeObserver: self.
	self attributesByName removeKey: anAttribute name.
	self notifyOf: #removedAttribute: with: anAttribute.! !

!XMLAttributeList methodsFor: 'converting' stamp: 'JAAyer 12/6/2010 22:55'!
asDictionary
	^ (OrderPreservingDictionary newFrom: self asNameValueAssociations) defaultValue: ''! !

!XMLAttributeList methodsFor: 'converting' stamp: 'JAAyer 6/25/2010 22:15'!
asNameValueAssociations
	^ self collect: [:each | each name -> each value]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:08'!
attributeNames
	^ self collect: [:each | each name]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:14'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self do: [:each | aTwoArgumentBlock value: each name value: each value]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:09'!
attributeNamesDo: aBlock
	self do: [:each | aBlock value: each name]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName
	^ self attributeNodeAt: aName ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName ifAbsent: aBlock
	^ self attributesByName at: aName ifAbsent: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri
	^ self attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock
	^ self
		detect: [:each | (each isNamed: aLocalName) and: [each namespaceURI = aUri]]
		ifNone: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:11'!
attributeValueAt: aName
	^ self attributeValueAt: aName ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:11'!
attributeValueAt: aName ifAbsent: aBlock
	^ (self attributesByName
		at: aName
		ifAbsent: [^ aBlock value]) value! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 7/11/2010 03:34'!
attributeValueAt: aName ifAbsentPut: aBlock
	^ self
		attributeValueAt: aName
		ifAbsent: [self attributeValueAt: aName put: aBlock value]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 12/6/2010 21:23'!
attributeValueAt: aName put: aValue
	(self includesAttribute: aName)
		ifTrue: [(self attributeNodeAt: aName) value: aValue]
		ifFalse: [self addName: aName value: aValue].
	^ aValue.! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 6/10/2010 05:51'!
attributesByName
	^ attributesByName ifNil: [attributesByName := Dictionary new]! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
beforeAdding: anObject
	anObject element
		ifNotNil: [XMLDOMException signal: 'Cannot add attribute node from other element']! !

!XMLAttributeList methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:23'!
includesAttribute: aName
	^ self attributesByName includesKey: aName! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 12/6/2010 21:24'!
removeAttributeNodeAt: aName
	^ self remove: (self attributeNodeAt: aName).! !

!XMLAttributeList methodsFor: 'notifications' stamp: 'JAAyer 6/15/2010 15:13'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self attributesByName
		removeKey: anOldName;
		at: aNewName put: anAttribute! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 12/6/2013 08:33'!
setFromDictionary: aDictionary andElement: anElement
	| attributeAssoc attributeNode |

	"Hack to reuse a dictionary for speed"
	attributesByName :=
		aDictionary class == OrderPreservingDictionary
			ifTrue: [aDictionary dictionary]
			ifFalse: [aDictionary].

	aDictionary keysDo: [:each |
		attributeAssoc :=
			attributesByName
				associationAt: each
				ifAbsent: [nil].
		(attributeNode :=
			XMLAttribute
				name: each
				value: attributeAssoc value)
			addObserver: self;
			addObserver: anElement;
			element: anElement.
		attributeAssoc value: attributeNode.
		collection add: attributeNode].! !

XMLObservableList subclass: #XMLInternalSubsetList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLInternalSubsetList commentStamp: 'JAAyer 7/27/2010 12:45' prior: 0!
This class represents the list of nodes that comprise a document's internal DTD subset.!

!XMLInternalSubsetList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
afterAdding: aNode
	self notifyOf: #addedNode: with: aNode.! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
afterRemoving: aNode
	self notifyOf: #removedNode: with: aNode.! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
beforeAdding: aNode! !

XMLObservableList subclass: #XMLNodeList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeList commentStamp: '<historical>' prior: 0!
This class represents an ordered list of child nodes for some kind of node with elements, probably either an XML document or element object. Non-modifying accessing messages like #first and #last will never raise exceptions and return nil instead.!

XMLNodeList subclass: #XMLCachingNodeList
	instanceVariableNames: 'elements elementsByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLCachingNodeList commentStamp: '<historical>' prior: 0!
This is a node list that caches child elements by name, keeping both an ordered list of elements and a dictionary that allows for faster retrieval of elements by qualified or local name.

You can inject it into the DOM parser like this:
parser := XMLDOMParser on: xmlSource.
parser nodeListClass: XMLCachingNodeList.!

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:58'!
afterAdding: aNode
	aNode isElement
		ifTrue: [
			((self last == aNode) or: [(self findElementAfter: aNode) isNil])
				ifTrue: [self elementAddedAtEnd: aNode]
				ifFalse: [self elementInsertedBeforeEnd: aNode]].
	super afterAdding: aNode.
			! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 22:01'!
afterRemoving: aNode
	aNode isElement
		ifTrue: [
			self elements remove: aNode.
			self elementsByNameAt: aNode name removeElement: aNode.
			aNode hasPrefix
				ifTrue: [self elementsByNameAt: aNode localName removeElement: aNode]].
	super afterRemoving: aNode.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementAddedAtEnd: anElement
	self elements add: anElement.
	self
		elementsByNameAt: anElement name
		addElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				addElement: anElement]! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:01'!
elementAt: aName ifAbsent: aBlock
	| elementsWithName |

	elementsWithName := self elementsByName at: aName ifAbsent: [^ aBlock value].

	^ elementsWithName isCollection
		ifTrue: [elementsWithName first]
		ifFalse: [elementsWithName]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementInsertedBeforeEnd: anElement
	| elementAfter |

	elementAfter := self findElementAfter: anElement.
	self elements add: anElement before: elementAfter.

	self elementsByNameAt: anElement name insertElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				insertElement: anElement]! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:01'!
elements
	^ elements ifNil: [elements := self species new]! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:01'!
elementsAt: aName
	| elementsWithName |

	elementsWithName :=
		self elementsByName
			at: aName
			ifAbsent: [^ self species new].

	^ elementsWithName isCollection
		ifTrue: [elementsWithName]
		ifFalse: [
			self elementsByName
				at: aName
				put: (self species with: elementsWithName)]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementsByName
	^ elementsByName ifNil: [elementsByName := Dictionary new].! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementsByNameAt: aName addElement: anElement
	| elementsWithName |

	elementsWithName := 
		self elementsByName
			at: aName
			ifAbsent: [
				self elementsByName at: aName put: anElement.
				^ self].

	elementsWithName isCollection
		ifTrue: [elementsWithName add: anElement]
		ifFalse: [
			self elementsByName
				at: aName
				put: (self species with: elementsWithName with: anElement)]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementsByNameAt: aName insertElement: anElement
	| elementsWithName elementAfter |

	(elementAfter := self findElementAfter: anElement withName: aName)
		ifNil: [^ self elementsByNameAt: aName addElement: anElement].

	(elementsWithName := self elementsByName at: aName)
		isCollection
			ifTrue: [elementsWithName add: anElement before: elementAfter]
			ifFalse: [
				self elementsByName
					at: aName
					put: (self species with: anElement with: elementAfter)]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementsByNameAt: aName removeElement: anElement
	| elementsWithName |

	(elementsWithName := self elementsByName at: aName)
		isCollection
			ifTrue: [
				elementsWithName remove: anElement.
				elementsWithName ifEmpty: [self elementsByName removeKey: aName]]
			ifFalse: [self elementsByName removeKey: aName]! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:02'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self elements detect: aBlock ifNone: anExceptionBlock! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:02'!
elementsDo: aBlock
	self elements do: aBlock! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:02'!
elementsSelect: aBlock
	^ self elements select: aBlock! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
findElementAfter: anElement
	| previousElement |

	previousElement := nil.
	self do: [:each |
		each isElement
			ifTrue: [
				previousElement == anElement
					ifTrue: [^ each].
				previousElement := each]].
	^ nil.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
findElementAfter: anElement withName: aName
	| previousElementWithName |

	previousElementWithName := nil.
	self do: [:each |
		(each isElement and: [each isNamed: aName])
			ifTrue: [
				(previousElementWithName == anElement)
					ifTrue: [^ each].
				previousElementWithName := each]].
	^ nil.! !

!XMLCachingNodeList methodsFor: 'testing' stamp: 'monty 11/26/2013 22:01'!
hasElements
	^ elements notNil and: [elements notEmpty]! !

!XMLCachingNodeList methodsFor: 'testing' stamp: 'monty 11/26/2013 22:01'!
includesElementAt: aName
	^ self elementsByName includesKey: aName! !

!XMLCachingNodeList methodsFor: 'notifications' stamp: 'monty 11/26/2013 22:03'!
renamedElement: anElement from: anOldName to: aNewName
	self elementsByNameAt: anOldName removeElement: anElement.
	anOldName splitQualifiedNameInto: [:oldPrefix :oldLocalName |
		oldPrefix ifNotEmpty: [
			self
				elementsByNameAt: oldLocalName
				removeElement: anElement]].

	self elementsByNameAt: anElement name insertElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				insertElement: anElement].! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:59'!
afterAdding: aNode
	aNode addObserver: self.
	aNode isElement
		ifTrue: [self notifyOf: #addedElement: with: aNode].
	self notifyOf: #addedNode: with: aNode.! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 22:01'!
afterRemoving: aNode
	aNode removeObserver: self.
	aNode isElement
		ifTrue: [self notifyOf: #removedElement: with: aNode].
	self notifyOf: #removedNode: with: aNode.! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
beforeAdding: aNode
	aNode hasParent
		ifTrue: [XMLDOMException signal: 'Cannot add child node that already has a parent']! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName
	^ self elementAt: aName ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:13'!
elementAt: aName ifAbsent: aBlock
	^ self
		detect: [:each | each isElement and: [each isNamed: aName]]
		ifNone: aBlock! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri
	^ self elementAt: aName namespaceURI: aUri ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri ifAbsent: aBlock
	^ (self elementsAt: aName)
		detect: [:each | each namespaceURI = aUri]
		ifNone: aBlock! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:07'!
elements
	^ self select: [:each | each isElement]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:12'!
elementsAt: aName
	^ self elementsSelect: [:each | each isNamed: aName]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:33'!
elementsAt: aName namespaceURI: aUri
	^ (self elementsAt: aName) select: [:each | each namespaceURI = aUri]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:34'!
elementsCollect: aBlock
	^ self elements collect: aBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock
	^ self elementsDetect: aBlock ifNone: [self defaultValue]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:10'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self
		detect: [:each | each isElement and: [aBlock value: each]]
		ifNone: anExceptionBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:11'!
elementsDo: aBlock
	self do: [:each | each isElement ifTrue: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:08'!
elementsSelect: aBlock
	^ self select: [:each | each isElement and: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 11/26/2013 22:03'!
hasElements
	^ self anySatisfy: [:each | each isElement]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 11/26/2013 22:04'!
includesElementAt: aName
	^ (self elementAt: aName) notNil! !

!XMLNodeList methodsFor: 'notifications' stamp: 'monty 11/26/2013 22:03'!
renamedElement: anElement from: anOldName to: aNewName! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 14:46'!
add: anObject after: aPriorObject
	^ self with: [super add: anObject after: aPriorObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:56'!
add: anObject afterIndex: anIndex
	^ self with: [super add: anObject afterIndex: anIndex] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 14:46'!
add: anObject before: aSubsequentObject
	^ self with: [super add: anObject before: aSubsequentObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
add: anObject beforeIndex: anIndex
	^ self with: [super add: anObject beforeIndex: anIndex] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
addFirst: anObject
	^ self with: [super addFirst: anObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
addLast: anObject
	^ self with: [super addLast: anObject] add: anObject! !

!XMLObservableList methodsFor: 'observing' stamp: 'OttoBehrens 2/14/2011 17:32'!
addObserver: anObserver
	self notifier addObserver: anObserver! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 11/23/2010 05:41'!
afterAdding: anObject
	self notifyOf: #added:to: with: anObject with: self.! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
afterRemoving: anObject
	self notifyOf: #removed:from: with: anObject with: self.! !

!XMLObservableList methodsFor: 'accessing' stamp: 'JAA 3/21/2011 20:08'!
at: anIndex put: anObject
	| presentObject |

	(presentObject := self at: anIndex)
		ifNotNil: [
			self
				with: [super at: anIndex put: nil]
				remove: presentObject].

	^ self with: [super at: anIndex put: anObject] add: anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
beforeAdding: anObject
	self notifyOf: #adding:to: with: anObject with: self! !

!XMLObservableList methodsFor: 'accessing' stamp: 'JAAyer 6/11/2010 10:50'!
notifier
	^ notifier ifNil: [notifier := XMLNotifier new]! !

!XMLObservableList methodsFor: 'notification' stamp: 'OttoBehrens 2/14/2011 17:32'!
notifyOf: aSelector with: anArgument
	self notifier notifyOf: aSelector with: anArgument! !

!XMLObservableList methodsFor: 'notification' stamp: 'OttoBehrens 2/14/2011 17:32'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument
	self notifier notifyOf: aSelector with: aFirstArgument with: aSecondArgument! !

!XMLObservableList methodsFor: 'notification' stamp: 'OttoBehrens 2/14/2011 17:32'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument with: aThirdArgument
	self notifier
		notifyOf: aSelector
		with: aFirstArgument
		with: aSecondArgument
		with: aThirdArgument! !

!XMLObservableList methodsFor: 'notification' stamp: 'OttoBehrens 2/14/2011 17:32'!
notifyOf: aSelector withArguments: anArray
	self notifier notifyOf: aSelector withArguments: anArray! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 7/22/2010 10:58'!
remove: anObject ifAbsent: aBlock
	^ self with: [super remove: anObject ifAbsent: [^ aBlock value]] remove: anObject! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 6/11/2010 09:03'!
removeAt: anIndex
	^ self with: [super removeAt: anIndex] remove: (self at: anIndex)! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 6/11/2010 09:03'!
removeFirst
	^ self removeAt: 1! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 6/11/2010 09:03'!
removeLast
	^ self removeAt: self size! !

!XMLObservableList methodsFor: 'observing' stamp: 'OttoBehrens 2/14/2011 17:32'!
removeObserver: anObserver
	self notifier removeObserver: anObserver! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
with: aBlock add: anObject
	self
		beforeAdding: anObject;
		with: [aBlock value] onError: [^ nil];
		afterAdding: anObject.
	^ anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'JAA 3/26/2011 19:04'!
with: aBlock onError: errorBlock
	^ aBlock
		on: Error
		do: [:error |
			error signal.
			errorBlock valueWithPossibleArgument: error]! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
with: aBlock remove: anObject
	self
		with: [aBlock value] onError: [^ nil];
		afterRemoving: anObject.
	^ anObject.! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:44'!
new
	^ self new: 10! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 16:33'!
new: aSize
	^ self basicNew setCollection: (OrderedCollection new: aSize)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:41'!
new: aSize withAll: anObject
	^ self basicNew setCollection: (OrderedCollection new: aSize withAll: anObject)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:42'!
newFrom: aCollection
	^ self basicNew setCollection: (OrderedCollection newFrom: aCollection)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:43'!
ofSize: aSize
	^ self new: aSize withAll: nil! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:01'!
add: anObject
	^ self addLast: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:08'!
add: anObject after: aPriorObject
	^ collection add: anObject after: aPriorObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject afterIndex: anIndex
	^ collection add: anObject afterIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject before: aSubsequentObject
	^ collection add: anObject before: aSubsequentObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:10'!
add: anObject beforeIndex: anIndex
	^ collection add: anObject beforeIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:02'!
addAll: aCollection
	^ self addAllLast: aCollection! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:10'!
addAllFirst: aCollection
	aCollection reverseDo: [:each | self addFirst: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:11'!
addAllLast: aCollection
	aCollection do: [:each | self addLast: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:11'!
addFirst: anObject
	^ collection addFirst: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:12'!
addLast: anObject
	^ collection addLast: anObject! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:48'!
after: anObject
	^ collection after: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'converting' stamp: 'JAAyer 8/4/2010 12:53'!
asArray
	^ collection asArray! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:49'!
at: anIndex
	^ collection at: anIndex ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:12'!
at: anIndex ifAbsentPut: aValue
	^ self
		at: anIndex
		ifAbsent: [self at: anIndex put: aValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAA 3/25/2011 14:32'!
at: anIndex put: aValue
	^ collection at: anIndex put: aValue! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/5/2010 00:55'!
atLast: anIndexFromEnd
	^ self atLast: anIndexFromEnd ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:50'!
before: anObject
	^ collection before: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 13:14'!
capacity
	^ collection capacity! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
collect: aBlock
	^ self newWithCollection: (collection collect: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
collect: aBlock from: aStartIndex to: anEndIndex
	^ self newWithCollection:
		(collection collect: aBlock from: aStartIndex to: anEndIndex)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
collect: aCollectBlock thenSelect: aSelectBlock
	^ self newWithCollection:
		(collection collect: aCollectBlock thenSelect: aSelectBlock)! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAA 3/25/2011 15:35'!
collection
	^ collection! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAAyer 8/4/2010 14:54'!
copy
	^ self species basicNew setCollection: collection copy! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAAyer 3/12/2010 07:30'!
copyEmpty
	^ self species new: self size! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAAyer 11/28/2010 13:04'!
copyFrom: aStart to: anEnd
	^ self newWithCollection:
		(collection copyFrom: (aStart max: 1) to: (self size min: anEnd))! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAA 3/25/2011 15:18'!
copyWith: anElement 
	^ self copy
	 	add: anElement;
		yourself! !

!XMLOrderedList methodsFor: 'defaults' stamp: 'JAAyer 3/12/2010 06:48'!
defaultValue
	^ nil! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 12:49'!
do: aBlock
	collection do: aBlock! !

!XMLOrderedList methodsFor: 'inspecting' stamp: 'SD 3/3/2013 10:28'!
inspectorClass
	^ Smalltalk tools inspector! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 11/28/2010 13:04'!
newWithCollection: aCollection
	^ self species basicNew setCollection: aCollection! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:33'!
remove: anObject ifAbsent: aBlock
	^ collection remove: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 16:00'!
removeAll
	[self isEmpty] whileFalse: [self removeLast]! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 15:46'!
removeAt: anIndex
	^ collection removeAt: anIndex! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:34'!
removeFirst
	^ collection removeFirst! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:34'!
removeFirst: aNumber
	^ collection removeFirst: aNumber! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:34'!
removeLast
	^ collection removeLast! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:34'!
removeLast: aNumber
	^ collection removeLast: aNumber! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'SD 3/3/2013 10:32'!
replaceFrom: aStartingIndex to: anEndingIndex with: aCollection startingAt: aReplacementStart

	collection
		replaceFrom: aStartingIndex
		to: anEndingIndex
		with: aCollection
		startingAt: aReplacementStart! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 13:35'!
reverseDo: aBlock
	collection reverseDo: aBlock! !

!XMLOrderedList methodsFor: 'converting' stamp: 'JAA 3/26/2011 18:54'!
reversed
	| reversed |

	reversed := self species new.
	self reverseDo: [:each | reversed addLast: each].
	^ reversed.
! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
select: aBlock
	^ self newWithCollection: (collection select: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
select: aSelectBlock thenCollect: aCollectBlock
	^ self newWithCollection:
		(collection select: aSelectBlock thenCollect: aCollectBlock)! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 8/4/2010 13:38'!
setCollection: aCollection
	collection := aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:49'!
size
	^ collection size! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'JAAyer 8/4/2010 13:43'!
sort
	collection sort! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'JAAyer 8/4/2010 13:43'!
sort: aSortBlock
	collection sort: aSortBlock! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 3/12/2010 11:02'!
species
	^ XMLOrderedList! !

!XMLOrderedList methodsFor: 'private' stamp: 'OttoBehrens 2/16/2011 18:36'!
speciesForCollect
	^ self species! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
with: aCollection collect: aBlock
	^ self newWithCollection: (collection with: aCollection collect: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
withIndexCollect: aTwoArgumentBlock
	^ self newWithCollection: (collection withIndexCollect: aTwoArgumentBlock)! !

Object subclass: #DTDAttributeDefinitionParser
	instanceVariableNames: 'element attribute type defaultPragma defaultValue'
	classVariableNames: 'DefaultValueValidators NonEnumerationValidators'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDAttributeDefinitionParser commentStamp: '<historical>' prior: 0!
This class parses an attribute definition in an ATTLIST declaration and creates a validator for it.!

!DTDAttributeDefinitionParser class methodsFor: 'instance creation' stamp: 'monty 12/30/2013 23:53'!
element: anElement attribute: anAttribute type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	^ self new
		setElement: anElement
		attribute: anAttribute
		type: aType
		defaultPragma: aDefaultPragma
		defaultValue: aDefaultValue! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 12/31/2013 02:54'!
initialize
	"self initialize"

	super initialize.
	self
		initializeDefaultValueValidators;
		initializeNonEnumerationValidators.! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 12/31/2013 02:53'!
initializeDefaultValueValidators
	(DefaultValueValidators := Dictionary new)
		at: 'FIXED' put: XMLFixedValueAttributeValidator;
		at: 'IMPLIED' put: XMLImpliedValueAttributeValidator;
		at: 'REQUIRED' put: XMLRequiredValueAttributeValidator! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 12/31/2013 02:54'!
initializeNonEnumerationValidators
	(NonEnumerationValidators := Dictionary new)
		at: 'CDATA' put: XMLCDataAttributeValidator;
		at: 'ID' put: XMLIDAttributeValidator;
		at: 'IDREF' put: XMLIDRefAttributeValidator;
		at: 'IDREFS' put: XMLIDRefsAttributeValidator;
		at: 'ENTITY' put: XMLEntityAttributeValidator;
		at: 'ENTITIES' put: XMLEntitiesAttributeValidator;
		at: 'NMTOKEN' put: XMLNameAttributeValidator;
		at: 'NMTOKENS' put: XMLNamesAttributeValidator! !

!DTDAttributeDefinitionParser methodsFor: 'testing' stamp: 'monty 12/11/2013 02:38'!
isNotationList: aType
	^ aType beginsWith: 'NOTATION '! !

!DTDAttributeDefinitionParser methodsFor: 'parsing' stamp: 'monty 1/4/2014 04:43'!
parse
	| defaultValueValidator attributeValidator |

	defaultValueValidator :=
		(DefaultValueValidators
			at: defaultPragma
			ifAbsent: [XMLDefaultValueAttributeValidator])
				attribute: attribute
				defaultValue: defaultValue.

	attributeValidator :=
		self validatorClassForType
			element: element
			attribute: attribute
			defaultValueValidator: defaultValueValidator.
	attributeValidator isValueListValidator
		ifTrue: [self parseValueListTypeFor: attributeValidator].

	^ attributeValidator.! !

!DTDAttributeDefinitionParser methodsFor: 'parsing' stamp: 'monty 1/4/2014 04:59'!
parseValueListTypeFor: anAttributeValidator
	| listParserClass isNotationList |

	isNotationList := self isNotationList: type.
	listParserClass :=
		isNotationList
			ifTrue: [DTDNotationListParser]
			ifFalse: [DTDEnumerationListParser].

	anAttributeValidator allowedValues: (listParserClass on: type) parse.! !

!DTDAttributeDefinitionParser methodsFor: 'initialization' stamp: 'monty 12/30/2013 23:52'!
setElement: anElement attribute: anAttribute type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	element := anElement.
	attribute := anAttribute.
	type := aType.
	defaultPragma := aDefaultPragma.
	defaultValue := aDefaultValue.! !

!DTDAttributeDefinitionParser methodsFor: 'parsing' stamp: 'monty 1/4/2014 03:43'!
validatorClassForType
	type ifEmpty: [XMLValidationException signal: 'Missing attribute type'].

	^ NonEnumerationValidators 
		at: type
		ifAbsent: [XMLValueListAttributeValidator]! !

Object subclass: #DTDContentModelParser
	instanceVariableNames: 'element contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDContentModelParser commentStamp: '<historical>' prior: 0!
This is a base class for DTD content model parsers that turn element declarations into validators.!

!DTDContentModelParser class methodsFor: 'instance creation' stamp: 'monty 12/21/2013 03:14'!
element: aName contentModel: aContentModel
	^ self new
		setElement: aName
		contentModel: aContentModel! !

!DTDContentModelParser class methodsFor: 'parsing' stamp: 'monty 12/21/2013 20:00'!
parserForContentModel: aContentModel
	"the tokenizer removes all spaces from the content model"
	^ (aContentModel notEmpty and: [aContentModel first == $(])
		ifTrue: [
			(aContentModel size > 1 and: [aContentModel second == $#])
				ifTrue: [DTDMixedContentModelParser]
				ifFalse: [DTDDFAContentModelParser]]
		ifFalse: [DTDLiteralContentModelParser]! !

!DTDContentModelParser methodsFor: 'parsing' stamp: 'monty 12/21/2013 19:57'!
parse
	self subclassResponsibility! !

!DTDContentModelParser methodsFor: 'instance creation' stamp: 'monty 12/21/2013 03:13'!
setElement: aName contentModel: aContentModel
	element := aName.
	contentModel := aContentModel.! !

DTDContentModelParser subclass: #DTDDFAContentModelParser
	instanceVariableNames: 'contentModelStream nameStream operatorStack nfaBuilder'
	classVariableNames: 'OperatorApplicators SpecialCharacterParsers SpecialCharacters'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDFAContentModelParser commentStamp: 'monty 12/29/2013 04:00' prior: 0!
This class parses a content model using postfix conversion and an NFABuilder to build a NFA before converting it to a DFA.!

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 12/29/2013 02:35'!
initialize
	"self initialize"

	super initialize.
	self
		initializeOperatorApplicators;
		initializeSpecialCharacterParsers;
		initializeSpecialCharacters.! !

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 12/29/2013 05:08'!
initializeOperatorApplicators
	"use Array for faster lookup"
	(OperatorApplicators := Array new: 128)
		at: $, asciiValue put: #applyConcatenation;
		at: $| asciiValue put: #applyBranch;
		at: $? asciiValue put: #applyOptional;
		at: $* asciiValue put: #applyStar;
		at: $+ asciiValue put: #applyPlus! !

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 12/29/2013 05:08'!
initializeSpecialCharacterParsers
	"use Array for faster lookup"
	(SpecialCharacterParsers := Array new: 128)
		at: $( asciiValue put: #parseParen:;
		at: $) asciiValue put: #parseParen:;
		at: $, asciiValue put: #parseBinaryOperator:;
		at: $| asciiValue put: #parseBinaryOperator:;
		at: $? asciiValue put: #parseUnaryOperator:;
		at: $* asciiValue put: #parseUnaryOperator:;
		at: $+ asciiValue put: #parseUnaryOperator:! !

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 12/29/2013 04:42'!
initializeSpecialCharacters
	(SpecialCharacters := CharacterSet new)
		addAll: '(),|?*+'! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/28/2013 19:46'!
applyBinaryOperatorOnStack
	nfaBuilder totalFragments > 1
		ifFalse: [self errorMissingOperand].

	self applyOperator: operatorStack removeLast.! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/30/2013 22:29'!
applyOperator: aCharacter
	nfaBuilder perform: (OperatorApplicators at: aCharacter asciiValue)! !

!DTDDFAContentModelParser methodsFor: 'errors' stamp: 'monty 12/18/2013 22:44'!
errorMismatchedParens
	XMLValidationException signal: 'Mismatched parentheses in content model'! !

!DTDDFAContentModelParser methodsFor: 'errors' stamp: 'monty 12/18/2013 22:44'!
errorMissingOperand
	XMLValidationException signal: 'Missing element name in content model'! !

!DTDDFAContentModelParser methodsFor: 'testing' stamp: 'monty 12/29/2013 02:37'!
hasOperatorOnStack
	^ operatorStack notEmpty and: [operatorStack last ~~ $(]! !

!DTDDFAContentModelParser methodsFor: 'testing' stamp: 'monty 12/18/2013 22:44'!
isBinaryOperator: aFirstOperator higherPrecedenceThan: aSecondOperator
	^ aFirstOperator == $, and: [aSecondOperator == $|]! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 12/30/2013 22:28'!
parse
	contentModelStream := contentModel readStream.
	nameStream := WriteStream on: (String new: 16).
	operatorStack := OrderedCollection new.
	nfaBuilder := XMLNFABuilder new.

	[contentModelStream atEnd]
		whileFalse: [ 
			(self parserForSpecialCharacter: contentModelStream peek)
				ifNotNil: [:selector |
					self
						perform: selector
						with: contentModelStream next]
				ifNil: [self parseElementName]].
	self popRemainingOperators.

	^ XMLDFAElementValidator
		element: element
		dfa: nfaBuilder firstStateOfAcceptingNFA asDFAState.! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 02:23'!
parseBinaryOperator: aCharacter
	self hasOperatorOnStack
		ifTrue: [
			(self
				isBinaryOperator: aCharacter
				higherPrecedenceThan: operatorStack last)
					ifFalse: [self applyBinaryOperatorOnStack]].

	operatorStack addLast: aCharacter.! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 03:49'!
parseElementName
	| matcher |

	nameStream reset.
	[contentModelStream atEnd
		or: [SpecialCharacters includes: contentModelStream peek]]
		whileFalse: [nameStream nextPut: contentModelStream next].

	matcher := nameStream contents.
	matcher = '#PCDATA'
		ifTrue: [XMLValidationException signal: 'Unexpected #PCDATA in content model'].

	nfaBuilder pushMatcher: matcher.! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 02:24'!
parseParen: aCharacter
	aCharacter == $)
		ifTrue: [self popParenthesizedOperators]
		ifFalse: [operatorStack addLast: $(].! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 02:23'!
parseUnaryOperator: aCharacter
	nfaBuilder hasFragments
		ifFalse: [self errorMissingOperand].

	self applyOperator: aCharacter.! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 12/30/2013 22:29'!
parserForSpecialCharacter: aCharacter
	| i |

	^ ((i := aCharacter asciiValue) > 0
		and: [i <= SpecialCharacterParsers size])
			ifTrue: [SpecialCharacterParsers at: i]
			ifFalse: [nil]! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/28/2013 23:02'!
popParenthesizedOperators
	[operatorStack notEmpty]
		whileTrue: [
			operatorStack last == $(
				ifTrue: [
					operatorStack removeLast.
					^ self]
				ifFalse: [self applyBinaryOperatorOnStack]].

	self errorMismatchedParens.! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/18/2013 22:44'!
popRemainingOperators
	[operatorStack notEmpty]
		whileTrue: [
			operatorStack last == $(
				ifTrue: [self errorMismatchedParens].
			self applyBinaryOperatorOnStack].

	nfaBuilder totalFragments = 1
		ifFalse: [self errorMissingOperand].! !

DTDContentModelParser subclass: #DTDLiteralContentModelParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDLiteralContentModelParser methodsFor: 'parsing' stamp: 'monty 12/21/2013 03:13'!
parse
	contentModel = 'ANY'
		ifTrue: [^ XMLAnyElementValidator element: element].
	contentModel = 'EMPTY'
		ifTrue: [^ XMLEmptyElementValidator element: element].

	XMLValidationException signal: 'Invalid content model "', contentModel, '"'.! !

DTDContentModelParser subclass: #DTDMixedContentModelParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDMixedContentModelParser commentStamp: 'monty 12/28/2013 23:02' prior: 0!
This class parses mixed content declarations like "(#PCDATA|one|two)"
!

!DTDMixedContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 00:44'!
parse
	^ XMLMixedContentElementValidator
		element: element
		allowedElements: self parseAllowedElements! !

!DTDMixedContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 00:43'!
parseAllowedElements
	| allowedElements |

	allowedElements := (DTDMixedContentListParser on: contentModel) parse.
	allowedElements remove: '#PCDATA'.

	(allowedElements isEmpty or: [contentModel last == $*])
		ifFalse: [
			XMLValidationException signal:
				'Missing * after mixed content list with elements'].

	^ allowedElements.! !

Object subclass: #DTDDoctypeDefinition
	instanceVariableNames: 'rootElement externalSubsetUri isStandalone elementValidators attributeValidators generalEntities parameterEntities unparsedEntities notations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDoctypeDefinition commentStamp: '<historical>' prior: 0!
This class represents the parsed internal and external subset of a DTD.!

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 03:44'!
addExternalGeneralEntity: aName resolver: aResolver uri: aSystemUri from: aDocumentOrEntityUri
	self generalEntities
		at: aName
		ifAbsentPut: [
			aResolver
				resolveGeneralEntity: aName
				uri: aSystemUri
				from: aDocumentOrEntityUri]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 03:45'!
addExternalParameterEntity: aName resolver: aResolver uri: aSystemUri from: aDocumentOrEntityUri
	self parameterEntities
		at: aName
		ifAbsentPut: [
			aResolver
				resolveParameterEntity: aName
				uri: aSystemUri
				from: aDocumentOrEntityUri]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 03:44'!
addGeneralEntity: aName replacement: aReplacement from: aDocumentOrEntityUri
	self generalEntities
		at: aName
		ifAbsentPut: [
			DTDGeneralEntity 
				name: aName
				replacement: aReplacement
				uri: aDocumentOrEntityUri]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 11/14/2013 23:59'!
addNotation: aName
	self notations add: aName! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 03:45'!
addParameterEntity: aName replacement: aReplacement from: aDocumentOrEntityUri
	self parameterEntities
		at: aName
		ifAbsentPut: [
			DTDParameterEntity
				name: aName
				replacement: aReplacement
				uri: aDocumentOrEntityUri]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 03:49'!
addUnparsedEntity: aName ndata: aNotation
	self unparsedEntities
		at: aName
		ifAbsentPut: [aNotation]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 05:14'!
addValidatorForAttributeIn: anElement named: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	| attributeValidatorsForElement validator |

	attributeValidatorsForElement :=
		self attributeValidators
			at: anElement
			ifAbsentPut: [Dictionary new].
	(attributeValidatorsForElement includesKey: aName)
		ifTrue: [^ nil].

	validator :=
		(DTDAttributeDefinitionParser
			element:  anElement
			attribute: aName
			type: aType
			defaultPragma: aDefaultPragma
			defaultValue: aDefaultValue) parse.

	validator
		validateAndAddTo: attributeValidatorsForElement
		usingDeclarationsFrom: self.! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 12/21/2013 03:14'!
addValidatorForElement: anElement contentModel: aContentModel
	self elementValidators
		at: anElement
		ifAbsentPut: [
			((DTDContentModelParser parserForContentModel: aContentModel)
				element: anElement
				contentModel: aContentModel) parse]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/19/2013 01:01'!
attributeValidators
	^ attributeValidators ifNil: [attributeValidators := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/18/2013 19:26'!
elementValidators
	^ elementValidators ifNil: [elementValidators := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/4/2014 03:48'!
externalSubsetURI
	^ externalSubsetUri ifNil: [externalSubsetUri := '']! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 20:51'!
generalEntities
	^ generalEntities ifNil: [generalEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 20:50'!
generalEntityAt: aName
	^ self generalEntities at: aName ifAbsent: [nil]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/18/2013 19:45'!
hasAttributeValidators
	^ attributeValidators notNil and: [attributeValidators notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/18/2013 19:45'!
hasElementValidators
	^ elementValidators notNil and: [elementValidators notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 1/4/2014 03:49'!
hasExternalSubset
	^ self isStandalone not and: [self externalSubsetURI notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/15/2013 00:00'!
includesNotation: aName
	^ self notations includes: aName! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/22/2013 09:11'!
isStandalone
	^ isStandalone ifNil: [isStandalone := false]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/22/2013 09:11'!
isStandalone: aBoolean
	isStandalone := aBoolean! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 23:59'!
notations
	^ notations ifNil: [notations := Set new]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 20:51'!
parameterEntities
	^ parameterEntities ifNil: [parameterEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 20:51'!
parameterEntityAt: aName
	^ self parameterEntities at: aName ifAbsent: [nil]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/19/2013 23:42'!
rootElement
	^ rootElement ifNil: ['']! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/4/2014 03:48'!
rootElement: aRootElement externalSubsetURI: aUri
	rootElement := aRootElement.
	externalSubsetUri := aUri.! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 23:50'!
unparsedEntities
	^ unparsedEntities ifNil: [unparsedEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 23:55'!
unparsedEntityAt: aName
	^ self unparsedEntities at: aName ifAbsent: [nil]! !

Object subclass: #DTDExternalEntityResolver
	instanceVariableNames: 'httpClient fileClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalEntityResolver commentStamp: '<historical>' prior: 0!
This class resolves external entity references using HTTP and file clients.!

DTDExternalEntityResolver subclass: #DTDCachingEntityResolver
	instanceVariableNames: 'entityCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDCachingEntityResolver commentStamp: '<historical>' prior: 0!
This resolver can cache entities so they don't have to be retrieved again (like over HTTP). Use it like this:
	entityResolver := DTDCachingEntityResolver new.
	(XMLDOMParser on: xml)
		externalEntityResolver: entityResolver;
		parseDocument.!

!DTDCachingEntityResolver methodsFor: 'clearing' stamp: 'monty 12/18/2013 22:22'!
clearEntityCache
	self entityCache removeAll! !

!DTDCachingEntityResolver methodsFor: 'accessing' stamp: 'monty 12/18/2013 22:23'!
entityCache
	^ entityCache ifNil: [entityCache := Dictionary new]! !

!DTDCachingEntityResolver methodsFor: 'resolving' stamp: 'monty 12/18/2013 22:23'!
resolveEntity: aUri
	^ (self entityCache
		at: aUri
		ifAbsentPut: [(super resolveEntity: aUri) contents]) readStream! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 12/27/2013 06:29'!
clientForURI: aUri
	^ self clientForURIs: (Array with: aUri)! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 12/27/2013 06:29'!
clientForURIs: aUriColleciton
	^ (aUriColleciton anySatisfy: [:each |
		self httpClient beginsWithSupportedURLProtocol: each])
		ifTrue: [self httpClient]
		ifFalse: [self fileClient]! !

!DTDExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 12/13/2013 05:02'!
defaultDocumentURI
	^ self fileClient
		pathToFile: 'default'
		inDirectory: self fileClient currentWorkingDirectory! !

!DTDExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 12/27/2013 06:30'!
entityURIFromRelative: aRelativeUri andBase: aBaseUri
	^ (self clientForURIs: (Array with: aBaseUri with: aRelativeUri))
		combineRelative: aRelativeUri
		withBase: aBaseUri! !

!DTDExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 11/18/2013 22:40'!
fileClient
	^ fileClient ifNil: [fileClient := XMLFileClient defaultImplementation new]! !

!DTDExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 11/18/2013 22:24'!
fileClient: aClient
	fileClient := aClient! !

!DTDExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 11/18/2013 22:40'!
httpClient
	^ httpClient ifNil: [httpClient := XMLHTTPClient defaultImplementation new]! !

!DTDExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 11/18/2013 22:22'!
httpClient: aClient
	httpClient := aClient! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 12/27/2013 06:30'!
resolveEntity: aUri
	^ (self clientForURI: aUri) get: aUri! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 11/19/2013 02:16'!
resolveEntity: aName uri: anEntityUri from: aSourceUri class: aClass
	| absoluteUri |

	absoluteUri :=
		self
			entityURIFromRelative: anEntityUri
			andBase: aSourceUri.
	^ aClass
		name: aName
		replacement: (self resolveEntity: absoluteUri) contents
		uri: absoluteUri.! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 11/19/2013 02:07'!
resolveGeneralEntity: aName uri: anEntityUri from: aSourceUri
	^ self
		resolveEntity: aName
		uri: anEntityUri
		from: aSourceUri
		class: DTDExternalGeneralEntity! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 11/19/2013 02:08'!
resolveParameterEntity: aName uri: anEntityUri from: aSourceUri
	^ self
		resolveEntity: aName
		uri: anEntityUri
		from: aSourceUri
		class: DTDExternalParameterEntity! !

Object subclass: #DTDParsedEntity
	instanceVariableNames: 'name replacement uri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDParsedEntity commentStamp: '<historical>' prior: 0!
This is a base class for internal and external parsed entities with replacements that can be inserted into an XML document.!

DTDParsedEntity subclass: #DTDGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDGeneralEntity commentStamp: 'monty 1/5/2014 00:54' prior: 0!
This class is for general parsed entities with replacements.
!

DTDGeneralEntity subclass: #DTDBypassedGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDBypassedGeneralEntity methodsFor: 'replacement' stamp: 'monty 12/7/2013 19:13'!
replacement
	^ self asEscapedReference! !

DTDGeneralEntity subclass: #DTDExternalGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalGeneralEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:57'!
isExternal
	^ true! !

!DTDGeneralEntity methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:03'!
escapedReferenceSymbol
	^ '&amp;'! !

!DTDGeneralEntity methodsFor: 'accessing' stamp: 'monty 11/12/2013 20:09'!
referenceSymbol
	^ $&! !

!DTDGeneralEntity methodsFor: 'replacement' stamp: 'monty 11/14/2013 20:22'!
replacementForContentContext
	^ self replacement! !

DTDParsedEntity subclass: #DTDParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDParameterEntity commentStamp: 'monty 1/5/2014 00:54' prior: 0!
This class is for paramter entities with replacements.!

DTDParameterEntity subclass: #DTDBypassedParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDBypassedParameterEntity methodsFor: 'replacement' stamp: 'monty 12/7/2013 19:14'!
replacement
	^ self asEscapedReference! !

!DTDBypassedParameterEntity methodsFor: 'replacement' stamp: 'monty 12/7/2013 19:14'!
replacementForDTDContext
	^ ''! !

DTDParameterEntity subclass: #DTDExternalParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalParameterEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:58'!
isExternal
	^ true! !

!DTDParameterEntity methodsFor: 'accessing' stamp: 'monty 12/7/2013 20:23'!
escapedReferenceSymbol
	^ '&#37;'! !

!DTDParameterEntity methodsFor: 'accessing' stamp: 'monty 11/12/2013 20:09'!
referenceSymbol
	^ $%! !

!DTDParameterEntity methodsFor: 'replacement' stamp: 'monty 12/7/2013 19:41'!
replacementForDTDContext
	^ (self replacement copyWithFirst: Character space) copyWith: Character space.! !

!DTDParsedEntity class methodsFor: 'instance creation' stamp: 'monty 11/16/2013 06:02'!
name: aString replacement: aValueString
	^ self new
		setName: aString
		replacement: aValueString
		uri: nil! !

!DTDParsedEntity class methodsFor: 'instance creation' stamp: 'monty 11/16/2013 02:55'!
name: aString replacement: aValueString uri: aUri
	^ self new
		setName: aString
		replacement: aValueString
		uri: aUri! !

!DTDParsedEntity class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:37'!
named: aString
	^ self
		name: aString
		replacement: ''! !

!DTDParsedEntity methodsFor: 'converting' stamp: 'monty 12/7/2013 19:01'!
asEscapedReference
	^ self asReferenceStartingWith: self escapedReferenceSymbol! !

!DTDParsedEntity methodsFor: 'converting' stamp: 'monty 12/7/2013 19:11'!
asReference
	^ self asReferenceStartingWith: self referenceSymbol asString! !

!DTDParsedEntity methodsFor: 'private' stamp: 'monty 12/9/2013 01:29'!
asReferenceStartingWith: aString
	^ (aString, self name) copyWith: $;! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:02'!
escapedReferenceSymbol
	self subclassResponsibility! !

!DTDParsedEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:56'!
isExternal
	^ false! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 11/13/2013 23:45'!
name
	^ name ifNil: [name := '']! !

!DTDParsedEntity methodsFor: 'printing' stamp: 'monty 11/20/2013 01:23'!
printOn: aStream
	self asReference printOn: aStream
	! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 11/12/2013 20:09'!
referenceSymbol
	self subclassResponsibility! !

!DTDParsedEntity methodsFor: 'replacement' stamp: 'monty 11/13/2013 19:15'!
replacement
	^ replacement ifNil: [replacement := '']! !

!DTDParsedEntity methodsFor: 'replacement' stamp: 'monty 12/29/2013 04:33'!
replacementForLiteralContext
	| aReplacement |

	aReplacement := (String new: self replacement size) writeStream.
	self replacement do: [:each |
		each ==  $"
			ifTrue: [aReplacement nextPutAll: '&#34;']
			ifFalse: [
				each == $'
					ifTrue: [aReplacement nextPutAll: '&#39;']
					ifFalse: [aReplacement nextPut: each]]].
	^ aReplacement contents.! !

!DTDParsedEntity methodsFor: 'initializing' stamp: 'monty 11/16/2013 02:55'!
setName: aName replacement: aReplacement uri: aUri
	name := aName.
	replacement := aReplacement.
	uri := aUri.! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 11/16/2013 02:55'!
uri
	^ uri ifNil: [uri := '']! !

Object subclass: #DTDValueListParser
	instanceVariableNames: 'valueListStream valueStream listValues'
	classVariableNames: 'ListValueDelimiters'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDValueListParser commentStamp: '<historical>' prior: 0!
This is an abstract parser of lists in the form of "(one | two | three)"!

DTDValueListParser subclass: #DTDEnumerationListParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDEnumerationListParser methodsFor: 'testing' stamp: 'monty 12/31/2013 02:08'!
isValidListValue: aValue
	^ XMLTokenizer isNmtoken: aValue! !

DTDValueListParser subclass: #DTDMixedContentListParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDMixedContentListParser methodsFor: 'testing' stamp: 'monty 12/31/2013 02:32'!
isValidListValue: aValue
	^ aValue = '#PCDATA' or: [XMLTokenizer isName: aValue]! !

!DTDMixedContentListParser methodsFor: 'parsing' stamp: 'monty 12/31/2013 02:46'!
parseAfterList
	(valueListStream atEnd
		or: [valueListStream next == $*
			and: [valueListStream atEnd]])
		ifFalse: [
			XMLValidationException signal:
				'Mixed content list can only be followed by "*"'].! !

DTDValueListParser subclass: #DTDNotationListParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDNotationListParser methodsFor: 'testing' stamp: 'monty 12/31/2013 02:08'!
isValidListValue: aValue
	^ XMLTokenizer isName: aValue! !

!DTDNotationListParser methodsFor: 'accessing' stamp: 'monty 1/4/2014 04:45'!
valueList: aString
	super valueList:
		(aString
			copyFrom: (aString indexOf: $()
			to: aString size)! !

!DTDValueListParser class methodsFor: 'class initialization' stamp: 'monty 12/31/2013 02:44'!
initialize
	"self initialize"

	super initialize.

	(ListValueDelimiters := CharacterSet new)
		add: $|;
		add: $);
		addAll: Character separators! !

!DTDValueListParser class methodsFor: 'instance creation' stamp: 'monty 12/9/2013 02:27'!
on: aString
	^ self new valueList: aString! !

!DTDValueListParser methodsFor: 'testing' stamp: 'monty 12/31/2013 02:08'!
isValidListValue: aValue
	self subclassResponsibility! !

!DTDValueListParser methodsFor: 'parsing' stamp: 'monty 12/30/2013 23:09'!
parse
	(valueListStream atEnd not
		and: [valueListStream next == $(])
		ifFalse: [XMLValidationException signal: 'Expected value list'].

	[valueListStream atEnd
		or: [valueListStream peek == $)]]
		whileFalse: [
			self parseListValue.
			(valueListStream atEnd not
				and: [valueListStream peek == $|])
				ifTrue: [valueListStream next]].
	self parseListTerminator.

	^ listValues.! !

!DTDValueListParser methodsFor: 'parsing' stamp: 'monty 12/31/2013 02:46'!
parseAfterList! !

!DTDValueListParser methodsFor: 'parsing' stamp: 'monty 12/31/2013 02:46'!
parseListTerminator
	(valueListStream atEnd not
		and: [valueListStream next == $)])
		ifFalse: [XMLValidationException signal: 'Missing list terminator ")"'].

	self parseAfterList.! !

!DTDValueListParser methodsFor: 'parsing' stamp: 'monty 12/31/2013 02:43'!
parseListValue
	| value oldSize |

	valueStream reset.
	valueListStream skipSeparators.
	[valueListStream atEnd
		or: [ListValueDelimiters includes: valueListStream peek]]
		whileFalse: [valueStream nextPut: valueListStream next].
	valueListStream skipSeparators.

	(value := valueStream contents)
		ifEmpty: [XMLValidationException signal: 'Empty list value'].
	(self isValidListValue: value)
		ifFalse: [XMLValidationException signal: 'Invalid value "', value, '" in list value'].

	oldSize := listValues size.
	listValues add: value.
	(listValues size > oldSize)
		ifFalse: [XMLValidationException signal: 'Duplicate "', value, '" in list'].! !

!DTDValueListParser methodsFor: 'accessing' stamp: 'monty 12/29/2013 04:07'!
valueList: aString
	valueListStream := ReadStream on: aString.
	valueStream := WriteStream on: (String new: 16).
	listValues := Set new.! !

Object subclass: #SAXHandler
	instanceVariableNames: 'input documentUri configuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXHandler commentStamp: 'monty 12/28/2013 22:29' prior: 0!
This class is an XML parser that uses event handling. The acronym "SAX" refers to Java's "Simple API for XML," on which SAXHandler's API is based. To use this class, create a subclass and override handlers under the "content" and "lexical" categories as needed. By default, namespace support, validation, and external entity resolution are enabled.

To instantiate a SAXHandler, send any of the #on: or #parse- messages to the class.

The class-side #parse- messages take some input source and parse it immediately, returning the result of #parseDocument:
	SAXHandlerSubclass parse: xml.
	SAXHandlerSubclass parseURL: aUrl. 
	SAXHandlerSubclass parse: xml usingNamespaces: false

The #on: messages create new parsers on the given input string, stream, URL, or file and return it; the instance can then be further configured and ultimately sent #parseDocument to parse the given input:
	(SAXHandlerSubclass on: xml)
		isValidating: false;
		resolvesExternalEntities: false;
		parseDocument.

There is also #parseDocumentWhile: to stop parsing before the end.
	(SAXHandlerSubclass on: xml)
		parseDocumentWhile: [self shouldKeepParsing].

To parse incrementally, send #parser to an instance to get the underlying XMLParser object and send it #parseToken repeatedly:
	(handler := SAXHandlerSubclass on: xml)
		isValidating: false; 
		preservesUndeclaredEntityReferences: true.
	parser := handler parser.

	"Only parse the first 10 tokens:"
	10 timesRepeat: [parser parseToken].!

SAXHandler subclass: #HTMLHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTML'!

!HTMLHandler commentStamp: '<historical>' prior: 0!
This is class is a SAXHandler for HTML.!

!HTMLHandler methodsFor: 'private' stamp: 'monty 12/6/2013 08:35'!
driverClass
	^ HTMLDriver! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/27/2010 14:06'!
on: aStringOrStream
	^ self new input: aStringOrStream! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 11/15/2013 21:13'!
on: aStringOrStream documentURI: aUri
	^ self new
		input: aStringOrStream
		documentURI: aUri! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 11/16/2013 03:17'!
onFileNamed: aFileName
	^ self
		on: (XMLFileClient defaultImplementation get: aFileName)
		documentURI: aFileName! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 11/16/2013 03:26'!
onURL: aUrlString
	^ self
		on: (XMLHTTPClient defaultImplementation get: aUrlString)
		documentURI: aUrlString! !

!SAXHandler class methodsFor: 'parsing' stamp: 'JAAyer 11/10/2010 05:21'!
parse: aStringOrStream
	^ (self on: aStringOrStream) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'JAAyer 11/10/2010 05:30'!
parse: aStringOrStream usingNamespaces: aBoolean
	^ (self on: aStringOrStream)
		usesNamespaces: aBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 11/15/2013 23:33'!
parse: aStringOrStream usingNamespaces: aNamespaceBoolean validation: aValidationBoolean
	^ (self on: aStringOrStream)
		usesNamespaces: aNamespaceBoolean;
		isValidating: aValidationBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 11/16/2013 07:30'!
parse: aStringOrStream usingNamespaces: aNamespaceBoolean validation: aValidationBoolean externalEntities: anEntityBoolean
	^ (self on: aStringOrStream)
		usesNamespaces: aNamespaceBoolean;
		isValidating: aValidationBoolean;
		resolvesExternalEntities: anEntityBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 11/15/2013 23:34'!
parseDocumentFrom: aStringOrStream
	self deprecated: 'use #parse: instead'.

	^ self parse: aStringOrStream.! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 11/15/2013 23:34'!
parseDocumentFrom: aStringOrStream useNamespaces: aBoolean
	self deprecated: 'use #parse:usingNamespaces: instead'.

	^ self parse: aStringOrStream usingNamespaces: aBoolean.! !

!SAXHandler class methodsFor: 'parsing' stamp: 'JAAyer 11/13/2010 20:43'!
parseFileNamed: aFileName
	^ (self onFileNamed: aFileName) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 11/15/2013 23:35'!
parseURL: aUrl
	^ (self onURL: aUrl) parseDocument! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 12/30/2013 23:57'!
attributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue! !

!SAXHandler methodsFor: 'content' stamp: 'mir 1/8/2002 18:27'!
characters: aString
	"This call corresponds to the Java SAX call
	characters(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:23'!
comment: aCommentString
	"This call corresponds to the Java SAX ext call
	comment(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:37'!
configuration
	^ configuration ifNil: [configuration := self configurationClass new]! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:13'!
configuration: aConfiguration
	configuration := aConfiguration! !

!SAXHandler methodsFor: 'private' stamp: 'JAAyer 11/23/2010 15:35'!
configurationClass
	^ XMLConfiguration! !

!SAXHandler methodsFor: 'private' stamp: 'monty 12/27/2013 03:36'!
currentScope: aNamespaceScope! !

!SAXHandler methodsFor: 'private' stamp: 'monty 12/9/2013 06:31'!
defaultDocumentURI
	^ (input respondsTo: #path)
		ifTrue: [input path]
		ifFalse: [self externalEntityResolver defaultDocumentURI].! !

!SAXHandler methodsFor: 'accessing' stamp: 'monty 11/15/2013 21:14'!
documentURI
	^ documentUri ifNil: [documentUri := self defaultDocumentURI]! !

!SAXHandler methodsFor: 'private' stamp: 'monty 11/22/2013 18:46'!
driverClass
	^ SAXDriver! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/13/2013 23:54'!
elementDeclaration: aName contentModel: aContentModel
! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 2/21/2010 15:13'!
endCData
	"This call corresponds to the Java SAX ext call
	endCData()."! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/12/2013 18:01'!
endDTD
! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/27/2010 06:50'!
endDocument
	"This call corresponds to the Java SAX call
	endDocument()."! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:23'!
endElement: aQualifiedName
! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 16:50'!
endElement: aQualifiedName prefix: aPrefix uri: aUri
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self endElement: aQualifiedName! !

!SAXHandler methodsFor: 'content' stamp: 'monty 11/14/2013 00:13'!
endElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self endElement: aQualifiedName prefix: aPrefix uri:  aUri! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:23'!
endEntity: anEntityName
	"This call corresponds to the Java SAX ext call
	endEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'content' stamp: 'JAA 3/22/2011 11:32'!
endPrefixMapping: aPrefix
	"This call corresonds to the Java SAX call
	endPrefixMapping(java.lang.String prefix)."! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/8/2013 02:33'!
externalEntityResolver
	^ self configuration externalEntityResolver! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 01:55'!
externalEntityResolver: anExternalEntityResolver
	"This can be used to set a different DTDExternalEntityResolver"

	self configuration externalEntityResolver: anExternalEntityResolver! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/14/2013 04:01'!
generalEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/14/2013 04:02'!
generalEntityDeclaration: aName replacement: aReplacement! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:25'!
ignorableWhitespace: aString
	"This call corresonds to the Java SAX call
	ignorableWhitespace(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 14:05'!
input
	^ input! !

!SAXHandler methodsFor: 'private' stamp: 'JAAyer 11/27/2010 14:05'!
input: aStringOrStream
	input := aStringOrStream.! !

!SAXHandler methodsFor: 'private' stamp: 'monty 11/15/2013 21:13'!
input: aStringOrStream documentURI: aUri
	input := aStringOrStream.
	documentUri := aUri.! !

!SAXHandler methodsFor: 'configuration' stamp: 'JAAyer 11/23/2010 14:20'!
isValidating
	^ self configuration isValidating! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:00'!
isValidating: aBoolean
	"If true and document contstraints are present (in a DTD),
	the document will be checked against them (default).

	Enabling disables preservesUndeclaredEntityReferences.
	Disabling disables requiresDocumentConstraints."

	self configuration isValidating: aBoolean! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/13/2013 23:55'!
notationDeclaration: aName publicID: aPublicId systemID: aSytemId
! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/14/2013 04:05'!
parameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/14/2013 04:05'!
parameterEntityDeclaration: aName replacement: aReplacement! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/19/2013 00:37'!
parseDocument
	self parser parseDocument.

	^ self parsingResult.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/19/2013 07:17'!
parseDocumentWhile: aBlock
	self parser parseDocumentWhile: aBlock.

	^ self parsingResult.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 11/22/2013 18:47'!
parser
	self configuration parserHandlerClass: self class.

	^ XMLParser
		driver: (self driverClass saxHandler: self)
		on: input.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/19/2013 00:37'!
parsingResult
	^ self! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/7/2013 19:18'!
preservesUndeclaredEntityReferences
	^ self configuration preservesUndeclaredEntityReferences! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:01'!
preservesUndeclaredEntityReferences: aBoolean
	"If true, references to undeclared entities in content are
	preserved (not default).
	Disables validation"

	self configuration preservesUndeclaredEntityReferences: aBoolean! !

!SAXHandler methodsFor: 'content' stamp: 'JAA 3/22/2011 11:31'!
processingInstruction: aTarget data: aDataString
	"This call corresonds to the Java SAX call
	processingInstruction(java.lang.String target, java.lang.String data)."! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/21/2013 20:07'!
requiresDocumentConstraints
	^ self configuration requiresDocumentConstraints! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:23'!
requiresDocumentConstraints: aBoolean
	"If true, constraints on elements and attributes (in a DTD)
	will be required (not default).
	Enables validation."

	self configuration requiresDocumentConstraints: aBoolean! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 11/16/2013 07:33'!
resolvesExternalEntities
	^ self configuration resolvesExternalEntities! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:01'!
resolvesExternalEntities: aBoolean
	"When true, external DTDs and parsed entities will be
	fetched (default)."

	self configuration resolvesExternalEntities: aBoolean! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:22'!
skippedEntity: anEntityName
	"This call corresonds to the Java SAX call
	skippedEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'lexical' stamp: 'mir 8/11/2000 18:53'!
startCData
	"This call corresponds to the Java SAX ext call
	startCData()."! !

!SAXHandler methodsFor: 'declaration' stamp: 'JAAyer 11/10/2010 19:21'!
startDTD: aRootElementName publicID: aPublicId systemID: aSystemId
	"This call corresponds to the Java SAX ext call
	startDTD(java.lang.String name, java.lang.String publicId, java.lang.String systemId)."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:45'!
startDocument
	"This call corresonds to the Java SAX call
	startDocument()."! !

!SAXHandler methodsFor: 'content' stamp: 'monty 11/14/2013 00:13'!
startElement: aQualifiedName attributes: aDictionary
! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 16:47'!
startElement: aQualifiedName prefix: aPrefix uri: aUri attributes: aDictionary
	self startElement: aQualifiedName attributes: aDictionary! !

!SAXHandler methodsFor: 'content' stamp: 'monty 11/14/2013 00:14'!
startElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName attributes: aDictionary
	"This call corresonds to the Java SAX call
	startElement(java.lang.String namespaceURI, java.lang.String localName,
		java.lang.String qName, Attributes atts)."

	self
		startElement: aQualifiedName
		prefix: aPrefix
		uri: aUri
		attributes: aDictionary! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:22'!
startEntity: anEntityName
	"This call corresponds to the Java SAX ext call
	startEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:22'!
startPrefixMapping: aPrefix uri: aUri
	"This call corresonds to the Java SAX call
	startPrefixMapping(java.lang.String prefix, java.lang.String uri)."! !

!SAXHandler methodsFor: 'configuration' stamp: 'JAAyer 11/23/2010 14:20'!
usesNamespaces
	^ self configuration usesNamespaces! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:03'!
usesNamespaces: aBoolean
	"Namespace checking when true (default)"

	self configuration usesNamespaces: aBoolean! !

!SAXHandler methodsFor: 'content' stamp: 'monty 11/14/2013 00:14'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
! !

SAXHandler subclass: #XMLDOMParser
	instanceVariableNames: 'document nodeStack isInCDataSection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMParser commentStamp: 'monty 12/31/2013 00:36' prior: 0!
This class is an XML parser that produces a tree of nodes representing the parsed document's structure. The root of this tree will typically be a document node, and it will have a single child element node as its root. The root element can have any number of additional elements as its children, as well as non-element child nodes. Those elements can themselves have an arbitrary number of children, and so on. To see what type of node objects a node tree created by this parser will contain and for how to manipulate them browse the XML-Parser-DOM category.

By default, CDATA sections are treated as regular string nodes and comments are ignored entirely. To suppress this behavior, enable #preservesCDataNodes: or #preservesCommentNodes: before parsing. 

Lastly. you can control what node classes the DOM parser uses to create node objects by injecting a custom node factory with #nodeFactory: prior to parsing. See XMLNodeFactory for more information.!

XMLDOMParser subclass: #HTMLParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTML'!

!HTMLParser commentStamp: '<historical>' prior: 0!
This class is a DOM parser that turns potentially invalid and malformed HTML into a DOM tree. All tag and attribute names are converted to lowercase and markup is unrecognized in <script> and <style> sections.

(When printing the resulting DOM, use canonicallyPrintOn: and canonicallyPrinted.)!

!HTMLParser methodsFor: 'private' stamp: 'monty 11/22/2013 18:46'!
driverClass
	^ HTMLDriver! !

!HTMLParser methodsFor: 'content' stamp: 'monty 11/30/2013 08:43'!
endElement: anElementName prefix: aPrefix uri: aUri localName: aLocalName
	(self topNode isElement and: [self topNode isNamed: anElementName])
		ifTrue: [self popNode]
! !

!HTMLParser methodsFor: 'private' stamp: 'monty 11/30/2013 18:19'!
findUniqueElementFor: anElementName
	^ (#('html' 'head' 'body') includes: anElementName)
		ifTrue: [self document findElementNamed: anElementName]
		ifFalse: [nil]! !

!HTMLParser methodsFor: 'content' stamp: 'monty 11/30/2013 18:21'!
startElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName attributes: aDictionary
	| element |

	(element := self findUniqueElementFor: aQualifiedName)
		ifNotNil: [
			self pushNode: element.
			^ self].

	super
		startElement: aQualifiedName
		prefix: aPrefix
		uri: aUri
		localName: aLocalName
		attributes: aDictionary.! !

!XMLDOMParser class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 04:09'!
on: aStringOrStream withRootNode: aRootNode
	^ (self on: aStringOrStream)
		pushNode: aRootNode;
		yourself! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/31/2013 00:02'!
attributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self topNode addNode:
		(self nodeFactory attributeDeclarationClass
			element: anElement
			attribute: aName
			type: aType
			defaultPragma: aDefaultPragma
			defaultValue: aDefaultValue)! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/23/2010 14:19'!
characters: aString
	(self preservesCDataNodes and: [self isInCDataSection])
		ifTrue: [self topNode addNode: (self nodeFactory cdataClass string: aString)]
		ifFalse: [self topNode addContent: (self nodeFactory stringClass string: aString)]
! !

!XMLDOMParser methodsFor: 'lexical' stamp: 'JAAyer 11/23/2010 14:43'!
comment: aComment
	self preservesCommentNodes
		ifTrue: [self topNode addNode: (self nodeFactory commentClass string: aComment)]! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/23/2010 14:37'!
configurationClass
	^ XMLDOMConfiguration! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 12/27/2013 03:43'!
currentScope: aNamespaceScope
	self topNode setNamespaceScope: aNamespaceScope! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 3/9/2010 01:38'!
document
	^ document! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 12:31'!
document: aDocument
	(document := aDocument) configuration: self configuration! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:14'!
elementDeclaration: aName contentModel: aContentModel
	self topNode addNode:
		(self nodeFactory elementDeclarationClass
			element: aName
			contentModel: aContentModel)! !

!XMLDOMParser methodsFor: 'lexical' stamp: 'JAAyer 8/9/2010 17:27'!
endCData
	isInCDataSection := false! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 11/12/2013 23:00'!
endDTD
	self popNode! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/27/2010 06:53'!
endDocument
	self popNode! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAA 3/21/2011 19:45'!
endElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName
	self popNode! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:01'!
generalEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation
	self topNode addNode:
		(self nodeFactory generalEntityDeclarationClass
			name: aName
			publicID: aPublicId
			systemID: aSystemId
			ndata: aNotation)! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:02'!
generalEntityDeclaration: aName replacement: aReplacement
	self topNode addNode:
		(self nodeFactory generalEntityDeclarationClass
			name: aName
			replacement: aReplacement)! !

!XMLDOMParser methodsFor: 'testing' stamp: 'JAAyer 8/9/2010 17:27'!
isInCDataSection
	^ isInCDataSection ifNil: [isInCDataSection := false]! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
nodeFactory
	^ self configuration nodeFactory! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
nodeFactory: aNodeFactory
	self configuration nodeFactory: aNodeFactory! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 12/9/2013 01:37'!
nodeListClass
	^ self configuration nodeListClass! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 12/9/2013 01:37'!
nodeListClass: aClass
	self configuration nodeListClass: aClass! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 12/6/2013 06:48'!
nodeStack
	^ nodeStack ifNil: [nodeStack := OrderedCollection new: 15]! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:03'!
notationDeclaration: aName publicID: aPublicId systemID: aSystemId
	self topNode addNode:
		(self nodeFactory notationDeclarationClass
			name: aName
			publicID: aPublicId
			systemID: aSystemId)! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:02'!
parameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId
	self topNode addNode:
		(self nodeFactory parameterEntityDeclarationClass
			name: aName
			publicID: aPublicId
			systemID: aSystemId)! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:02'!
parameterEntityDeclaration: aName replacement: aReplacement
	self topNode addNode:
		(self nodeFactory parameterEntityDeclarationClass
			name: aName
			replacement: aReplacement)! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'monty 12/27/2013 08:21'!
parser
	| parser |

	parser := super parser.
	(self nodeStack notEmpty and: [self topNode isElement])
		ifTrue: [
			self
				configuration: self topNode configuration;
				document: self topNode document.
			parser driver
				rootElement: self topNode name;
				currentScope: self topNode namespaceScope].

	^ parser.! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'monty 12/19/2013 00:38'!
parsingResult
	^ self document! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/16/2010 09:31'!
popNode
	^ self nodeStack removeLast! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCDataNodes
	^ self configuration preservesCDataNodes! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:17'!
preservesCDataNodes: aBoolean
	self configuration preservesCDataNodes: aBoolean! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCommentNodes
	^ self configuration preservesCommentNodes! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCommentNodes: aBoolean
	self configuration preservesCommentNodes: aBoolean! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAA 3/22/2011 11:31'!
processingInstruction: aTarget data: aDataString
	self topNode addNode: (self nodeFactory piClass target: aTarget data: aDataString)! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 12/9/2013 04:21'!
pushElement: anElement
	self nodeStack ifNotEmpty: [self topNode addNode: anElement].
	self pushNode: anElement.! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/28/2010 13:25'!
pushNode: aNode
	^ self nodeStack addLast: aNode
! !

!XMLDOMParser methodsFor: 'lexical' stamp: 'JAAyer 8/9/2010 17:27'!
startCData
	isInCDataSection := true! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 11/12/2013 23:02'!
startDTD: aRootElementName publicID: aPublicId systemID: aSystemId
	| doctypeDeclaration |

	doctypeDeclaration := 
		self nodeFactory doctypeDeclarationClass
			root: aRootElementName
			publicID: aPublicId
			systemID: aSystemId.
	self pushNode: doctypeDeclaration.
	self document doctypeDeclaration: doctypeDeclaration.! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/28/2010 13:25'!
startDocument
	self topNode ifNotNil: [^ self].

	self document: self nodeFactory documentClass new.
	self pushNode: self document ! !

!XMLDOMParser methodsFor: 'content' stamp: 'monty 12/27/2013 03:41'!
startElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName attributes: aDictionary
	self pushElement:
		((self nodeFactory
			classForElement: aQualifiedName
			prefix: aPrefix
			uri: aUri
			localName: aLocalName)
				qualifiedName: aQualifiedName
				prefix: aPrefix
				localName: aLocalName
				attributes: aDictionary)! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/16/2010 09:29'!
topNode
	^ self nodeStack isEmpty
		ifTrue: [nil]
		ifFalse: [self nodeStack last]! !

!XMLDOMParser methodsFor: 'content' stamp: 'monty 11/19/2013 05:37'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
	self document ifNotNil: [
		self document
			version: aVersionFloat;
			encoding: anEncoding;
			isStandalone: aBoolean]! !

Object subclass: #XMLAttributeDefaultValidator
	instanceVariableNames: 'defaultValue attribute'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeDefaultValidator commentStamp: '<historical>' prior: 0!
This is a base class for attribute default value validators that implement the constraints (like #REQUIRED) from the default declaration.!

!XMLAttributeDefaultValidator class methodsFor: 'instance creation' stamp: 'monty 12/10/2013 06:16'!
attribute: anAttribute defaultValue: aDefaultValue
	^ self new setAttribute: anAttribute defaultValue: aDefaultValue! !

!XMLAttributeDefaultValidator methodsFor: 'initialization' stamp: 'monty 12/10/2013 06:17'!
setAttribute: anAttribute defaultValue: aDefaultValue
	attribute := anAttribute.
	defaultValue := aDefaultValue.! !

!XMLAttributeDefaultValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 20:02'!
validateAttributeDefaultIn: anAttributeDictionary
	self subclassResponsibility! !

XMLAttributeDefaultValidator subclass: #XMLDefaultValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDefaultValueAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:23'!
validateAttributeDefaultIn: anAttributeDictionary
	((anAttributeDictionary at: attribute ifAbsent: ['']) isEmpty 
		and: [defaultValue notEmpty])
		ifTrue: [anAttributeDictionary at: attribute put: defaultValue]! !

XMLDefaultValueAttributeValidator subclass: #XMLFixedValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLFixedValueAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:23'!
validateAttributeDefaultIn: anAttributeDictionary
	anAttributeDictionary at: attribute put: defaultValue! !

XMLAttributeDefaultValidator subclass: #XMLImpliedValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLImpliedValueAttributeValidator methodsFor: 'validation' stamp: 'monty 12/27/2013 15:36'!
validateAttributeDefaultIn: anAttributeDictionary! !

XMLAttributeDefaultValidator subclass: #XMLRequiredValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLRequiredValueAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:23'!
validateAttributeDefaultIn: anAttributeDictionary
	(anAttributeDictionary includesKey: attribute)
		ifFalse: [XMLValidationException signal: 'Required attribute "', attribute, '" is missing']! !

Object subclass: #XMLAttributeValidator
	instanceVariableNames: 'element attribute defaultValueValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeValidator commentStamp: '<historical>' prior: 0!
This is a base class for attribute validators.!

!XMLAttributeValidator class methodsFor: 'instance creation' stamp: 'monty 12/10/2013 06:15'!
element: anElement attribute: anAttribute defaultValueValidator: aDefaultValueValidator
	^ self new
		setElement: anElement
		attribute: anAttribute
		defaultValueValidator: aDefaultValueValidator! !

!XMLAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/10/2013 06:17'!
attribute
	^ attribute! !

!XMLAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/10/2013 06:17'!
element
	^ element! !

!XMLAttributeValidator methodsFor: 'normalization' stamp: 'monty 12/30/2013 22:41'!
furtherNormalizeAttributeValueIn: anAttributeDictionary
	| readStream writeStream |

	readStream := (anAttributeDictionary at: attribute) readStream.
	writeStream := (String new: readStream size) writeStream.
	readStream skipSeparators.
	[readStream atEnd]
		whileFalse: [
			readStream peek isSeparator
				ifTrue: [
					readStream skipSeparators.
					readStream atEnd
						ifFalse: [writeStream space]]
				ifFalse: [writeStream nextPut: readStream next]].
	anAttributeDictionary
		at: attribute
		put: writeStream contents.! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:57'!
hasIDRefs
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:57'!
hasIDs
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 1/4/2014 05:02'!
hasNotationValues
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 21:59'!
isEntityValidator
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/10/2013 06:06'!
isIDValidator
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/9/2013 02:24'!
isValueListValidator
	^ false! !

!XMLAttributeValidator methodsFor: 'initialization' stamp: 'monty 12/10/2013 06:15'!
setElement: anElement attribute: anAttribute defaultValueValidator: aDefaultValueValidator
	element := anElement.
	attribute := anAttribute.
	defaultValueValidator := aDefaultValueValidator.! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/11/2013 02:14'!
shouldFurtherNormalizeAttributeValue
	^ true! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 1/4/2014 22:49'!
validateAndAddTo: anAttributeValidatorDictionary usingDeclarationsFrom: aDeclarationDelegate
	anAttributeValidatorDictionary
		at: attribute
		put: self! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:23'!
validateAttributeDefaultIn: anAttributeDictionary
	defaultValueValidator validateAttributeDefaultIn: anAttributeDictionary.

	(self shouldFurtherNormalizeAttributeValue
		and: [anAttributeDictionary includesKey: attribute])
		ifTrue: [self furtherNormalizeAttributeValueIn: anAttributeDictionary].! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 20:03'!
validateAttributeValue: aValue
	self subclassResponsibility! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:23'!
validateAttributes: anAttributeDictionary
	self
		validateAttributeDefaultIn: anAttributeDictionary;
		validateAttributeValue: (anAttributeDictionary at: attribute ifAbsent: [^ self])
						! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 12/10/2013 06:14'!
validateID: anId
	(XMLTokenizer isName: anId)
		ifFalse: [XMLValidationException signal: 'Invalid ID ', anId, ' in attribute "', attribute, '"']! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 12/10/2013 06:14'!
validateName: aName
	(XMLTokenizer isNmtoken: aName)
		ifFalse: [XMLValidationException signal: 'Invalid name "', aName, '" in attribute "', attribute, '"']! !

XMLAttributeValidator subclass: #XMLCDataAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLCDataAttributeValidator methodsFor: 'testing' stamp: 'monty 12/11/2013 02:14'!
shouldFurtherNormalizeAttributeValue
	^ false! !

!XMLCDataAttributeValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 20:04'!
validateAttributeValue: aValue
	! !

XMLAttributeValidator subclass: #XMLEntityAttributeValidator
	instanceVariableNames: 'unparsedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

XMLEntityAttributeValidator subclass: #XMLEntitiesAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEntitiesAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:12'!
validateAttributeValue: aValue
	Character space
		split: aValue
		do: [:each | self validateEntityReference: each]! !

!XMLEntityAttributeValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 22:00'!
isEntityValidator
	^ true! !

!XMLEntityAttributeValidator methodsFor: 'accessing' stamp: 'monty 11/19/2013 18:54'!
unparsedEntities
	^ unparsedEntities ifNil: [unparsedEntities := Dictionary new]! !

!XMLEntityAttributeValidator methodsFor: 'validation' stamp: 'monty 1/4/2014 22:50'!
validateAndAddTo: anAttributeValidatorDictionary usingDeclarationsFrom: aDeclarationDelegate
	unparsedEntities := aDeclarationDelegate unparsedEntities.
	super
		validateAndAddTo: anAttributeValidatorDictionary
		usingDeclarationsFrom: aDeclarationDelegate.! !

!XMLEntityAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:12'!
validateAttributeValue: aValue
	self validateEntityReference: aValue! !

!XMLEntityAttributeValidator methodsFor: 'validation' stamp: 'monty 12/10/2013 06:14'!
validateEntityReference: aName
	self validateName: aName.
	(self unparsedEntities includesKey: aName)
		ifFalse: [XMLValidationException signal: 'Unrecognized unpasred entity ', aName, ' in attribute "', attribute, '"']! !

XMLAttributeValidator subclass: #XMLIDAttributeValidator
	instanceVariableNames: 'ids'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:57'!
hasIDs
	^ self ids notEmpty! !

!XMLIDAttributeValidator methodsFor: 'accessing' stamp: 'monty 11/19/2013 08:18'!
ids
	^ ids ifNil: [ids := Set new]
	! !

!XMLIDAttributeValidator methodsFor: 'testing' stamp: 'monty 12/10/2013 06:07'!
isIDValidator
	^ true! !

!XMLIDAttributeValidator methodsFor: 'validation' stamp: 'monty 1/4/2014 22:50'!
validateAndAddTo: anAttributeValidatorDictionary usingDeclarationsFrom: aDeclarationDelegate
	anAttributeValidatorDictionary valuesDo: [:each |
		each isIDValidator
			ifTrue: [
				XMLValidationException signal:
					'Element <', element, '> has multiple ID attributes']].
	
	super
		validateAndAddTo: anAttributeValidatorDictionary
		usingDeclarationsFrom: aDeclarationDelegate! !

!XMLIDAttributeValidator methodsFor: 'validation' stamp: 'monty 12/18/2013 20:13'!
validateAttributeValue: aValue
	self validateID: aValue.
	(self ids includes: aValue)
		ifTrue: [
			XMLValidationException signal:
				'ID "', aValue, '" used more than once for <', element, '>'].

	self ids add: aValue.! !

XMLAttributeValidator subclass: #XMLIDRefAttributeValidator
	instanceVariableNames: 'idrefs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDRefAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:58'!
hasIDRefs
	^ self idRefs notEmpty! !

!XMLIDRefAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/29/2013 02:58'!
idRefs
	^ idrefs ifNil: [idrefs := Set new].! !

!XMLIDRefAttributeValidator methodsFor: 'validation' stamp: 'monty 12/29/2013 02:58'!
validateAttributeValue: aValue
	self validateID: aValue.
	self idRefs add: aValue.! !

XMLIDRefAttributeValidator subclass: #XMLIDRefsAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDRefsAttributeValidator methodsFor: 'validation' stamp: 'monty 12/29/2013 02:58'!
validateAttributeValue: aValue
	Character space
		split: aValue
		do: [:each |
			self validateID: each.
			self idRefs add: each].! !

XMLAttributeValidator subclass: #XMLNameAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNameAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:12'!
validateAttributeValue: aValue
	self validateName: aValue! !

XMLNameAttributeValidator subclass: #XMLNamesAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNamesAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:12'!
validateAttributeValue: aValue
	Character space
		split: aValue
		do: [:each | self validateName: each]! !

XMLAttributeValidator subclass: #XMLValueListAttributeValidator
	instanceVariableNames: 'allowedValues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

XMLValueListAttributeValidator subclass: #XMLNotationListAttributeValidator
	instanceVariableNames: 'notations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNotationListAttributeValidator methodsFor: 'testing' stamp: 'monty 1/4/2014 04:58'!
hasNotationValues
	^ true! !

!XMLNotationListAttributeValidator methodsFor: 'accessing' stamp: 'monty 1/4/2014 05:00'!
notations
	^ notations ifNil: [notations := Set new]! !

!XMLNotationListAttributeValidator methodsFor: 'validation' stamp: 'monty 1/4/2014 22:50'!
validateAndAddTo: anAttributeValidatorDictionary usingDeclarationsFrom: aDeclarationDelegate
	anAttributeValidatorDictionary valuesDo: [:each |
		each hasNotationValues
			ifTrue: [
				XMLValidationException signal:
					'Element <', element, '> has multiple NOTATION list attributes']].

	notations := aDeclarationDelegate notations.
	super
		validateAndAddTo: anAttributeValidatorDictionary
		usingDeclarationsFrom: aDeclarationDelegate.! !

!XMLNotationListAttributeValidator methodsFor: 'validation' stamp: 'monty 1/4/2014 08:16'!
validateAttributeValue: aValue
	super validateAttributeValue: aValue.

	(self notations includes: aValue)
		ifFalse: [
			XMLValidationException signal:
				'Undeclared NOTATION "', aValue, '" in attribute "', attribute, '"'].! !

!XMLValueListAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/9/2013 02:17'!
allowedValues
	^ allowedValues ifNil: [allowedValues := Set new]! !

!XMLValueListAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/9/2013 02:16'!
allowedValues: aValueCollection
	allowedValues := aValueCollection.! !

!XMLValueListAttributeValidator methodsFor: 'testing' stamp: 'monty 12/9/2013 02:24'!
isValueListValidator
	^ true! !

!XMLValueListAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:12'!
validateAttributeValue: aValue
	(self allowedValues includes: aValue)
		ifFalse: [XMLValidationException signal: 'Invalid value "', aValue, '" for attribute "', attribute, '"']! !

Object subclass: #XMLConfiguration
	instanceVariableNames: 'isValidating usesNamespaces requiresDtdDeclarations resolvesExternalEntities preservesUndeclaredEntityReferences externalEntityResolver parserHandlerClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLConfiguration commentStamp: '<historical>' prior: 0!
This class stores the shared settings and components for a parser.!

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/18/2013 22:27'!
externalEntityResolver
	^ externalEntityResolver ifNil: [externalEntityResolver := DTDExternalEntityResolver new]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/18/2013 22:18'!
externalEntityResolver: anExternalEntityResolver
	externalEntityResolver := anExternalEntityResolver! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:09'!
isValidating
	^ isValidating ifNil: [isValidating := true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/29/2013 02:42'!
isValidating: aBoolean
	(isValidating := aBoolean)
		ifTrue: [self preservesUndeclaredEntityReferences: false]
		ifFalse: [self requiresDocumentConstraints: false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/16/2013 03:10'!
parserHandlerClass
	^ parserHandlerClass ifNil: [parserHandlerClass := SAXHandler]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/16/2013 03:10'!
parserHandlerClass: aClass
	parserHandlerClass := aClass! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:16'!
preservesUndeclaredEntityReferences
	^ preservesUndeclaredEntityReferences ifNil: [preservesUndeclaredEntityReferences := false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/29/2013 02:42'!
preservesUndeclaredEntityReferences: aBoolean
	(preservesUndeclaredEntityReferences := aBoolean)
		ifTrue: [self isValidating: false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/21/2013 20:07'!
requiresDocumentConstraints
	^ requiresDtdDeclarations ifNil: [requiresDtdDeclarations := false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/21/2013 20:07'!
requiresDocumentConstraints: aBoolean
	(requiresDtdDeclarations := aBoolean)
		ifTrue: [self isValidating: true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/13/2013 22:18'!
resolvesExternalEntities
	^ resolvesExternalEntities ifNil: [resolvesExternalEntities := true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/13/2013 22:18'!
resolvesExternalEntities: aBoolean
	resolvesExternalEntities := aBoolean! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:07'!
usesNamespaces
	^ usesNamespaces ifNil: [usesNamespaces := true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:06'!
usesNamespaces: aBoolean
	usesNamespaces := aBoolean! !

XMLConfiguration subclass: #XMLDOMConfiguration
	instanceVariableNames: 'preservesCDataNodes preservesCommentNodes nodeFactory nodeListClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMConfiguration commentStamp: '<historical>' prior: 0!
This class stores additional settings and components relevant to the DOM parser and DOM nodes.!

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:10'!
nodeFactory
	^ nodeFactory ifNil: [nodeFactory := XMLNodeFactory new]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:10'!
nodeFactory: aFactory
	nodeFactory := aFactory! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:22'!
nodeListClass
	^ nodeListClass ifNil: [nodeListClass := XMLNodeList]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:22'!
nodeListClass: aClass
	nodeListClass := aClass! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 11/16/2013 03:10'!
parserHandlerClass
	^ parserHandlerClass ifNil: [parserHandlerClass := XMLDOMParser]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCDataNodes
	^ preservesCDataNodes ifNil: [preservesCDataNodes := false]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCDataNodes: aBoolean
	preservesCDataNodes := aBoolean! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCommentNodes
	^ preservesCommentNodes ifNil: [preservesCommentNodes := false]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCommentNodes: aBoolean
	preservesCommentNodes := aBoolean! !

Object subclass: #XMLDocumentValidator
	instanceVariableNames: 'elementValidatorStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDocumentValidator commentStamp: '<historical>' prior: 0!
This is a base class for XML document validators.!

XMLDocumentValidator subclass: #DTDDocumentValidator
	instanceVariableNames: 'dtd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDocumentValidator commentStamp: '<historical>' prior: 0!
This class validates documents using a DTD.!

!DTDDocumentValidator class methodsFor: 'instance creation' stamp: 'monty 11/18/2013 05:13'!
dtd: aDtd
	^ self new dtd: aDtd! !

!DTDDocumentValidator methodsFor: 'enumerating' stamp: 'monty 12/9/2013 02:29'!
allAttributeValidatorsDo: aBlock
	^ self dtd attributeValidators valuesDo: [:each | each valuesDo: aBlock]! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 12/29/2013 05:04'!
attributeValidatorsFor: anElement
	^ dtd attributeValidators
		at: anElement
		ifAbsent: [nil]! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 05:13'!
dtd
	^ dtd! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 05:13'!
dtd: aDtd
	dtd := aDtd! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/20/2013 01:20'!
elementValidatorFor: aName
	^ (dtd elementValidators
		at: aName
		ifAbsent: [nil]) copy! !

!DTDDocumentValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 01:03'!
hasAttributeValidators
	^ dtd hasAttributeValidators! !

!DTDDocumentValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 01:03'!
hasElementValidators
	^ dtd hasElementValidators! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 21:07'!
unvalidatedAttributes: anAttributeCollection
	XMLValidationException signal:
		'Undeclared attribute(s) ', (', ' join: anAttributeCollection)! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:29'!
unvalidatedElement: aName
	XMLValidationException signal: 'Undeclared element <', aName, '>'! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 02:38'!
validateEndDocument
	self hasAttributeValidators
		ifTrue: [self validateIDsAndReferences]	! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 12/29/2013 02:59'!
validateIDsAndReferences
	| ids references |

	ids := Set new.
	references := Set new.
	self allAttributeValidatorsDo: [:each |
		each hasIDs
			ifTrue: [ids addAll: each ids]
			ifFalse: [
				each hasIDRefs
					ifTrue: [references addAll: each idRefs]]].

	references do: [:each |
		(ids includes: each)
			ifFalse: [XMLValidationException signal: 'Invalid ID reference ', each]].! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:15'!
validateRootElement: anElementName
	(dtd rootElement notEmpty
		and: [dtd rootElement ~= anElementName])
		ifTrue: [
			XMLValidationException signal:
				'Element <', anElementName, '> cannot be root']! !

!XMLDocumentValidator methodsFor: 'enumerating' stamp: 'monty 11/19/2013 18:26'!
allAttributeValidatorsDo: aBlock
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/19/2013 01:04'!
attributeValidatorsFor: anElement
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 03:47'!
currentElementValidator
	^ self elementValidatorStack last! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 20:07'!
defaultElementValidatorFor: aName
	^ XMLAnyElementValidator element: aName! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 03:57'!
elementValidatorFor: aName
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 03:46'!
elementValidatorStack
	^ elementValidatorStack ifNil: [elementValidatorStack := OrderedCollection new]! !

!XMLDocumentValidator methodsFor: 'private' stamp: 'monty 11/28/2013 19:42'!
errorMissingContentFor: aName
	XMLValidationException signal: 'Element <', aName, '> terminated prematurely'! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 03:31'!
hasAttributeValidators
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 03:32'!
hasElementValidators
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:29'!
unvalidatedAttributes: anAttributeCollection! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:29'!
unvalidatedElement: aName! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/29/2013 05:05'!
validateAttributes: anAttributeDictionary with: aValidatorDictionary
	| unvalidatedAttributes |

	unvalidatedAttributes := anAttributeDictionary keys asSet.
	aValidatorDictionary
		ifNotNil: [
			aValidatorDictionary keysAndValuesDo: [:attribute :validator |
				validator validateAttributes: anAttributeDictionary.
				unvalidatedAttributes
					remove: attribute
				ifAbsent: []]].

	unvalidatedAttributes
		ifNotEmpty: [self unvalidatedAttributes: unvalidatedAttributes].! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 02:22'!
validateEndDocument
! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/7/2013 19:45'!
validateEndTag: aName
	self hasElementValidators
		ifTrue: [
			(self currentElementValidator element = aName)
				ifTrue: [
					self currentElementValidator atEnd
						ifFalse: [self errorMissingContentFor: aName]].
			self elementValidatorStack removeLast]! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 03:44'!
validatePCData: aString
	self hasElementValidators
		ifTrue: [self currentElementValidator validatePCData: aString]! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:08'!
validateRootElement: anElementName! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 23:56'!
validateStartTag: aName
	self elementValidatorStack
		ifNotEmpty: [self currentElementValidator validateStartTag: aName].

	self elementValidatorStack addLast:
		((self elementValidatorFor: aName)
			ifNil: [
				self
					unvalidatedElement: aName;
					defaultElementValidatorFor: aName]).! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/29/2013 05:05'!
validateStartTag: aName attributes: anAttributeDictionary
	(self hasElementValidators or: [self hasAttributeValidators])
		ifFalse: [^ self].

	self validateStartTag: aName.
	self
		validateAttributes: anAttributeDictionary
		with: (self attributeValidatorsFor: aName).! !

Object subclass: #XMLDriverTokenizer
	instanceVariableNames: 'driver context streamReader streamWriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

XMLDriverTokenizer class
	instanceVariableNames: 'predefinedEntities'!

!XMLDriverTokenizer commentStamp: '<historical>' prior: 0!
This is a base class for tokenizers. Be careful changing the code in this class or its subclases. It is heavily optimized.!

XMLDriverTokenizer class
	instanceVariableNames: 'predefinedEntities'!

XMLDriverTokenizer subclass: #HTMLTokenizer
	instanceVariableNames: 'nonHtmlCommentStarter nonHtmlElement'
	classVariableNames: 'HtmlNameDelimiters'
	poolDictionaries: ''
	category: 'XML-Parser-HTML'!

!HTMLTokenizer commentStamp: '<historical>' prior: 0!
This is the implementation of the tokenizer for the HTML driver.!

!HTMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/1/2013 13:25'!
compileHTMLEntityPageWithNamedEntities: anAssociationCollection
	"There are too many entities to represent literally in a single method,
	so they're put in a big string and parsed at initialization."
	| source |

	(source := String new writeStream)
		nextPutAll: 'htmlEntityPage';
		cr;
		tab;
		nextPut: $^;
		space;
		nextPut: $'.

	anAssociationCollection do: [:each |
		source
			cr;
			tab;
			nextPut: $[;
			nextPutAll: each key;
			nextPut: $=;
			nextPutAll: each value;
			nextPut: $]].
	source nextPut: $'.

	self class
		compile: source contents
		classified: 'class initialization' asSymbol.! !

!HTMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/7/2013 04:36'!
generateHTMLEntityPage
	"self generateHTMLEntityPage"
	| namedEntities json fields entityName codepoint |

	namedEntities := OrderedCollection new.
	json :=
		XMLHTTPClient defaultImplementation
			get: 'http://www.w3.org/TR/html5/entities.json'.
	[json atEnd]
		whileFalse: [
			fields := $: split: (json upToAny: String crlf).
			fields size > 1
				ifTrue: [
					entityName := fields first copyWithoutAll: '&;"{', Character separators.
					codepoint := (fields third copyAfter: $[) copyUpTo: $].
					namedEntities addLast: entityName -> codepoint]].

	self compileHTMLEntityPageWithNamedEntities: namedEntities.! !

!HTMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/1/2013 13:17'!
htmlEntityPage
	^ '
	[Aacute=193]
	[Aacute=193]
	[aacute=225]
	[aacute=225]
	[Abreve=258]
	[abreve=259]
	[ac=8766]
	[acd=8767]
	[acE=8766, 819]
	[Acirc=194]
	[Acirc=194]
	[acirc=226]
	[acirc=226]
	[acute=180]
	[acute=180]
	[Acy=1040]
	[acy=1072]
	[AElig=198]
	[AElig=198]
	[aelig=230]
	[aelig=230]
	[af=8289]
	[Afr=120068]
	[afr=120094]
	[Agrave=192]
	[Agrave=192]
	[agrave=224]
	[agrave=224]
	[alefsym=8501]
	[aleph=8501]
	[Alpha=913]
	[alpha=945]
	[Amacr=256]
	[amacr=257]
	[amalg=10815]
	[AMP=38]
	[AMP=38]
	[amp=38]
	[amp=38]
	[And=10835]
	[and=8743]
	[andand=10837]
	[andd=10844]
	[andslope=10840]
	[andv=10842]
	[ang=8736]
	[ange=10660]
	[angle=8736]
	[angmsd=8737]
	[angmsdaa=10664]
	[angmsdab=10665]
	[angmsdac=10666]
	[angmsdad=10667]
	[angmsdae=10668]
	[angmsdaf=10669]
	[angmsdag=10670]
	[angmsdah=10671]
	[angrt=8735]
	[angrtvb=8894]
	[angrtvbd=10653]
	[angsph=8738]
	[angst=197]
	[angzarr=9084]
	[Aogon=260]
	[aogon=261]
	[Aopf=120120]
	[aopf=120146]
	[ap=8776]
	[apacir=10863]
	[apE=10864]
	[ape=8778]
	[apid=8779]
	[apos=39]
	[ApplyFunction=8289]
	[approx=8776]
	[approxeq=8778]
	[Aring=197]
	[Aring=197]
	[aring=229]
	[aring=229]
	[Ascr=119964]
	[ascr=119990]
	[Assign=8788]
	[ast=42]
	[asymp=8776]
	[asympeq=8781]
	[Atilde=195]
	[Atilde=195]
	[atilde=227]
	[atilde=227]
	[Auml=196]
	[Auml=196]
	[auml=228]
	[auml=228]
	[awconint=8755]
	[awint=10769]
	[backcong=8780]
	[backepsilon=1014]
	[backprime=8245]
	[backsim=8765]
	[backsimeq=8909]
	[Backslash=8726]
	[Barv=10983]
	[barvee=8893]
	[Barwed=8966]
	[barwed=8965]
	[barwedge=8965]
	[bbrk=9141]
	[bbrktbrk=9142]
	[bcong=8780]
	[Bcy=1041]
	[bcy=1073]
	[bdquo=8222]
	[becaus=8757]
	[Because=8757]
	[because=8757]
	[bemptyv=10672]
	[bepsi=1014]
	[bernou=8492]
	[Bernoullis=8492]
	[Beta=914]
	[beta=946]
	[beth=8502]
	[between=8812]
	[Bfr=120069]
	[bfr=120095]
	[bigcap=8898]
	[bigcirc=9711]
	[bigcup=8899]
	[bigodot=10752]
	[bigoplus=10753]
	[bigotimes=10754]
	[bigsqcup=10758]
	[bigstar=9733]
	[bigtriangledown=9661]
	[bigtriangleup=9651]
	[biguplus=10756]
	[bigvee=8897]
	[bigwedge=8896]
	[bkarow=10509]
	[blacklozenge=10731]
	[blacksquare=9642]
	[blacktriangle=9652]
	[blacktriangledown=9662]
	[blacktriangleleft=9666]
	[blacktriangleright=9656]
	[blank=9251]
	[blk12=9618]
	[blk14=9617]
	[blk34=9619]
	[block=9608]
	[bne=61, 8421]
	[bnequiv=8801, 8421]
	[bNot=10989]
	[bnot=8976]
	[Bopf=120121]
	[bopf=120147]
	[bot=8869]
	[bottom=8869]
	[bowtie=8904]
	[boxbox=10697]
	[boxDL=9559]
	[boxDl=9558]
	[boxdL=9557]
	[boxdl=9488]
	[boxDR=9556]
	[boxDr=9555]
	[boxdR=9554]
	[boxdr=9484]
	[boxH=9552]
	[boxh=9472]
	[boxHD=9574]
	[boxHd=9572]
	[boxhD=9573]
	[boxhd=9516]
	[boxHU=9577]
	[boxHu=9575]
	[boxhU=9576]
	[boxhu=9524]
	[boxminus=8863]
	[boxplus=8862]
	[boxtimes=8864]
	[boxUL=9565]
	[boxUl=9564]
	[boxuL=9563]
	[boxul=9496]
	[boxUR=9562]
	[boxUr=9561]
	[boxuR=9560]
	[boxur=9492]
	[boxV=9553]
	[boxv=9474]
	[boxVH=9580]
	[boxVh=9579]
	[boxvH=9578]
	[boxvh=9532]
	[boxVL=9571]
	[boxVl=9570]
	[boxvL=9569]
	[boxvl=9508]
	[boxVR=9568]
	[boxVr=9567]
	[boxvR=9566]
	[boxvr=9500]
	[bprime=8245]
	[Breve=728]
	[breve=728]
	[brvbar=166]
	[brvbar=166]
	[Bscr=8492]
	[bscr=119991]
	[bsemi=8271]
	[bsim=8765]
	[bsime=8909]
	[bsol=92]
	[bsolb=10693]
	[bsolhsub=10184]
	[bull=8226]
	[bullet=8226]
	[bump=8782]
	[bumpE=10926]
	[bumpe=8783]
	[Bumpeq=8782]
	[bumpeq=8783]
	[Cacute=262]
	[cacute=263]
	[Cap=8914]
	[cap=8745]
	[capand=10820]
	[capbrcup=10825]
	[capcap=10827]
	[capcup=10823]
	[capdot=10816]
	[CapitalDifferentialD=8517]
	[caps=8745, 65024]
	[caret=8257]
	[caron=711]
	[Cayleys=8493]
	[ccaps=10829]
	[Ccaron=268]
	[ccaron=269]
	[Ccedil=199]
	[Ccedil=199]
	[ccedil=231]
	[ccedil=231]
	[Ccirc=264]
	[ccirc=265]
	[Cconint=8752]
	[ccups=10828]
	[ccupssm=10832]
	[Cdot=266]
	[cdot=267]
	[cedil=184]
	[cedil=184]
	[Cedilla=184]
	[cemptyv=10674]
	[cent=162]
	[cent=162]
	[CenterDot=183]
	[centerdot=183]
	[Cfr=8493]
	[cfr=120096]
	[CHcy=1063]
	[chcy=1095]
	[check=10003]
	[checkmark=10003]
	[Chi=935]
	[chi=967]
	[cir=9675]
	[circ=710]
	[circeq=8791]
	[circlearrowleft=8634]
	[circlearrowright=8635]
	[circledast=8859]
	[circledcirc=8858]
	[circleddash=8861]
	[CircleDot=8857]
	[circledR=174]
	[circledS=9416]
	[CircleMinus=8854]
	[CirclePlus=8853]
	[CircleTimes=8855]
	[cirE=10691]
	[cire=8791]
	[cirfnint=10768]
	[cirmid=10991]
	[cirscir=10690]
	[ClockwiseContourIntegral=8754]
	[CloseCurlyDoubleQuote=8221]
	[CloseCurlyQuote=8217]
	[clubs=9827]
	[clubsuit=9827]
	[Colon=8759]
	[colon=58]
	[Colone=10868]
	[colone=8788]
	[coloneq=8788]
	[comma=44]
	[commat=64]
	[comp=8705]
	[compfn=8728]
	[complement=8705]
	[complexes=8450]
	[cong=8773]
	[congdot=10861]
	[Congruent=8801]
	[Conint=8751]
	[conint=8750]
	[ContourIntegral=8750]
	[Copf=8450]
	[copf=120148]
	[coprod=8720]
	[Coproduct=8720]
	[COPY=169]
	[COPY=169]
	[copy=169]
	[copy=169]
	[copysr=8471]
	[CounterClockwiseContourIntegral=8755]
	[crarr=8629]
	[Cross=10799]
	[cross=10007]
	[Cscr=119966]
	[cscr=119992]
	[csub=10959]
	[csube=10961]
	[csup=10960]
	[csupe=10962]
	[ctdot=8943]
	[cudarrl=10552]
	[cudarrr=10549]
	[cuepr=8926]
	[cuesc=8927]
	[cularr=8630]
	[cularrp=10557]
	[Cup=8915]
	[cup=8746]
	[cupbrcap=10824]
	[CupCap=8781]
	[cupcap=10822]
	[cupcup=10826]
	[cupdot=8845]
	[cupor=10821]
	[cups=8746, 65024]
	[curarr=8631]
	[curarrm=10556]
	[curlyeqprec=8926]
	[curlyeqsucc=8927]
	[curlyvee=8910]
	[curlywedge=8911]
	[curren=164]
	[curren=164]
	[curvearrowleft=8630]
	[curvearrowright=8631]
	[cuvee=8910]
	[cuwed=8911]
	[cwconint=8754]
	[cwint=8753]
	[cylcty=9005]
	[Dagger=8225]
	[dagger=8224]
	[daleth=8504]
	[Darr=8609]
	[dArr=8659]
	[darr=8595]
	[dash=8208]
	[Dashv=10980]
	[dashv=8867]
	[dbkarow=10511]
	[dblac=733]
	[Dcaron=270]
	[dcaron=271]
	[Dcy=1044]
	[dcy=1076]
	[DD=8517]
	[dd=8518]
	[ddagger=8225]
	[ddarr=8650]
	[DDotrahd=10513]
	[ddotseq=10871]
	[deg=176]
	[deg=176]
	[Del=8711]
	[Delta=916]
	[delta=948]
	[demptyv=10673]
	[dfisht=10623]
	[Dfr=120071]
	[dfr=120097]
	[dHar=10597]
	[dharl=8643]
	[dharr=8642]
	[DiacriticalAcute=180]
	[DiacriticalDot=729]
	[DiacriticalDoubleAcute=733]
	[DiacriticalGrave=96]
	[DiacriticalTilde=732]
	[diam=8900]
	[Diamond=8900]
	[diamond=8900]
	[diamondsuit=9830]
	[diams=9830]
	[die=168]
	[DifferentialD=8518]
	[digamma=989]
	[disin=8946]
	[div=247]
	[divide=247]
	[divide=247]
	[divideontimes=8903]
	[divonx=8903]
	[DJcy=1026]
	[djcy=1106]
	[dlcorn=8990]
	[dlcrop=8973]
	[dollar=36]
	[Dopf=120123]
	[dopf=120149]
	[Dot=168]
	[dot=729]
	[DotDot=8412]
	[doteq=8784]
	[doteqdot=8785]
	[DotEqual=8784]
	[dotminus=8760]
	[dotplus=8724]
	[dotsquare=8865]
	[doublebarwedge=8966]
	[DoubleContourIntegral=8751]
	[DoubleDot=168]
	[DoubleDownArrow=8659]
	[DoubleLeftArrow=8656]
	[DoubleLeftRightArrow=8660]
	[DoubleLeftTee=10980]
	[DoubleLongLeftArrow=10232]
	[DoubleLongLeftRightArrow=10234]
	[DoubleLongRightArrow=10233]
	[DoubleRightArrow=8658]
	[DoubleRightTee=8872]
	[DoubleUpArrow=8657]
	[DoubleUpDownArrow=8661]
	[DoubleVerticalBar=8741]
	[DownArrow=8595]
	[Downarrow=8659]
	[downarrow=8595]
	[DownArrowBar=10515]
	[DownArrowUpArrow=8693]
	[DownBreve=785]
	[downdownarrows=8650]
	[downharpoonleft=8643]
	[downharpoonright=8642]
	[DownLeftRightVector=10576]
	[DownLeftTeeVector=10590]
	[DownLeftVector=8637]
	[DownLeftVectorBar=10582]
	[DownRightTeeVector=10591]
	[DownRightVector=8641]
	[DownRightVectorBar=10583]
	[DownTee=8868]
	[DownTeeArrow=8615]
	[drbkarow=10512]
	[drcorn=8991]
	[drcrop=8972]
	[Dscr=119967]
	[dscr=119993]
	[DScy=1029]
	[dscy=1109]
	[dsol=10742]
	[Dstrok=272]
	[dstrok=273]
	[dtdot=8945]
	[dtri=9663]
	[dtrif=9662]
	[duarr=8693]
	[duhar=10607]
	[dwangle=10662]
	[DZcy=1039]
	[dzcy=1119]
	[dzigrarr=10239]
	[Eacute=201]
	[Eacute=201]
	[eacute=233]
	[eacute=233]
	[easter=10862]
	[Ecaron=282]
	[ecaron=283]
	[ecir=8790]
	[Ecirc=202]
	[Ecirc=202]
	[ecirc=234]
	[ecirc=234]
	[ecolon=8789]
	[Ecy=1069]
	[ecy=1101]
	[eDDot=10871]
	[Edot=278]
	[eDot=8785]
	[edot=279]
	[ee=8519]
	[efDot=8786]
	[Efr=120072]
	[efr=120098]
	[eg=10906]
	[Egrave=200]
	[Egrave=200]
	[egrave=232]
	[egrave=232]
	[egs=10902]
	[egsdot=10904]
	[el=10905]
	[Element=8712]
	[elinters=9191]
	[ell=8467]
	[els=10901]
	[elsdot=10903]
	[Emacr=274]
	[emacr=275]
	[empty=8709]
	[emptyset=8709]
	[EmptySmallSquare=9723]
	[emptyv=8709]
	[EmptyVerySmallSquare=9643]
	[emsp=8195]
	[emsp13=8196]
	[emsp14=8197]
	[ENG=330]
	[eng=331]
	[ensp=8194]
	[Eogon=280]
	[eogon=281]
	[Eopf=120124]
	[eopf=120150]
	[epar=8917]
	[eparsl=10723]
	[eplus=10865]
	[epsi=949]
	[Epsilon=917]
	[epsilon=949]
	[epsiv=1013]
	[eqcirc=8790]
	[eqcolon=8789]
	[eqsim=8770]
	[eqslantgtr=10902]
	[eqslantless=10901]
	[Equal=10869]
	[equals=61]
	[EqualTilde=8770]
	[equest=8799]
	[Equilibrium=8652]
	[equiv=8801]
	[equivDD=10872]
	[eqvparsl=10725]
	[erarr=10609]
	[erDot=8787]
	[Escr=8496]
	[escr=8495]
	[esdot=8784]
	[Esim=10867]
	[esim=8770]
	[Eta=919]
	[eta=951]
	[ETH=208]
	[ETH=208]
	[eth=240]
	[eth=240]
	[Euml=203]
	[Euml=203]
	[euml=235]
	[euml=235]
	[euro=8364]
	[excl=33]
	[exist=8707]
	[Exists=8707]
	[expectation=8496]
	[ExponentialE=8519]
	[exponentiale=8519]
	[fallingdotseq=8786]
	[Fcy=1060]
	[fcy=1092]
	[female=9792]
	[ffilig=64259]
	[fflig=64256]
	[ffllig=64260]
	[Ffr=120073]
	[ffr=120099]
	[filig=64257]
	[FilledSmallSquare=9724]
	[FilledVerySmallSquare=9642]
	[fjlig=102, 106]
	[flat=9837]
	[fllig=64258]
	[fltns=9649]
	[fnof=402]
	[Fopf=120125]
	[fopf=120151]
	[ForAll=8704]
	[forall=8704]
	[fork=8916]
	[forkv=10969]
	[Fouriertrf=8497]
	[fpartint=10765]
	[frac12=189]
	[frac12=189]
	[frac13=8531]
	[frac14=188]
	[frac14=188]
	[frac15=8533]
	[frac16=8537]
	[frac18=8539]
	[frac23=8532]
	[frac25=8534]
	[frac34=190]
	[frac34=190]
	[frac35=8535]
	[frac38=8540]
	[frac45=8536]
	[frac56=8538]
	[frac58=8541]
	[frac78=8542]
	[frasl=8260]
	[frown=8994]
	[Fscr=8497]
	[fscr=119995]
	[gacute=501]
	[Gamma=915]
	[gamma=947]
	[Gammad=988]
	[gammad=989]
	[gap=10886]
	[Gbreve=286]
	[gbreve=287]
	[Gcedil=290]
	[Gcirc=284]
	[gcirc=285]
	[Gcy=1043]
	[gcy=1075]
	[Gdot=288]
	[gdot=289]
	[gE=8807]
	[ge=8805]
	[gEl=10892]
	[gel=8923]
	[geq=8805]
	[geqq=8807]
	[geqslant=10878]
	[ges=10878]
	[gescc=10921]
	[gesdot=10880]
	[gesdoto=10882]
	[gesdotol=10884]
	[gesl=8923, 65024]
	[gesles=10900]
	[Gfr=120074]
	[gfr=120100]
	[Gg=8921]
	[gg=8811]
	[ggg=8921]
	[gimel=8503]
	[GJcy=1027]
	[gjcy=1107]
	[gl=8823]
	[gla=10917]
	[glE=10898]
	[glj=10916]
	[gnap=10890]
	[gnapprox=10890]
	[gnE=8809]
	[gne=10888]
	[gneq=10888]
	[gneqq=8809]
	[gnsim=8935]
	[Gopf=120126]
	[gopf=120152]
	[grave=96]
	[GreaterEqual=8805]
	[GreaterEqualLess=8923]
	[GreaterFullEqual=8807]
	[GreaterGreater=10914]
	[GreaterLess=8823]
	[GreaterSlantEqual=10878]
	[GreaterTilde=8819]
	[Gscr=119970]
	[gscr=8458]
	[gsim=8819]
	[gsime=10894]
	[gsiml=10896]
	[GT=62]
	[GT=62]
	[Gt=8811]
	[gt=62]
	[gt=62]
	[gtcc=10919]
	[gtcir=10874]
	[gtdot=8919]
	[gtlPar=10645]
	[gtquest=10876]
	[gtrapprox=10886]
	[gtrarr=10616]
	[gtrdot=8919]
	[gtreqless=8923]
	[gtreqqless=10892]
	[gtrless=8823]
	[gtrsim=8819]
	[gvertneqq=8809, 65024]
	[gvnE=8809, 65024]
	[Hacek=711]
	[hairsp=8202]
	[half=189]
	[hamilt=8459]
	[HARDcy=1066]
	[hardcy=1098]
	[hArr=8660]
	[harr=8596]
	[harrcir=10568]
	[harrw=8621]
	[Hat=94]
	[hbar=8463]
	[Hcirc=292]
	[hcirc=293]
	[hearts=9829]
	[heartsuit=9829]
	[hellip=8230]
	[hercon=8889]
	[Hfr=8460]
	[hfr=120101]
	[HilbertSpace=8459]
	[hksearow=10533]
	[hkswarow=10534]
	[hoarr=8703]
	[homtht=8763]
	[hookleftarrow=8617]
	[hookrightarrow=8618]
	[Hopf=8461]
	[hopf=120153]
	[horbar=8213]
	[HorizontalLine=9472]
	[Hscr=8459]
	[hscr=119997]
	[hslash=8463]
	[Hstrok=294]
	[hstrok=295]
	[HumpDownHump=8782]
	[HumpEqual=8783]
	[hybull=8259]
	[hyphen=8208]
	[Iacute=205]
	[Iacute=205]
	[iacute=237]
	[iacute=237]
	[ic=8291]
	[Icirc=206]
	[Icirc=206]
	[icirc=238]
	[icirc=238]
	[Icy=1048]
	[icy=1080]
	[Idot=304]
	[IEcy=1045]
	[iecy=1077]
	[iexcl=161]
	[iexcl=161]
	[iff=8660]
	[Ifr=8465]
	[ifr=120102]
	[Igrave=204]
	[Igrave=204]
	[igrave=236]
	[igrave=236]
	[ii=8520]
	[iiiint=10764]
	[iiint=8749]
	[iinfin=10716]
	[iiota=8489]
	[IJlig=306]
	[ijlig=307]
	[Im=8465]
	[Imacr=298]
	[imacr=299]
	[image=8465]
	[ImaginaryI=8520]
	[imagline=8464]
	[imagpart=8465]
	[imath=305]
	[imof=8887]
	[imped=437]
	[Implies=8658]
	[in=8712]
	[incare=8453]
	[infin=8734]
	[infintie=10717]
	[inodot=305]
	[Int=8748]
	[int=8747]
	[intcal=8890]
	[integers=8484]
	[Integral=8747]
	[intercal=8890]
	[Intersection=8898]
	[intlarhk=10775]
	[intprod=10812]
	[InvisibleComma=8291]
	[InvisibleTimes=8290]
	[IOcy=1025]
	[iocy=1105]
	[Iogon=302]
	[iogon=303]
	[Iopf=120128]
	[iopf=120154]
	[Iota=921]
	[iota=953]
	[iprod=10812]
	[iquest=191]
	[iquest=191]
	[Iscr=8464]
	[iscr=119998]
	[isin=8712]
	[isindot=8949]
	[isinE=8953]
	[isins=8948]
	[isinsv=8947]
	[isinv=8712]
	[it=8290]
	[Itilde=296]
	[itilde=297]
	[Iukcy=1030]
	[iukcy=1110]
	[Iuml=207]
	[Iuml=207]
	[iuml=239]
	[iuml=239]
	[Jcirc=308]
	[jcirc=309]
	[Jcy=1049]
	[jcy=1081]
	[Jfr=120077]
	[jfr=120103]
	[jmath=567]
	[Jopf=120129]
	[jopf=120155]
	[Jscr=119973]
	[jscr=119999]
	[Jsercy=1032]
	[jsercy=1112]
	[Jukcy=1028]
	[jukcy=1108]
	[Kappa=922]
	[kappa=954]
	[kappav=1008]
	[Kcedil=310]
	[kcedil=311]
	[Kcy=1050]
	[kcy=1082]
	[Kfr=120078]
	[kfr=120104]
	[kgreen=312]
	[KHcy=1061]
	[khcy=1093]
	[KJcy=1036]
	[kjcy=1116]
	[Kopf=120130]
	[kopf=120156]
	[Kscr=119974]
	[kscr=120000]
	[lAarr=8666]
	[Lacute=313]
	[lacute=314]
	[laemptyv=10676]
	[lagran=8466]
	[Lambda=923]
	[lambda=955]
	[Lang=10218]
	[lang=10216]
	[langd=10641]
	[langle=10216]
	[lap=10885]
	[Laplacetrf=8466]
	[laquo=171]
	[laquo=171]
	[Larr=8606]
	[lArr=8656]
	[larr=8592]
	[larrb=8676]
	[larrbfs=10527]
	[larrfs=10525]
	[larrhk=8617]
	[larrlp=8619]
	[larrpl=10553]
	[larrsim=10611]
	[larrtl=8610]
	[lat=10923]
	[lAtail=10523]
	[latail=10521]
	[late=10925]
	[lates=10925, 65024]
	[lBarr=10510]
	[lbarr=10508]
	[lbbrk=10098]
	[lbrace=123]
	[lbrack=91]
	[lbrke=10635]
	[lbrksld=10639]
	[lbrkslu=10637]
	[Lcaron=317]
	[lcaron=318]
	[Lcedil=315]
	[lcedil=316]
	[lceil=8968]
	[lcub=123]
	[Lcy=1051]
	[lcy=1083]
	[ldca=10550]
	[ldquo=8220]
	[ldquor=8222]
	[ldrdhar=10599]
	[ldrushar=10571]
	[ldsh=8626]
	[lE=8806]
	[le=8804]
	[LeftAngleBracket=10216]
	[LeftArrow=8592]
	[Leftarrow=8656]
	[leftarrow=8592]
	[LeftArrowBar=8676]
	[LeftArrowRightArrow=8646]
	[leftarrowtail=8610]
	[LeftCeiling=8968]
	[LeftDoubleBracket=10214]
	[LeftDownTeeVector=10593]
	[LeftDownVector=8643]
	[LeftDownVectorBar=10585]
	[LeftFloor=8970]
	[leftharpoondown=8637]
	[leftharpoonup=8636]
	[leftleftarrows=8647]
	[LeftRightArrow=8596]
	[Leftrightarrow=8660]
	[leftrightarrow=8596]
	[leftrightarrows=8646]
	[leftrightharpoons=8651]
	[leftrightsquigarrow=8621]
	[LeftRightVector=10574]
	[LeftTee=8867]
	[LeftTeeArrow=8612]
	[LeftTeeVector=10586]
	[leftthreetimes=8907]
	[LeftTriangle=8882]
	[LeftTriangleBar=10703]
	[LeftTriangleEqual=8884]
	[LeftUpDownVector=10577]
	[LeftUpTeeVector=10592]
	[LeftUpVector=8639]
	[LeftUpVectorBar=10584]
	[LeftVector=8636]
	[LeftVectorBar=10578]
	[lEg=10891]
	[leg=8922]
	[leq=8804]
	[leqq=8806]
	[leqslant=10877]
	[les=10877]
	[lescc=10920]
	[lesdot=10879]
	[lesdoto=10881]
	[lesdotor=10883]
	[lesg=8922, 65024]
	[lesges=10899]
	[lessapprox=10885]
	[lessdot=8918]
	[lesseqgtr=8922]
	[lesseqqgtr=10891]
	[LessEqualGreater=8922]
	[LessFullEqual=8806]
	[LessGreater=8822]
	[lessgtr=8822]
	[LessLess=10913]
	[lesssim=8818]
	[LessSlantEqual=10877]
	[LessTilde=8818]
	[lfisht=10620]
	[lfloor=8970]
	[Lfr=120079]
	[lfr=120105]
	[lg=8822]
	[lgE=10897]
	[lHar=10594]
	[lhard=8637]
	[lharu=8636]
	[lharul=10602]
	[lhblk=9604]
	[LJcy=1033]
	[ljcy=1113]
	[Ll=8920]
	[ll=8810]
	[llarr=8647]
	[llcorner=8990]
	[Lleftarrow=8666]
	[llhard=10603]
	[lltri=9722]
	[Lmidot=319]
	[lmidot=320]
	[lmoust=9136]
	[lmoustache=9136]
	[lnap=10889]
	[lnapprox=10889]
	[lnE=8808]
	[lne=10887]
	[lneq=10887]
	[lneqq=8808]
	[lnsim=8934]
	[loang=10220]
	[loarr=8701]
	[lobrk=10214]
	[LongLeftArrow=10229]
	[Longleftarrow=10232]
	[longleftarrow=10229]
	[LongLeftRightArrow=10231]
	[Longleftrightarrow=10234]
	[longleftrightarrow=10231]
	[longmapsto=10236]
	[LongRightArrow=10230]
	[Longrightarrow=10233]
	[longrightarrow=10230]
	[looparrowleft=8619]
	[looparrowright=8620]
	[lopar=10629]
	[Lopf=120131]
	[lopf=120157]
	[loplus=10797]
	[lotimes=10804]
	[lowast=8727]
	[lowbar=95]
	[LowerLeftArrow=8601]
	[LowerRightArrow=8600]
	[loz=9674]
	[lozenge=9674]
	[lozf=10731]
	[lpar=40]
	[lparlt=10643]
	[lrarr=8646]
	[lrcorner=8991]
	[lrhar=8651]
	[lrhard=10605]
	[lrm=8206]
	[lrtri=8895]
	[lsaquo=8249]
	[Lscr=8466]
	[lscr=120001]
	[Lsh=8624]
	[lsh=8624]
	[lsim=8818]
	[lsime=10893]
	[lsimg=10895]
	[lsqb=91]
	[lsquo=8216]
	[lsquor=8218]
	[Lstrok=321]
	[lstrok=322]
	[LT=60]
	[LT=60]
	[Lt=8810]
	[lt=60]
	[lt=60]
	[ltcc=10918]
	[ltcir=10873]
	[ltdot=8918]
	[lthree=8907]
	[ltimes=8905]
	[ltlarr=10614]
	[ltquest=10875]
	[ltri=9667]
	[ltrie=8884]
	[ltrif=9666]
	[ltrPar=10646]
	[lurdshar=10570]
	[luruhar=10598]
	[lvertneqq=8808, 65024]
	[lvnE=8808, 65024]
	[macr=175]
	[macr=175]
	[male=9794]
	[malt=10016]
	[maltese=10016]
	[Map=10501]
	[map=8614]
	[mapsto=8614]
	[mapstodown=8615]
	[mapstoleft=8612]
	[mapstoup=8613]
	[marker=9646]
	[mcomma=10793]
	[Mcy=1052]
	[mcy=1084]
	[mdash=8212]
	[mDDot=8762]
	[measuredangle=8737]
	[MediumSpace=8287]
	[Mellintrf=8499]
	[Mfr=120080]
	[mfr=120106]
	[mho=8487]
	[micro=181]
	[micro=181]
	[mid=8739]
	[midast=42]
	[midcir=10992]
	[middot=183]
	[middot=183]
	[minus=8722]
	[minusb=8863]
	[minusd=8760]
	[minusdu=10794]
	[MinusPlus=8723]
	[mlcp=10971]
	[mldr=8230]
	[mnplus=8723]
	[models=8871]
	[Mopf=120132]
	[mopf=120158]
	[mp=8723]
	[Mscr=8499]
	[mscr=120002]
	[mstpos=8766]
	[Mu=924]
	[mu=956]
	[multimap=8888]
	[mumap=8888]
	[nabla=8711]
	[Nacute=323]
	[nacute=324]
	[nang=8736, 8402]
	[nap=8777]
	[napE=10864, 824]
	[napid=8779, 824]
	[napos=329]
	[napprox=8777]
	[natur=9838]
	[natural=9838]
	[naturals=8469]
	[nbsp=160]
	[nbsp=160]
	[nbump=8782, 824]
	[nbumpe=8783, 824]
	[ncap=10819]
	[Ncaron=327]
	[ncaron=328]
	[Ncedil=325]
	[ncedil=326]
	[ncong=8775]
	[ncongdot=10861, 824]
	[ncup=10818]
	[Ncy=1053]
	[ncy=1085]
	[ndash=8211]
	[ne=8800]
	[nearhk=10532]
	[neArr=8663]
	[nearr=8599]
	[nearrow=8599]
	[nedot=8784, 824]
	[NegativeMediumSpace=8203]
	[NegativeThickSpace=8203]
	[NegativeThinSpace=8203]
	[NegativeVeryThinSpace=8203]
	[nequiv=8802]
	[nesear=10536]
	[nesim=8770, 824]
	[NestedGreaterGreater=8811]
	[NestedLessLess=8810]
	[NewLine=10]
	[nexist=8708]
	[nexists=8708]
	[Nfr=120081]
	[nfr=120107]
	[ngE=8807, 824]
	[nge=8817]
	[ngeq=8817]
	[ngeqq=8807, 824]
	[ngeqslant=10878, 824]
	[nges=10878, 824]
	[nGg=8921, 824]
	[ngsim=8821]
	[nGt=8811, 8402]
	[ngt=8815]
	[ngtr=8815]
	[nGtv=8811, 824]
	[nhArr=8654]
	[nharr=8622]
	[nhpar=10994]
	[ni=8715]
	[nis=8956]
	[nisd=8954]
	[niv=8715]
	[NJcy=1034]
	[njcy=1114]
	[nlArr=8653]
	[nlarr=8602]
	[nldr=8229]
	[nlE=8806, 824]
	[nle=8816]
	[nLeftarrow=8653]
	[nleftarrow=8602]
	[nLeftrightarrow=8654]
	[nleftrightarrow=8622]
	[nleq=8816]
	[nleqq=8806, 824]
	[nleqslant=10877, 824]
	[nles=10877, 824]
	[nless=8814]
	[nLl=8920, 824]
	[nlsim=8820]
	[nLt=8810, 8402]
	[nlt=8814]
	[nltri=8938]
	[nltrie=8940]
	[nLtv=8810, 824]
	[nmid=8740]
	[NoBreak=8288]
	[NonBreakingSpace=160]
	[Nopf=8469]
	[nopf=120159]
	[Not=10988]
	[not=172]
	[not=172]
	[NotCongruent=8802]
	[NotCupCap=8813]
	[NotDoubleVerticalBar=8742]
	[NotElement=8713]
	[NotEqual=8800]
	[NotEqualTilde=8770, 824]
	[NotExists=8708]
	[NotGreater=8815]
	[NotGreaterEqual=8817]
	[NotGreaterFullEqual=8807, 824]
	[NotGreaterGreater=8811, 824]
	[NotGreaterLess=8825]
	[NotGreaterSlantEqual=10878, 824]
	[NotGreaterTilde=8821]
	[NotHumpDownHump=8782, 824]
	[NotHumpEqual=8783, 824]
	[notin=8713]
	[notindot=8949, 824]
	[notinE=8953, 824]
	[notinva=8713]
	[notinvb=8951]
	[notinvc=8950]
	[NotLeftTriangle=8938]
	[NotLeftTriangleBar=10703, 824]
	[NotLeftTriangleEqual=8940]
	[NotLess=8814]
	[NotLessEqual=8816]
	[NotLessGreater=8824]
	[NotLessLess=8810, 824]
	[NotLessSlantEqual=10877, 824]
	[NotLessTilde=8820]
	[NotNestedGreaterGreater=10914, 824]
	[NotNestedLessLess=10913, 824]
	[notni=8716]
	[notniva=8716]
	[notnivb=8958]
	[notnivc=8957]
	[NotPrecedes=8832]
	[NotPrecedesEqual=10927, 824]
	[NotPrecedesSlantEqual=8928]
	[NotReverseElement=8716]
	[NotRightTriangle=8939]
	[NotRightTriangleBar=10704, 824]
	[NotRightTriangleEqual=8941]
	[NotSquareSubset=8847, 824]
	[NotSquareSubsetEqual=8930]
	[NotSquareSuperset=8848, 824]
	[NotSquareSupersetEqual=8931]
	[NotSubset=8834, 8402]
	[NotSubsetEqual=8840]
	[NotSucceeds=8833]
	[NotSucceedsEqual=10928, 824]
	[NotSucceedsSlantEqual=8929]
	[NotSucceedsTilde=8831, 824]
	[NotSuperset=8835, 8402]
	[NotSupersetEqual=8841]
	[NotTilde=8769]
	[NotTildeEqual=8772]
	[NotTildeFullEqual=8775]
	[NotTildeTilde=8777]
	[NotVerticalBar=8740]
	[npar=8742]
	[nparallel=8742]
	[nparsl=11005, 8421]
	[npart=8706, 824]
	[npolint=10772]
	[npr=8832]
	[nprcue=8928]
	[npre=10927, 824]
	[nprec=8832]
	[npreceq=10927, 824]
	[nrArr=8655]
	[nrarr=8603]
	[nrarrc=10547, 824]
	[nrarrw=8605, 824]
	[nRightarrow=8655]
	[nrightarrow=8603]
	[nrtri=8939]
	[nrtrie=8941]
	[nsc=8833]
	[nsccue=8929]
	[nsce=10928, 824]
	[Nscr=119977]
	[nscr=120003]
	[nshortmid=8740]
	[nshortparallel=8742]
	[nsim=8769]
	[nsime=8772]
	[nsimeq=8772]
	[nsmid=8740]
	[nspar=8742]
	[nsqsube=8930]
	[nsqsupe=8931]
	[nsub=8836]
	[nsubE=10949, 824]
	[nsube=8840]
	[nsubset=8834, 8402]
	[nsubseteq=8840]
	[nsubseteqq=10949, 824]
	[nsucc=8833]
	[nsucceq=10928, 824]
	[nsup=8837]
	[nsupE=10950, 824]
	[nsupe=8841]
	[nsupset=8835, 8402]
	[nsupseteq=8841]
	[nsupseteqq=10950, 824]
	[ntgl=8825]
	[Ntilde=209]
	[Ntilde=209]
	[ntilde=241]
	[ntilde=241]
	[ntlg=8824]
	[ntriangleleft=8938]
	[ntrianglelefteq=8940]
	[ntriangleright=8939]
	[ntrianglerighteq=8941]
	[Nu=925]
	[nu=957]
	[num=35]
	[numero=8470]
	[numsp=8199]
	[nvap=8781, 8402]
	[nVDash=8879]
	[nVdash=8878]
	[nvDash=8877]
	[nvdash=8876]
	[nvge=8805, 8402]
	[nvgt=62, 8402]
	[nvHarr=10500]
	[nvinfin=10718]
	[nvlArr=10498]
	[nvle=8804, 8402]
	[nvlt=60, 8402]
	[nvltrie=8884, 8402]
	[nvrArr=10499]
	[nvrtrie=8885, 8402]
	[nvsim=8764, 8402]
	[nwarhk=10531]
	[nwArr=8662]
	[nwarr=8598]
	[nwarrow=8598]
	[nwnear=10535]
	[Oacute=211]
	[Oacute=211]
	[oacute=243]
	[oacute=243]
	[oast=8859]
	[ocir=8858]
	[Ocirc=212]
	[Ocirc=212]
	[ocirc=244]
	[ocirc=244]
	[Ocy=1054]
	[ocy=1086]
	[odash=8861]
	[Odblac=336]
	[odblac=337]
	[odiv=10808]
	[odot=8857]
	[odsold=10684]
	[OElig=338]
	[oelig=339]
	[ofcir=10687]
	[Ofr=120082]
	[ofr=120108]
	[ogon=731]
	[Ograve=210]
	[Ograve=210]
	[ograve=242]
	[ograve=242]
	[ogt=10689]
	[ohbar=10677]
	[ohm=937]
	[oint=8750]
	[olarr=8634]
	[olcir=10686]
	[olcross=10683]
	[oline=8254]
	[olt=10688]
	[Omacr=332]
	[omacr=333]
	[Omega=937]
	[omega=969]
	[Omicron=927]
	[omicron=959]
	[omid=10678]
	[ominus=8854]
	[Oopf=120134]
	[oopf=120160]
	[opar=10679]
	[OpenCurlyDoubleQuote=8220]
	[OpenCurlyQuote=8216]
	[operp=10681]
	[oplus=8853]
	[Or=10836]
	[or=8744]
	[orarr=8635]
	[ord=10845]
	[order=8500]
	[orderof=8500]
	[ordf=170]
	[ordf=170]
	[ordm=186]
	[ordm=186]
	[origof=8886]
	[oror=10838]
	[orslope=10839]
	[orv=10843]
	[oS=9416]
	[Oscr=119978]
	[oscr=8500]
	[Oslash=216]
	[Oslash=216]
	[oslash=248]
	[oslash=248]
	[osol=8856]
	[Otilde=213]
	[Otilde=213]
	[otilde=245]
	[otilde=245]
	[Otimes=10807]
	[otimes=8855]
	[otimesas=10806]
	[Ouml=214]
	[Ouml=214]
	[ouml=246]
	[ouml=246]
	[ovbar=9021]
	[OverBar=8254]
	[OverBrace=9182]
	[OverBracket=9140]
	[OverParenthesis=9180]
	[par=8741]
	[para=182]
	[para=182]
	[parallel=8741]
	[parsim=10995]
	[parsl=11005]
	[part=8706]
	[PartialD=8706]
	[Pcy=1055]
	[pcy=1087]
	[percnt=37]
	[period=46]
	[permil=8240]
	[perp=8869]
	[pertenk=8241]
	[Pfr=120083]
	[pfr=120109]
	[Phi=934]
	[phi=966]
	[phiv=981]
	[phmmat=8499]
	[phone=9742]
	[Pi=928]
	[pi=960]
	[pitchfork=8916]
	[piv=982]
	[planck=8463]
	[planckh=8462]
	[plankv=8463]
	[plus=43]
	[plusacir=10787]
	[plusb=8862]
	[pluscir=10786]
	[plusdo=8724]
	[plusdu=10789]
	[pluse=10866]
	[PlusMinus=177]
	[plusmn=177]
	[plusmn=177]
	[plussim=10790]
	[plustwo=10791]
	[pm=177]
	[Poincareplane=8460]
	[pointint=10773]
	[Popf=8473]
	[popf=120161]
	[pound=163]
	[pound=163]
	[Pr=10939]
	[pr=8826]
	[prap=10935]
	[prcue=8828]
	[prE=10931]
	[pre=10927]
	[prec=8826]
	[precapprox=10935]
	[preccurlyeq=8828]
	[Precedes=8826]
	[PrecedesEqual=10927]
	[PrecedesSlantEqual=8828]
	[PrecedesTilde=8830]
	[preceq=10927]
	[precnapprox=10937]
	[precneqq=10933]
	[precnsim=8936]
	[precsim=8830]
	[Prime=8243]
	[prime=8242]
	[primes=8473]
	[prnap=10937]
	[prnE=10933]
	[prnsim=8936]
	[prod=8719]
	[Product=8719]
	[profalar=9006]
	[profline=8978]
	[profsurf=8979]
	[prop=8733]
	[Proportion=8759]
	[Proportional=8733]
	[propto=8733]
	[prsim=8830]
	[prurel=8880]
	[Pscr=119979]
	[pscr=120005]
	[Psi=936]
	[psi=968]
	[puncsp=8200]
	[Qfr=120084]
	[qfr=120110]
	[qint=10764]
	[Qopf=8474]
	[qopf=120162]
	[qprime=8279]
	[Qscr=119980]
	[qscr=120006]
	[quaternions=8461]
	[quatint=10774]
	[quest=63]
	[questeq=8799]
	[QUOT=34]
	[QUOT=34]
	[quot=34]
	[quot=34]
	[rAarr=8667]
	[race=8765, 817]
	[Racute=340]
	[racute=341]
	[radic=8730]
	[raemptyv=10675]
	[Rang=10219]
	[rang=10217]
	[rangd=10642]
	[range=10661]
	[rangle=10217]
	[raquo=187]
	[raquo=187]
	[Rarr=8608]
	[rArr=8658]
	[rarr=8594]
	[rarrap=10613]
	[rarrb=8677]
	[rarrbfs=10528]
	[rarrc=10547]
	[rarrfs=10526]
	[rarrhk=8618]
	[rarrlp=8620]
	[rarrpl=10565]
	[rarrsim=10612]
	[Rarrtl=10518]
	[rarrtl=8611]
	[rarrw=8605]
	[rAtail=10524]
	[ratail=10522]
	[ratio=8758]
	[rationals=8474]
	[RBarr=10512]
	[rBarr=10511]
	[rbarr=10509]
	[rbbrk=10099]
	[rbrace=125]
	[rbrack=93]
	[rbrke=10636]
	[rbrksld=10638]
	[rbrkslu=10640]
	[Rcaron=344]
	[rcaron=345]
	[Rcedil=342]
	[rcedil=343]
	[rceil=8969]
	[rcub=125]
	[Rcy=1056]
	[rcy=1088]
	[rdca=10551]
	[rdldhar=10601]
	[rdquo=8221]
	[rdquor=8221]
	[rdsh=8627]
	[Re=8476]
	[real=8476]
	[realine=8475]
	[realpart=8476]
	[reals=8477]
	[rect=9645]
	[REG=174]
	[REG=174]
	[reg=174]
	[reg=174]
	[ReverseElement=8715]
	[ReverseEquilibrium=8651]
	[ReverseUpEquilibrium=10607]
	[rfisht=10621]
	[rfloor=8971]
	[Rfr=8476]
	[rfr=120111]
	[rHar=10596]
	[rhard=8641]
	[rharu=8640]
	[rharul=10604]
	[Rho=929]
	[rho=961]
	[rhov=1009]
	[RightAngleBracket=10217]
	[RightArrow=8594]
	[Rightarrow=8658]
	[rightarrow=8594]
	[RightArrowBar=8677]
	[RightArrowLeftArrow=8644]
	[rightarrowtail=8611]
	[RightCeiling=8969]
	[RightDoubleBracket=10215]
	[RightDownTeeVector=10589]
	[RightDownVector=8642]
	[RightDownVectorBar=10581]
	[RightFloor=8971]
	[rightharpoondown=8641]
	[rightharpoonup=8640]
	[rightleftarrows=8644]
	[rightleftharpoons=8652]
	[rightrightarrows=8649]
	[rightsquigarrow=8605]
	[RightTee=8866]
	[RightTeeArrow=8614]
	[RightTeeVector=10587]
	[rightthreetimes=8908]
	[RightTriangle=8883]
	[RightTriangleBar=10704]
	[RightTriangleEqual=8885]
	[RightUpDownVector=10575]
	[RightUpTeeVector=10588]
	[RightUpVector=8638]
	[RightUpVectorBar=10580]
	[RightVector=8640]
	[RightVectorBar=10579]
	[ring=730]
	[risingdotseq=8787]
	[rlarr=8644]
	[rlhar=8652]
	[rlm=8207]
	[rmoust=9137]
	[rmoustache=9137]
	[rnmid=10990]
	[roang=10221]
	[roarr=8702]
	[robrk=10215]
	[ropar=10630]
	[Ropf=8477]
	[ropf=120163]
	[roplus=10798]
	[rotimes=10805]
	[RoundImplies=10608]
	[rpar=41]
	[rpargt=10644]
	[rppolint=10770]
	[rrarr=8649]
	[Rrightarrow=8667]
	[rsaquo=8250]
	[Rscr=8475]
	[rscr=120007]
	[Rsh=8625]
	[rsh=8625]
	[rsqb=93]
	[rsquo=8217]
	[rsquor=8217]
	[rthree=8908]
	[rtimes=8906]
	[rtri=9657]
	[rtrie=8885]
	[rtrif=9656]
	[rtriltri=10702]
	[RuleDelayed=10740]
	[ruluhar=10600]
	[rx=8478]
	[Sacute=346]
	[sacute=347]
	[sbquo=8218]
	[Sc=10940]
	[sc=8827]
	[scap=10936]
	[Scaron=352]
	[scaron=353]
	[sccue=8829]
	[scE=10932]
	[sce=10928]
	[Scedil=350]
	[scedil=351]
	[Scirc=348]
	[scirc=349]
	[scnap=10938]
	[scnE=10934]
	[scnsim=8937]
	[scpolint=10771]
	[scsim=8831]
	[Scy=1057]
	[scy=1089]
	[sdot=8901]
	[sdotb=8865]
	[sdote=10854]
	[searhk=10533]
	[seArr=8664]
	[searr=8600]
	[searrow=8600]
	[sect=167]
	[sect=167]
	[semi=59]
	[seswar=10537]
	[setminus=8726]
	[setmn=8726]
	[sext=10038]
	[Sfr=120086]
	[sfr=120112]
	[sfrown=8994]
	[sharp=9839]
	[SHCHcy=1065]
	[shchcy=1097]
	[SHcy=1064]
	[shcy=1096]
	[ShortDownArrow=8595]
	[ShortLeftArrow=8592]
	[shortmid=8739]
	[shortparallel=8741]
	[ShortRightArrow=8594]
	[ShortUpArrow=8593]
	[shy=173]
	[shy=173]
	[Sigma=931]
	[sigma=963]
	[sigmaf=962]
	[sigmav=962]
	[sim=8764]
	[simdot=10858]
	[sime=8771]
	[simeq=8771]
	[simg=10910]
	[simgE=10912]
	[siml=10909]
	[simlE=10911]
	[simne=8774]
	[simplus=10788]
	[simrarr=10610]
	[slarr=8592]
	[SmallCircle=8728]
	[smallsetminus=8726]
	[smashp=10803]
	[smeparsl=10724]
	[smid=8739]
	[smile=8995]
	[smt=10922]
	[smte=10924]
	[smtes=10924, 65024]
	[SOFTcy=1068]
	[softcy=1100]
	[sol=47]
	[solb=10692]
	[solbar=9023]
	[Sopf=120138]
	[sopf=120164]
	[spades=9824]
	[spadesuit=9824]
	[spar=8741]
	[sqcap=8851]
	[sqcaps=8851, 65024]
	[sqcup=8852]
	[sqcups=8852, 65024]
	[Sqrt=8730]
	[sqsub=8847]
	[sqsube=8849]
	[sqsubset=8847]
	[sqsubseteq=8849]
	[sqsup=8848]
	[sqsupe=8850]
	[sqsupset=8848]
	[sqsupseteq=8850]
	[squ=9633]
	[Square=9633]
	[square=9633]
	[SquareIntersection=8851]
	[SquareSubset=8847]
	[SquareSubsetEqual=8849]
	[SquareSuperset=8848]
	[SquareSupersetEqual=8850]
	[SquareUnion=8852]
	[squarf=9642]
	[squf=9642]
	[srarr=8594]
	[Sscr=119982]
	[sscr=120008]
	[ssetmn=8726]
	[ssmile=8995]
	[sstarf=8902]
	[Star=8902]
	[star=9734]
	[starf=9733]
	[straightepsilon=1013]
	[straightphi=981]
	[strns=175]
	[Sub=8912]
	[sub=8834]
	[subdot=10941]
	[subE=10949]
	[sube=8838]
	[subedot=10947]
	[submult=10945]
	[subnE=10955]
	[subne=8842]
	[subplus=10943]
	[subrarr=10617]
	[Subset=8912]
	[subset=8834]
	[subseteq=8838]
	[subseteqq=10949]
	[SubsetEqual=8838]
	[subsetneq=8842]
	[subsetneqq=10955]
	[subsim=10951]
	[subsub=10965]
	[subsup=10963]
	[succ=8827]
	[succapprox=10936]
	[succcurlyeq=8829]
	[Succeeds=8827]
	[SucceedsEqual=10928]
	[SucceedsSlantEqual=8829]
	[SucceedsTilde=8831]
	[succeq=10928]
	[succnapprox=10938]
	[succneqq=10934]
	[succnsim=8937]
	[succsim=8831]
	[SuchThat=8715]
	[Sum=8721]
	[sum=8721]
	[sung=9834]
	[Sup=8913]
	[sup=8835]
	[sup1=185]
	[sup1=185]
	[sup2=178]
	[sup2=178]
	[sup3=179]
	[sup3=179]
	[supdot=10942]
	[supdsub=10968]
	[supE=10950]
	[supe=8839]
	[supedot=10948]
	[Superset=8835]
	[SupersetEqual=8839]
	[suphsol=10185]
	[suphsub=10967]
	[suplarr=10619]
	[supmult=10946]
	[supnE=10956]
	[supne=8843]
	[supplus=10944]
	[Supset=8913]
	[supset=8835]
	[supseteq=8839]
	[supseteqq=10950]
	[supsetneq=8843]
	[supsetneqq=10956]
	[supsim=10952]
	[supsub=10964]
	[supsup=10966]
	[swarhk=10534]
	[swArr=8665]
	[swarr=8601]
	[swarrow=8601]
	[swnwar=10538]
	[szlig=223]
	[szlig=223]
	[Tab=9]
	[target=8982]
	[Tau=932]
	[tau=964]
	[tbrk=9140]
	[Tcaron=356]
	[tcaron=357]
	[Tcedil=354]
	[tcedil=355]
	[Tcy=1058]
	[tcy=1090]
	[tdot=8411]
	[telrec=8981]
	[Tfr=120087]
	[tfr=120113]
	[there4=8756]
	[Therefore=8756]
	[therefore=8756]
	[Theta=920]
	[theta=952]
	[thetasym=977]
	[thetav=977]
	[thickapprox=8776]
	[thicksim=8764]
	[ThickSpace=8287, 8202]
	[thinsp=8201]
	[ThinSpace=8201]
	[thkap=8776]
	[thksim=8764]
	[THORN=222]
	[THORN=222]
	[thorn=254]
	[thorn=254]
	[Tilde=8764]
	[tilde=732]
	[TildeEqual=8771]
	[TildeFullEqual=8773]
	[TildeTilde=8776]
	[times=215]
	[times=215]
	[timesb=8864]
	[timesbar=10801]
	[timesd=10800]
	[tint=8749]
	[toea=10536]
	[top=8868]
	[topbot=9014]
	[topcir=10993]
	[Topf=120139]
	[topf=120165]
	[topfork=10970]
	[tosa=10537]
	[tprime=8244]
	[TRADE=8482]
	[trade=8482]
	[triangle=9653]
	[triangledown=9663]
	[triangleleft=9667]
	[trianglelefteq=8884]
	[triangleq=8796]
	[triangleright=9657]
	[trianglerighteq=8885]
	[tridot=9708]
	[trie=8796]
	[triminus=10810]
	[TripleDot=8411]
	[triplus=10809]
	[trisb=10701]
	[tritime=10811]
	[trpezium=9186]
	[Tscr=119983]
	[tscr=120009]
	[TScy=1062]
	[tscy=1094]
	[TSHcy=1035]
	[tshcy=1115]
	[Tstrok=358]
	[tstrok=359]
	[twixt=8812]
	[twoheadleftarrow=8606]
	[twoheadrightarrow=8608]
	[Uacute=218]
	[Uacute=218]
	[uacute=250]
	[uacute=250]
	[Uarr=8607]
	[uArr=8657]
	[uarr=8593]
	[Uarrocir=10569]
	[Ubrcy=1038]
	[ubrcy=1118]
	[Ubreve=364]
	[ubreve=365]
	[Ucirc=219]
	[Ucirc=219]
	[ucirc=251]
	[ucirc=251]
	[Ucy=1059]
	[ucy=1091]
	[udarr=8645]
	[Udblac=368]
	[udblac=369]
	[udhar=10606]
	[ufisht=10622]
	[Ufr=120088]
	[ufr=120114]
	[Ugrave=217]
	[Ugrave=217]
	[ugrave=249]
	[ugrave=249]
	[uHar=10595]
	[uharl=8639]
	[uharr=8638]
	[uhblk=9600]
	[ulcorn=8988]
	[ulcorner=8988]
	[ulcrop=8975]
	[ultri=9720]
	[Umacr=362]
	[umacr=363]
	[uml=168]
	[uml=168]
	[UnderBar=95]
	[UnderBrace=9183]
	[UnderBracket=9141]
	[UnderParenthesis=9181]
	[Union=8899]
	[UnionPlus=8846]
	[Uogon=370]
	[uogon=371]
	[Uopf=120140]
	[uopf=120166]
	[UpArrow=8593]
	[Uparrow=8657]
	[uparrow=8593]
	[UpArrowBar=10514]
	[UpArrowDownArrow=8645]
	[UpDownArrow=8597]
	[Updownarrow=8661]
	[updownarrow=8597]
	[UpEquilibrium=10606]
	[upharpoonleft=8639]
	[upharpoonright=8638]
	[uplus=8846]
	[UpperLeftArrow=8598]
	[UpperRightArrow=8599]
	[Upsi=978]
	[upsi=965]
	[upsih=978]
	[Upsilon=933]
	[upsilon=965]
	[UpTee=8869]
	[UpTeeArrow=8613]
	[upuparrows=8648]
	[urcorn=8989]
	[urcorner=8989]
	[urcrop=8974]
	[Uring=366]
	[uring=367]
	[urtri=9721]
	[Uscr=119984]
	[uscr=120010]
	[utdot=8944]
	[Utilde=360]
	[utilde=361]
	[utri=9653]
	[utrif=9652]
	[uuarr=8648]
	[Uuml=220]
	[Uuml=220]
	[uuml=252]
	[uuml=252]
	[uwangle=10663]
	[vangrt=10652]
	[varepsilon=1013]
	[varkappa=1008]
	[varnothing=8709]
	[varphi=981]
	[varpi=982]
	[varpropto=8733]
	[vArr=8661]
	[varr=8597]
	[varrho=1009]
	[varsigma=962]
	[varsubsetneq=8842, 65024]
	[varsubsetneqq=10955, 65024]
	[varsupsetneq=8843, 65024]
	[varsupsetneqq=10956, 65024]
	[vartheta=977]
	[vartriangleleft=8882]
	[vartriangleright=8883]
	[Vbar=10987]
	[vBar=10984]
	[vBarv=10985]
	[Vcy=1042]
	[vcy=1074]
	[VDash=8875]
	[Vdash=8873]
	[vDash=8872]
	[vdash=8866]
	[Vdashl=10982]
	[Vee=8897]
	[vee=8744]
	[veebar=8891]
	[veeeq=8794]
	[vellip=8942]
	[Verbar=8214]
	[verbar=124]
	[Vert=8214]
	[vert=124]
	[VerticalBar=8739]
	[VerticalLine=124]
	[VerticalSeparator=10072]
	[VerticalTilde=8768]
	[VeryThinSpace=8202]
	[Vfr=120089]
	[vfr=120115]
	[vltri=8882]
	[vnsub=8834, 8402]
	[vnsup=8835, 8402]
	[Vopf=120141]
	[vopf=120167]
	[vprop=8733]
	[vrtri=8883]
	[Vscr=119985]
	[vscr=120011]
	[vsubnE=10955, 65024]
	[vsubne=8842, 65024]
	[vsupnE=10956, 65024]
	[vsupne=8843, 65024]
	[Vvdash=8874]
	[vzigzag=10650]
	[Wcirc=372]
	[wcirc=373]
	[wedbar=10847]
	[Wedge=8896]
	[wedge=8743]
	[wedgeq=8793]
	[weierp=8472]
	[Wfr=120090]
	[wfr=120116]
	[Wopf=120142]
	[wopf=120168]
	[wp=8472]
	[wr=8768]
	[wreath=8768]
	[Wscr=119986]
	[wscr=120012]
	[xcap=8898]
	[xcirc=9711]
	[xcup=8899]
	[xdtri=9661]
	[Xfr=120091]
	[xfr=120117]
	[xhArr=10234]
	[xharr=10231]
	[Xi=926]
	[xi=958]
	[xlArr=10232]
	[xlarr=10229]
	[xmap=10236]
	[xnis=8955]
	[xodot=10752]
	[Xopf=120143]
	[xopf=120169]
	[xoplus=10753]
	[xotime=10754]
	[xrArr=10233]
	[xrarr=10230]
	[Xscr=119987]
	[xscr=120013]
	[xsqcup=10758]
	[xuplus=10756]
	[xutri=9651]
	[xvee=8897]
	[xwedge=8896]
	[Yacute=221]
	[Yacute=221]
	[yacute=253]
	[yacute=253]
	[YAcy=1071]
	[yacy=1103]
	[Ycirc=374]
	[ycirc=375]
	[Ycy=1067]
	[ycy=1099]
	[yen=165]
	[yen=165]
	[Yfr=120092]
	[yfr=120118]
	[YIcy=1031]
	[yicy=1111]
	[Yopf=120144]
	[yopf=120170]
	[Yscr=119988]
	[yscr=120014]
	[YUcy=1070]
	[yucy=1102]
	[Yuml=376]
	[yuml=255]
	[yuml=255]
	[Zacute=377]
	[zacute=378]
	[Zcaron=381]
	[zcaron=382]
	[Zcy=1047]
	[zcy=1079]
	[Zdot=379]
	[zdot=380]
	[zeetrf=8488]
	[ZeroWidthSpace=8203]
	[Zeta=918]
	[zeta=950]
	[Zfr=8488]
	[zfr=120119]
	[ZHcy=1046]
	[zhcy=1078]
	[zigrarr=8669]
	[Zopf=8484]
	[zopf=120171]
	[Zscr=119989]
	[zscr=120015]
	[zwj=8205]
	[zwnj=8204]'! !

!HTMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/19/2013 07:23'!
initialize
	"self initialize"

	super initialize.
	self initializeHTMLNameDelimiters.! !

!HTMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/1/2013 00:49'!
initializeHTMLNameDelimiters
	(HtmlNameDelimiters := CharacterSet new)
		addAll: Character separators;
		add: $>;
		add: $/;
		add: $<;
		add: $=;
		add: $";
		add: $';
		add: $;! !

!HTMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/29/2013 00:17'!
initializePredefinedEntities
	| entityPage entityName codePoint |

	predefinedEntities := Dictionary new.
	entityPage := self htmlEntityPage readStream.
	[entityPage atEnd]
		whileFalse: [
			entityPage skipTo: $[.
			entityName := entityPage upTo: $=.
			codePoint := entityPage upTo: $].
			(entityName notEmpty and: [codePoint notEmpty])
				ifTrue: [
					predefinedEntities
						at: entityName
						put: (self encodingClass value: codePoint asInteger)]].! !

!HTMLTokenizer class methodsFor: 'testing' stamp: 'monty 12/1/2013 01:02'!
isNonHTMLCodeSectionTag: aTagName
	^ #('script' 'style') includes: aTagName! !

!HTMLTokenizer class methodsFor: 'testing' stamp: 'monty 12/1/2013 01:02'!
isNonHTMLSectionTag: aTagName
	^ #('title' 'script' 'style') includes: aTagName! !

!HTMLTokenizer methodsFor: 'testing' stamp: 'monty 12/7/2013 03:25'!
atEnd
	context atEnd
		ifTrue: [^ true].
	(context isInitialized and: [streamReader atEnd])
		ifTrue: [
			self nextEndDocument.
			^ true].
	^ false.! !

!HTMLTokenizer methodsFor: 'testing' stamp: 'monty 11/23/2013 18:23'!
atNonHTMLSectionEndTag
	(streamReader nextMatchAll: '</')
		ifFalse: [^ false].
	nonHtmlElement doWithIndex: [:each :i |
		(streamReader peek asLowercase == each)
			ifTrue: [streamReader next]
			ifFalse: [
				streamReader pushBack: '</', (nonHtmlElement copyFrom: 1to: i - 1).
				^ false]].
	streamReader pushBack: '</', nonHtmlElement.
	^ true.! !

!HTMLTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/18/2013 23:16'!
characterFromCodePoint: aCodePoint
	^ aCodePoint > 0
		ifTrue: [
			[self class encodingClass value: aCodePoint]
				on: Error
				do: [:error | Character space]]
		ifFalse: [Character space]! !

!HTMLTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 11/22/2013 19:19'!
expectNext: aCharacter
	(streamReader peek == aCharacter)
		ifTrue: [streamReader next].
	^ aCharacter.! !

!HTMLTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 11/22/2013 19:11'!
expectNextAll: aString
	aString doWithIndex: [:each :i |
		each == streamReader peek
			ifTrue: [streamReader next]
			ifFalse: [
				self nextName.
				^ aString]].
	^ aString.! !

!HTMLTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/29/2013 00:17'!
expectTerminator: aCharacter
	streamReader skipTo: aCharacter! !

!HTMLTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:38'!
expectUpToAll: aString
	^ streamReader upToAll: aString! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/9/2013 01:34'!
nextAttributeValue
	| quote value |

	streamReader peek == $&
		ifTrue: [^ self nextUnparsedEntityReference].

	streamReader atQuote
		ifFalse: [^ self nextName].

	quote := streamReader next.
	context resetAfter: [
		context enterLiteralValue.
		value := self
			nextDelimitedBy: quote and: nil
			entityChar: $&
			normalizeWhitespace: false
			ignorableWhitespace: false].
	self expectNext: quote.

	^ value.! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/10/2013 07:27'!
nextContentMarkupToken
	streamReader peek == $?
		ifTrue: [^ self nextPIOrXMLDecl].
	streamReader peek == $!!
		ifTrue: [
			streamReader next.
			streamReader peek == $-
				ifTrue: [^ self nextComment].
			streamReader peek == $[
				ifTrue: [^ self nextCDataSection]].
	self nextTag.! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/30/2013 07:56'!
nextContentToken
	| pcData |

	context isInNonHTMLSection
		ifTrue: [^ self nextNonHTMLSectionToken].

	(pcData := self nextPCDataToken)
		ifNotEmpty: [driver handlePCData: pcData]
		ifEmpty: [
			streamReader peek == $<
				ifTrue: [
					streamReader next.
					(streamReader atEnd not and: [streamReader peek isSeparator])
						ifTrue: [
							streamReader pushBack: '&lt;'.
							driver handlePCData: pcData.
							^ self].
			self nextContentMarkupToken]]! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/30/2013 08:09'!
nextEndTag
	| tagName |

	streamReader next.
	tagName := self nextName asLowercase.
	streamReader skipSeparators.
	self expectTerminator: $>.

	driver handleEndTag: tagName.! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/14/2013 02:08'!
nextGeneralEntityReference
	| name |

	name := self nextName.
	self expectNext: $;.
	^ self class predefinedEntities
		at: name
		ifAbsent: [nil].! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/14/2013 02:12'!
nextName
	| nextChar |

	^ streamWriter writeWith: [:writeStream |
		 [(nextChar := streamReader peek) isNil
			or: [(HtmlNameDelimiters includes: nextChar)
				or: [(context isInDoctype and: [nextChar == $[])
					or: [context isInInternalSubset and: [nextChar == $]]]]]]
			whileFalse: [writeStream nextPut: streamReader next].

		writeStream contents].! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/30/2013 21:23'!
nextNonHTMLCodeSectionOn: aWriteStream
	| nextChar quote |

	nextChar := streamReader next.
	context isInNonHTMLComment
		ifTrue: [
			(nonHtmlCommentStarter = '//' and: [nextChar == Character lf])
				ifTrue: [context enterNonHTMLSection].
			(nonHtmlCommentStarter = '/*'
				and: [nextChar == $*
					and: [streamReader peek == $/]])
				ifTrue: [
					aWriteStream nextPut: nextChar.
					nextChar := streamReader next.
					context enterNonHTMLSection]]
		ifFalse: [
			(nextChar == $" or: [nextChar == $'])
				ifTrue: [
					aWriteStream
						nextPut: (quote := nextChar);
						nextPutAll: (streamReader upTo: quote)]
				ifFalse: [
					((nextChar == $/)
						and: [streamReader peek == $*
							or: [streamReader peek == $/]])
						ifTrue: [
							context enterNonHTMLComment.
							aWriteStream nextPut: $/.
							nextChar := streamReader next.
							nonHtmlCommentStarter := '/'  copyWith: nextChar]]].
	aWriteStream nextPut: nextChar.! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/1/2013 01:02'!
nextNonHTMLSectionToken
	| nonHtmlSection nextChar  |

	streamWriter writeWith: [:writeStream |
		[streamReader atEnd
			or: [self atNonHTMLSectionEndTag]]
			whileFalse: [
				(self class isNonHTMLCodeSectionTag: nonHtmlElement)
					ifTrue: [self nextNonHTMLCodeSectionOn: writeStream]
					ifFalse: [
						nextChar := streamReader next.			
						writeStream nextPut: nextChar]].
		nonHtmlSection := writeStream contents].
	nonHtmlElement := nil.
	context enterContent.

	driver handleNonHTMLSection: nonHtmlSection.! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/18/2013 23:19'!
nextPCDataToken
	| isIgnorableWhitespace nextChar pcData |

	isIgnorableWhitespace := true.
	pcData := ''.
	streamWriter writeWith: [:writeStream |
		[(nextChar := streamReader peek) isNil
			or: [nextChar == $<]]
			whileFalse: [
				streamReader next.
				nextChar == $&
					ifTrue: [
						(streamReader atEnd not and: [streamReader peek isSeparator])
							ifTrue: [
								writeStream nextPut: $&.
								isIgnorableWhitespace := false]
							ifFalse: [
								streamReader peek == $#
									ifTrue: [
										writeStream nextPut: self nextDecodedCharReference.
										isIgnorableWhitespace := false]
									ifFalse: [
										(nextChar := self nextGeneralEntityReference)
											ifNotNil: [
												writeStream nextPut: nextChar.
												isIgnorableWhitespace := false]]]]
					ifFalse: [
						nextChar isSeparator
							ifFalse: [isIgnorableWhitespace := false].
						writeStream nextPut: nextChar]].
		isIgnorableWhitespace
			ifTrue: [
				writeStream position > 0
					ifTrue: [driver handleWhitespace: writeStream contents]]
			ifFalse: [pcData :=  writeStream contents]].
	^ pcData.! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:19'!
nextPITarget
	^ self nextName! !

!HTMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2013 01:49'!
nextParameterEntityReference
	self
		nextName;
		expectNext: $;! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 03:53'!
nextPrologToken
	streamReader skipSeparators.
	streamReader peek == $<
		ifTrue: [
			streamReader next.
			streamReader peek == $?
				ifTrue: [^ self nextPIOrXMLDecl].
			streamReader peek == $!!
				ifTrue: [
					streamReader next.
					streamReader peek == $-
						ifTrue: [^ self nextComment].
				^ self nextDocTypeDecl].

			context enterContent.
			^ (streamReader atEnd not and: [streamReader peek isSeparator])
				ifTrue: [
					streamReader pushBack: '<'.
					self nextContentToken]
				ifFalse: [self nextTag]].

	context enterContent.
	^ self nextContentToken.! !

!HTMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/22/2013 10:37'!
nextPubidLiteral
	^ self nextQuotedValue! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/6/2013 08:36'!
nextQuotedValue
	^ streamReader atQuote
		ifTrue: [streamReader upTo: streamReader next]
		ifFalse: [self nextName]! !

!HTMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2013 02:33'!
nextSubsetMarkupToken
	streamReader peek == $<
		ifTrue: [streamReader next]
		ifFalse: [^ self skipNonDeclarationOrEntity].

	self expectNext: $!!.
	streamReader peek == $-
		ifTrue: [^ self nextComment].

	self skipDeclaration.! !

!HTMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/22/2013 10:39'!
nextSystemLiteralRequired: aBoolean
	^ self nextQuotedValue! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/1/2013 01:04'!
nextTag
	| tagName attributes attributeName  nextChar |

	(streamReader peek = $/)
		ifTrue: [^ self nextEndTag].
	tagName := self nextName asLowercase.

	attributes := OrderPreservingDictionary new: 10 withDefaultValue: ''.
	[streamReader skipSeparators.
	((nextChar := streamReader peek) isNil
		or: [nextChar == $>
			or: [nextChar == $/
				or: [(attributeName := self nextName asLowercase) isEmpty]]])]
		whileFalse: [
			streamReader skipSeparators.
			self expectNext: $=.
			streamReader skipSeparators.
			attributes at: attributeName put: self nextAttributeValue].

	driver
		handleStartTag: tagName
		attributes: attributes
		namespaces: nil.
	streamReader peek == $/
		ifTrue: [
			streamReader next.
			driver handleEndTag: tagName]
		ifFalse: [
			(self class isNonHTMLSectionTag: tagName)
				ifTrue: [
					nonHtmlElement := tagName.
					context enterNonHTMLSection]].
	self expectTerminator: $>.! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/6/2013 08:37'!
nextValue
	^ streamReader atQuote
		ifTrue: [streamReader upTo: streamReader next]
		ifFalse: [self nextName]! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:48'!
nextXMLEncodingAttributeValue
	^ self nextQuotedValue! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:48'!
nextXMLStandaloneAttributeValue
	^ self nextQuotedValue! !

!HTMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:47'!
nextXMLVersionAttributeValue
	^ self nextQuotedValue! !

!HTMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/29/2013 00:17'!
skipDeclaration
	[streamReader atEnd
		or: [streamReader peek == $>]]
		whileFalse: [
			streamReader atQuote
				ifTrue: [streamReader skipTo: streamReader next]
				ifFalse: [streamReader next]].

	streamReader atEnd
		ifFalse: [streamReader next].! !

!HTMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/22/2013 20:13'!
skipNonDeclarationOrEntity
	[streamReader atEnd
		or: [(streamReader peek == $<)
			or: [(streamReader peek == $%)
				or: [context isInInternalSubset and: [streamReader peek ==$]]]]]]
		whileFalse: [streamReader next].
	! !

!HTMLTokenizer methodsFor: 'accessing' stamp: 'monty 11/23/2013 04:58'!
tokenContextClass
	^ HTMLTokenContext! !

!XMLDriverTokenizer class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 03:26'!
driver: aDriver on: aStringOrStream
	^self new
		setDriver: aDriver
		stream:
			(aStringOrStream isStream
				ifTrue: [aStringOrStream]
				ifFalse: [aStringOrStream readStream])! !

!XMLDriverTokenizer class methodsFor: 'accessing' stamp: 'monty 12/18/2013 23:16'!
encodingClass
	^ Unicode! !

!XMLDriverTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/19/2013 07:23'!
initialize
	"self initialize"

	super initialize.
	self initializePredefinedEntities.! !

!XMLDriverTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/7/2013 00:38'!
initializePredefinedEntities! !

!XMLDriverTokenizer class methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:37'!
predefinedEntities
	^ predefinedEntities! !

!XMLDriverTokenizer methodsFor: 'testing' stamp: 'monty 12/7/2013 00:42'!
atEnd
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/9/2013 07:53'!
characterFromCodePoint: aCodePoint
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
currentLineNumber
	^ streamReader currentLineNumber! !

!XMLDriverTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
driver
	^ driver! !

!XMLDriverTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:35'!
expectNext: aCharacter
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:37'!
expectNextAll: aString
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:36'!
expectTerminator: aCharacter
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:38'!
expectUpToAll: aString
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/9/2013 01:15'!
nextCDataSection
	self expectNextAll: '[CDATA['.

	driver handleCData: (self expectUpToAll: ']]>').
! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/29/2013 05:39'!
nextComment
	| comment nextChar |

	"Skip first -"
	streamReader next.
	self expectNext: $-.

	comment := streamWriter writeWith: [:writeStream |
		[streamReader atEnd
			or: [(nextChar := streamReader next) == $-
				and: [(streamReader peek) == $-]]]
			whileFalse: [writeStream nextPut: nextChar].
		writeStream contents].
	self
		expectNext: $-;
		expectTerminator: $>.

	driver handleComment: comment.! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 03:15'!
nextContentToken
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/9/2013 16:47'!
nextDecodedCharReference
	| codePoint |

	"skip #"
	streamReader next.
	codePoint := streamReader nextInteger.
	self expectNext: $;.

	^ self characterFromCodePoint: codePoint.! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/22/2013 06:14'!
nextDelimitedBy: aDelimiter and: aSecondDelimiter entityChar: anEntityStartChar normalizeWhitespace: shouldNormalize ignorableWhitespace: shouldSkip
	| isIgnorableWhitespace nextChar pcData |

	isIgnorableWhitespace := true.
	pcData := ''.
	streamWriter writeWith: [:writeStream |
		"separate arguments for delimiters are used instead of a collection for performance"
		[(nextChar := streamReader peek) isNil
			or: [nextChar == aDelimiter
				or: [nextChar == aSecondDelimiter]]]
			whileFalse: [
				streamReader next.
				nextChar == $&
					ifTrue: [
						streamReader peek == $#
							ifTrue: [
								writeStream nextPut: self nextDecodedCharReference.
								isIgnorableWhitespace := false]
							ifFalse: [
								anEntityStartChar == $&
									ifTrue: [
										(nextChar := self nextGeneralEntityReference)
											ifNotNil: [
												writeStream nextPut: nextChar.
												isIgnorableWhitespace := false]]
									ifFalse: [
										writeStream nextPut: $&.
										isIgnorableWhitespace := false]]]
					ifFalse: [
						nextChar == anEntityStartChar
							ifTrue: [self nextParameterEntityReference]
							ifFalse: [
								nextChar isSeparator
									ifTrue: [
										shouldNormalize
											ifTrue: [nextChar := Character space]]
									ifFalse: [isIgnorableWhitespace := false].
								writeStream nextPut: nextChar]]].
		isIgnorableWhitespace & shouldSkip
			ifTrue: [
				writeStream position > 0
					ifTrue: [driver handleWhitespace: writeStream contents]]
			ifFalse: [pcData :=  writeStream contents]].
	^ pcData.! !

!XMLDriverTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/12/2013 00:08'!
nextDocTypeDecl
	|  root externalId |

	context enterDoctype.
	self expectNextAll: 'DOCTYPE'.
	streamReader skipSeparators.

	root := self nextName.
	streamReader skipSeparators.
	externalId := self nextExternalIDSystemLiteralRequired: true.
	driver
		handleStartDTD: root
		publicID: externalId key
		systemID: externalId value.

	streamReader skipSeparators.
	streamReader peek == $[
		ifTrue: [self nextInternalSubsetStart]
		ifFalse: [self nextEndDocTypeDecl].! !

!XMLDriverTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 03:38'!
nextEndDocTypeDecl
	streamReader skipSeparators.
	self expectTerminator: $>.
	context enterProlog.

	driver handleEndDTD.! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 03:15'!
nextEndDocument
	context enterAtEnd.
	driver handleEndDocument.! !

!XMLDriverTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:31'!
nextEndInternalSubset
	self expectTerminator: $].

	context enterProlog.
	self nextEndDocTypeDecl.! !

!XMLDriverTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/12/2013 00:36'!
nextExternalIDSystemLiteralRequired: aBoolean
	| publicId systemId |

	(streamReader atEnd not
		and: [streamReader peek == $P
			or: [streamReader peek == $S]])
		ifTrue: [
			streamReader peek == $P
				ifTrue: [
					self
						expectNextAll: 'PUBLIC';
						skipSeparatorsReplacingParameterEntities.
					publicId := self nextPubidLiteral.

					self skipSeparatorsReplacingParameterEntities.
					systemId := self nextSystemLiteralRequired: aBoolean]
				ifFalse: [
					self
						expectNextAll: 'SYSTEM';
						skipSeparatorsReplacingParameterEntities.
					systemId := self nextSystemLiteralRequired: true]].

	^ (publicId ifNil: ['']) -> (systemId ifNil: ['']).! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/11/2013 01:47'!
nextGeneralEntityReference
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 03:21'!
nextInternalSubsetStart
	"skip ["
	streamReader next.
	context enterInternalSubset.! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 01:14'!
nextName
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:27'!
nextPIOrXMLDecl
	| piTarget piData |

	"Skip ?"
	streamReader next.
	(piTarget := self nextPITarget) = 'xml'
		ifTrue: [^ self nextXMLDecl]
		ifFalse: [
			context isInExternalSubsetTextDeclaration
				ifTrue: [context enterExternalSubset]].

	streamReader skipSeparators.
	piData := self expectUpToAll: '?>'.

	driver handlePI: piTarget data: piData.! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:16'!
nextPITarget
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2013 01:47'!
nextParameterEntityReference
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 03:19'!
nextPrologToken
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 03:18'!
nextPubidLiteral
	self subclassResponsiblity! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 03:15'!
nextStartDocument
	context enterProlog.
	driver handleStartDocument.! !

!XMLDriverTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:21'!
nextSubsetMarkupToken
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 03:16'!
nextSubsetToken
	streamReader skipSeparators.
	streamReader atEnd
		ifTrue: [^ self].

	streamReader peek == $%
		ifTrue: [^ self replaceParameterEntityReference].
	(context isInInternalSubset and: [streamReader peek == $]])
		ifTrue: [^ self nextEndInternalSubset].

	self nextSubsetMarkupToken.! !

!XMLDriverTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 03:22'!
nextSystemLiteralRequired: aBoolean
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 04:30'!
nextTag
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 03:14'!
nextToken
	self atEnd
		ifTrue: [^ self].
	context isInContent
		ifTrue: [^ self nextContentToken].
	context isInSubset
		ifTrue: [^ self nextSubsetToken].
	context notInitialized 
		ifTrue: [^ self nextStartDocument].

	self nextPrologToken.! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 01:26'!
nextUnparsedEntityReference
	| name |

	streamReader next.
	name := self nextName.
	self expectNext: $;.

	^ (driver handleUnparsedEntityReference: name) ifNil: [''].! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:33'!
nextXMLAttributeName: aName
	self expectNextAll: aName.
	streamReader skipSeparators.
	self expectNext: $=.
	streamReader skipSeparators.! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 08:09'!
nextXMLDecl
	| version encoding standalone isTextDeclaration |

	isTextDeclaration  := context isInTextDeclaration.
	version := self nextXMLVersionAttributeRequired: isTextDeclaration not.
	(encoding := self nextXMLEncodingAttributeRequired: isTextDeclaration)
		ifNotEmpty: [streamReader useConverterForEncoding: encoding].
	isTextDeclaration
		ifFalse: [standalone := self nextXMLStandaloneAttribute].

	streamReader skipSeparators.
	self
		expectNext: $?;
		expectTerminator: $>.

	isTextDeclaration
		ifTrue: [
			context isInExternalSubsetTextDeclaration
				ifTrue: [context enterExternalSubset]]
		ifFalse: [
			driver
				handleXMLVersion: version
				encoding: encoding
				standalone: standalone].! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:47'!
nextXMLEncodingAttributeRequired: aBoolean
	streamReader skipSeparators.
	(aBoolean or: [streamReader peek == $e])
		ifFalse: [^ ''].

	^ self
		nextXMLAttributeName: 'encoding';
		nextXMLEncodingAttributeValue.! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:47'!
nextXMLEncodingAttributeValue
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:43'!
nextXMLStandaloneAttribute
	streamReader skipSeparators.
	(streamReader peek == $s)
		ifFalse: [^ ''].

	^ self
		nextXMLAttributeName: 'standalone';
		nextXMLStandaloneAttributeValue.! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:44'!
nextXMLStandaloneAttributeValue
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:33'!
nextXMLVersionAttributeRequired: aBoolean
	streamReader skipSeparators.
	(aBoolean or: [streamReader peek == $v])
		ifFalse: [^ ''].

	^ self
		nextXMLAttributeName: 'version';
		nextXMLVersionAttributeValue.! !

!XMLDriverTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:34'!
nextXMLVersionAttributeValue
	self subclassResponsibility! !

!XMLDriverTokenizer methodsFor: 'accessing' stamp: 'monty 12/18/2013 17:26'!
normalizedLineEndingChar: aCharacter
	streamReader normalizedLineEndingChar: aCharacter! !

!XMLDriverTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 04:31'!
replaceParameterEntityReference
	streamReader next.
	self nextParameterEntityReference! !

!XMLDriverTokenizer methodsFor: 'initialization' stamp: 'monty 1/4/2014 08:06'!
setDriver: aDriver stream: aStream
	driver := aDriver.
	context := self tokenContextClass new.
	streamReader :=
		(XMLNestedStreamReader
			on: aStream
			readLimit: nil).
	streamWriter := XMLNestedStreamWriter new: 3.! !

!XMLDriverTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:36'!
skipSeparatorsReplacingParameterEntities
	"this can be used to replace references within declarations"

	streamReader skipSeparators.
	[streamReader atEnd not
		and: [context isInExternalSubset
			and: [streamReader peek == $%]]]
		whileTrue: [
			self replaceParameterEntityReference.
			streamReader skipSeparators].! !

!XMLDriverTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
streamReader
	^ streamReader! !

!XMLDriverTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
tokenContext
	^ context! !

!XMLDriverTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:36'!
tokenContextClass
	self subclassResponsibility! !

XMLDriverTokenizer subclass: #XMLTokenizer
	instanceVariableNames: 'activeEntities'
	classVariableNames: 'AdditionalNameChars AttributeTypeDelimiters EncodingNameChars NameStartChars PubidChars UnaryOperatorChars ValueListChars'
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLTokenizer commentStamp: '<historical>' prior: 0!
This class reads XML tokens from a stream using a nested stream reader. Sending it #nextToken causes a token to be read and one or more handler messages to be dispatched to a driver.!

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 1/4/2014 06:59'!
initialize
	"self initialize"

	super initialize.
	self
		initializeNameStartChars;
		initializeAdditionalNameChars;
		initializePubidChars;
		initializeAttributeTypeDelimiters;
		initializeValueListChars;
		initializeUnaryOperatorChars;
		initializeEncodingNameChars.! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'JAA 3/22/2011 12:43'!
initializeAdditionalNameChars
	"This creates a character set that contains all those defined in the XML spec for the
	NameChar production rule minus those also defined for NameStartChar
	(NameChar - NameStartChar)"

	(AdditionalNameChars := BitmapCharacterSet new: 16r2041)
		add: $-;
		add: $.;
		addAll: ($0 to: $9);
		add: (Character value: 16rB7).
	16r0300 to: 16r036F do: [:each | AdditionalNameChars add: (Character value: each)].
	16r203F to: 16r2040 do: [:each | AdditionalNameChars add: (Character value: each)].! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/29/2013 05:29'!
initializeAttributeTypeDelimiters
	AttributeTypeDelimiters := CharacterSet with: $# with: $' with: $" with: $>! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 1/4/2014 07:00'!
initializeEncodingNameChars
	(EncodingNameChars := CharacterSet new)
		addAll: ($a to: $z);
		addAll: ($A to: $Z);
		addAll: ($0 to: $9);
		add: $-;
		add: $_;
		add: $.! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'JAA 3/22/2011 12:43'!
initializeNameStartChars
	"This creates a character set that coveres the entire range of characters speicified for
	the NameStartChar production rule in the XML spec."

	(NameStartChars := BitmapCharacterSet new: 16rF0000)
		add: $:;
		addAll: ($A to: $Z);
		add: $_;
		addAll: ($a to: $z).
	16rC0 to: 16rD6 do: [:each | NameStartChars add: (Character value: each)].
	16rD8 to: 16rF6 do: [:each | NameStartChars add: (Character value: each)].
	16rF8 to: 16r2FF do: [:each | NameStartChars add: (Character value: each)].
	16r370 to: 16r37D do: [:each | NameStartChars add: (Character value: each)].
	16r37F to: 16r1FFF do: [:each | NameStartChars add: (Character value: each)].
	16r200C to: 16r200D do: [:each | NameStartChars add: (Character value: each)].
	16r2070 to: 16r218F do: [:each | NameStartChars add: (Character value: each)].
	16r2C00 to: 16r2FEF do: [:each | NameStartChars add: (Character value: each)].
	16r3001 to: 16rD7FF do: [:each | NameStartChars add: (Character value: each)].
	16rF900 to: 16rFDCF do: [:each | NameStartChars add: (Character value: each)].
	16rFDF0 to: 16rFFFD do: [:each | NameStartChars add: (Character value: each)].
	16r10000 to: 16rEFFFF do: [:each | NameStartChars add: (Character value: each)].! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/1/2013 00:43'!
initializePredefinedEntities
	(predefinedEntities := Dictionary new)
		at: 'lt' put: $<;
		at: 'gt' put: $>;
		at: 'amp' put: $&;
		at: 'apos' put: $';
		at: 'quot' put: $"! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 11/12/2013 01:44'!
initializePubidChars
	(PubidChars := BitmapCharacterSet new)
		add: Character space;
		add: Character cr;
		add: Character lf;
		addAll: ($a to: $z);
		addAll: ($A to: $Z);
		addAll: ($0 to: $9);
		addAll: '-''()+,./:=?;!!*#@$_%'! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/29/2013 05:31'!
initializeUnaryOperatorChars
	UnaryOperatorChars := CharacterSet with: $? with: $* with: $+! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/29/2013 05:29'!
initializeValueListChars
	ValueListChars := CharacterSet with: $( with: $| with: $)! !

!XMLTokenizer class methodsFor: 'testing' stamp: 'monty 11/19/2013 16:43'!
isName: aString
	aString ifEmpty: [^ false].

	aString doWithIndex: [:each :i | 
		((NameStartChars includes: each)
			or: [i > 1 and: [AdditionalNameChars includes: each]])
			ifFalse: [^ false]].

	^ true.! !

!XMLTokenizer class methodsFor: 'testing' stamp: 'monty 12/28/2013 23:18'!
isNmtoken: aString
	^ aString notEmpty and: [aString allSatisfy: [:each | self isNmtokenChar: each]]! !

!XMLTokenizer class methodsFor: 'testing' stamp: 'monty 12/28/2013 23:01'!
isNmtokenChar: aCharacter
	^ (NameStartChars includes: aCharacter)
		or: [AdditionalNameChars includes: aCharacter]! !

!XMLTokenizer methodsFor: 'accessing' stamp: 'monty 12/13/2013 19:54'!
activeEntities
	^ activeEntities
		ifNil: [activeEntities := OrderedCollection new: self maxEntityReplacementDepth]! !

!XMLTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/13/2013 19:54'!
addActiveEntity: anEntity
	self activeEntities addLast: anEntity.

	driver activeEntityURI: anEntity uri.! !

!XMLTokenizer methodsFor: 'testing' stamp: 'monty 11/13/2013 20:14'!
atEnd
	context atEnd
		ifTrue: [^ true].
	(context isInitialized and: [streamReader atEnd])
		ifTrue: [
			context isInInternalSubset
				ifTrue: [self errorExpected: 'end of internal subset'].
			self nextEndDocument.
			^ true].
	^ false.! !

!XMLTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/18/2013 23:16'!
characterFromCodePoint: aCodePoint
	aCodePoint > 0
		ifFalse: [self errorExpected: 'postive decimal or hex integer'].

	^ [self class encodingClass value: aCodePoint]
		on: Error
		do: [:error | self parseError: 'Invalid character code point ', aCodePoint printString]! !

!XMLTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/11/2013 01:55'!
checkTextDeclarationInReplacement: aReplacement forEntity: anEntity
	(aReplacement beginsWith: '<?xml ')
		ifTrue: [ 
			streamReader next.
			context resetAfter: [
				context enterTextDeclaration.
				self nextPIOrXMLDecl]].! !

!XMLTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/13/2013 18:30'!
endReplacementForGeneralEntity: anEntity isInContent: aBoolean
	self removeActiveEntity: anEntity.
	aBoolean
		ifTrue: [driver handleEndContentEntityReplacement: anEntity name].! !

!XMLTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/13/2013 18:30'!
endReplacementForParameterEntity: anEntity
	self removeActiveEntity: anEntity! !

!XMLTokenizer methodsFor: 'errors' stamp: 'monty 1/4/2014 07:42'!
entityDepthError: anErrorString
	XMLLimitException signal: anErrorString! !

!XMLTokenizer methodsFor: 'errors' stamp: 'monty 12/10/2013 07:28'!
errorExpected: anExpectedString
	self parseError: 'Expected ', anExpectedString! !

!XMLTokenizer methodsFor: 'errors' stamp: 'monty 12/9/2013 18:29'!
errorExpected: anExpectedCharacterOrString butGot: aReceivedCharacterOrString
	| expectedString receivedString |

	expectedString := anExpectedCharacterOrString asString.	
	(receivedString := (aReceivedCharacterOrString ifNil: ['']) asString)
		ifEmpty: [receivedString := 'nothing'].

	self errorExpected: expectedString, ' but got ', receivedString.! !

!XMLTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 1/4/2014 06:50'!
expectDigit
	| nextChar |

	((nextChar := streamReader next) notNil
		and: [nextChar isDigit])
		ifFalse: [
			self
				errorExpected: 'digit'
				butGot: nextChar].
	^ nextChar.! !

!XMLTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 1/4/2014 06:52'!
expectLetter
	| nextChar |

	((nextChar := streamReader next) notNil
		and: [nextChar isLetter])
		ifFalse: [
			self
				errorExpected: 'letter'
				butGot: nextChar].
	^ nextChar.! !

!XMLTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:35'!
expectNext: aCharacter
	| nextChar |

	(nextChar := streamReader next) == aCharacter
		ifFalse: [
			self
				errorExpected: aCharacter
				butGot: nextChar].
	^ nextChar.! !

!XMLTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:36'!
expectNextAll: anExpectedLiteral
	| nextChar |

	anExpectedLiteral doWithIndex: [:each :i |
		(nextChar := streamReader next) == each
			ifFalse: [
				self
					errorExpected: anExpectedLiteral
					butGot:
						(anExpectedLiteral
							copyReplaceFrom: i
							to: anExpectedLiteral size
							with: (nextChar ifNil: ['']) asString)]].
	^ anExpectedLiteral.! !

!XMLTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/9/2013 06:51'!
expectQuote
	| nextChar |

	(((nextChar := streamReader next) == $")
		or: [nextChar == $'])
		ifFalse: [
			self
				errorExpected: 'quote character delimiter'
				butGot: nextChar].
	^ nextChar.! !

!XMLTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:36'!
expectTerminator: aCharacter
	| nextChar |

	(nextChar := streamReader next) == aCharacter
		ifFalse: [
			self
				errorExpected: aCharacter asString, ' terminator'
				butGot: nextChar].
	^ nextChar.! !

!XMLTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/9/2013 01:22'!
expectUpToAll: aString
	| value isTerminated |

	"upToAll: can't distinguish between a missing terminator or terminator at the end of a string"
	isTerminated := false.
	value := streamWriter writeWith: [:writeStream |
		[streamReader atEnd
			or: [(isTerminated := streamReader nextMatchAll: aString)]]
			whileFalse: [writeStream nextPut: streamReader next].

		writeStream contents].

	isTerminated
		ifFalse: [self errorExpected: 'terminating ', aString].

	^ value.
! !

!XMLTokenizer methodsFor: 'testing' stamp: 'monty 12/13/2013 19:55'!
hasActiveEntities
	^ activeEntities notNil and: [activeEntities notEmpty]! !

!XMLTokenizer methodsFor: 'testing' stamp: 'monty 12/13/2013 19:58'!
hasActiveEntityNamed: aName
	^ self activeEntities anySatisfy: [:each | each name = aName]! !

!XMLTokenizer methodsFor: 'testing' stamp: 'monty 12/13/2013 19:59'!
hasActiveExternalEntity
	^ self activeEntities anySatisfy: [:each | each isExternal]! !

!XMLTokenizer methodsFor: 'accessing' stamp: 'monty 12/11/2013 01:52'!
maxEntityReplacementDepth
	^ 3! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/30/2013 23:47'!
nextAttlistDeclaration
	| element attribute attributeType defaultPragma defaultValue  |

	self
		expectNextAll: 'ATTLIST';
		skipSeparatorsReplacingParameterEntities.

	element := self nextName.
	[self skipSeparatorsReplacingParameterEntities.
	(streamReader atEnd or: [streamReader peek == $>])]
		whileFalse: [
			attribute := self nextName.
			attributeType := self nextAttributeType.
			defaultPragma := self nextAttributeDefaultPragma.
			defaultValue := self nextAttributeDefaultValueForPragma: defaultPragma.
			driver
				handleAttributeDeclaration: element
				name: attribute
				type: attributeType
				defaultPragma: defaultPragma
				defaultValue: defaultValue].

	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/30/2013 23:47'!
nextAttributeDefaultPragma
	self skipSeparatorsReplacingParameterEntities.
	streamReader peek == $#
		ifFalse: [^ ''].
	streamReader next.
	^ streamWriter writeWith: [:writeStream |
		streamReader peek == $R
			ifTrue: [writeStream nextPutAll: (self expectNextAll: 'REQUIRED')]
			ifFalse: [
				streamReader peek == $I
					ifTrue: [writeStream nextPutAll: (self expectNextAll: 'IMPLIED')]
					ifFalse: [writeStream nextPutAll: (self expectNextAll: 'FIXED')]].
		writeStream contents].! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/31/2013 00:32'!
nextAttributeDefaultValueForPragma: aPragma
	self skipSeparatorsReplacingParameterEntities.

	^ (aPragma isEmpty or: [aPragma first == $F]) "FIXED"
		ifTrue: [self nextAttributeValue]
		ifFalse: [''].
! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/29/2013 05:30'!
nextAttributeType
	| nextChar |

	^ streamWriter writeWith: [:writeStream |
		[self skipSeparatorsReplacingParameterEntities.
		(nextChar := streamReader peek) isNil
			or: [AttributeTypeDelimiters includes: nextChar]]
			whileFalse: [
				(ValueListChars includes: nextChar)
					ifTrue: [
						writeStream position > 0
							ifTrue: [writeStream space].
						writeStream nextPut: streamReader next]
					ifFalse: [self nextNmtokenOn: writeStream]].
		writeStream contents]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/22/2013 09:34'!
nextAttributeValue
	| quote value |

	streamReader peek == $&
		ifTrue: [^ self nextUnparsedEntityReference].

	quote := self expectQuote.
	context resetAfter: [
		context enterLiteralValue.
		value := self
			nextDelimitedBy: quote and: $<
			entityChar: $&
			normalizeWhitespace: true
			ignorableWhitespace: false].
	self expectNext: quote.

	^ value.! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/29/2013 05:53'!
nextConditionalSection
	"skip ["
	streamReader next.

	streamWriter writeWith: [:writeStream |
		self nextConditionalSectionOn: writeStream.
		streamReader pushBack: writeStream contents].! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/28/2013 22:29'!
nextConditionalSectionOn: aWriteStream
	self
		skipSeparatorsReplacingParameterEntities;
		expectNext: $I.

	^ streamReader peek == $N
		ifTrue: [self nextIncludeSectionOn: aWriteStream]
		ifFalse: [self nextIgnoreSection].! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/10/2013 07:27'!
nextContentMarkupToken
	streamReader peek == $?
		ifTrue: [^ self nextPIOrXMLDecl].
	streamReader peek == $!!
		ifTrue: [
			streamReader next.
			streamReader peek == $-
				ifTrue: [^ self nextComment].
			streamReader peek == $[
				ifTrue: [^ self nextCDataSection].
			self errorExpected: 'comment or CDATA section'].

	self nextTag.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/26/2013 02:01'!
nextContentToken
	| pcData |

	streamReader peek == $<
		ifTrue: [
			streamReader next.
			^ self nextContentMarkupToken].
	pcData := self
		nextDelimitedBy: $< and: nil
		entityChar: $&
		normalizeWhitespace: false
		ignorableWhitespace: true.
	pcData
		ifNotEmpty: [driver handlePCData: pcData].

		! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/29/2013 05:32'!
nextElementDeclaration
	| name nextChar contentModel |

	self
		expectNextAll: 'LEMENT';
		skipSeparatorsReplacingParameterEntities.

	name := self nextName.
	contentModel := streamWriter writeWith: [:writeStream |
		[self skipSeparatorsReplacingParameterEntities.
		(streamReader atEnd or: [(nextChar := streamReader peek) == $>])]
			whileFalse: [
				nextChar == $#
					ifTrue: [writeStream nextPutAll: (self expectNextAll: '#PCDATA')]
					ifFalse: [
						(nextChar == $, or: [ValueListChars includes: nextChar])
							ifTrue: [writeStream nextPut: streamReader next]
							ifFalse: [self nextNameOn: writeStream]].
				(UnaryOperatorChars includes: streamReader peek)
					ifTrue: [writeStream nextPut: streamReader next]].
		writeStream contents].

	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.

	driver
		handleElementDeclaration: name
		contentModel: contentModel! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:32'!
nextEndInternalSubset
	self hasActiveEntities
		ifTrue: [self parseError: 'Parameter entity replacement cannot terminate internal subset'].

	super nextEndInternalSubset.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/9/2013 01:27'!
nextEndTag
	| tagName |

	"Skip /"
	streamReader next.
	tagName := self nextName.
	streamReader skipSeparators.
	self expectTerminator: $>.

	driver handleEndTag: tagName.! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2013 01:44'!
nextEntityDeclaration
	| isParameterEntity name  |

	self expectNextAll: 'NTITY'.

	"this is to distinguish a parameter reference from declaration"
	isParameterEntity := false.
	[streamReader skipSeparators.
	streamReader peek == $% and: [isParameterEntity not]]
		whileTrue: [
			streamReader next.
			(context isInExternalSubset
				and: [streamReader atEnd not
					and: [streamReader peek isSeparator not]])
				ifTrue: [self nextParameterEntityReference]
				ifFalse: [isParameterEntity := true]].

	self skipSeparatorsReplacingParameterEntities.
	name := self nextName.
	self skipSeparatorsReplacingParameterEntities.
	streamReader atQuote
		ifTrue: [
			self
				nextInternalEntityDeclaration: name
				isParameterEntity: isParameterEntity]
		ifFalse: [
			self
				nextExternalEntityDeclaration: name
				isParameterEntity: isParameterEntity].! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2013 01:44'!
nextEntityValue
	| quote value  |

	quote := streamReader next.
	context resetAfter: [
		context enterLiteralValue.
		value := self
			nextDelimitedBy: quote and: nil
			entityChar: $%
			normalizeWhitespace: false
			ignorableWhitespace: false].
	self expectNext: quote.

	^ value.! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/12/2013 00:09'!
nextExternalEntityDeclaration: aName isParameterEntity: aBoolean
	| externalId notation |

	externalId := self nextExternalIDSystemLiteralRequired: true.
	aBoolean
		ifTrue: [
			driver
				handleParameterEntityDeclaration: aName
				publicID: externalId key
				systemID: externalId value]
		ifFalse: [
			self skipSeparatorsReplacingParameterEntities.
			streamReader peek == $>
				ifTrue: [notation := '']
				ifFalse: [
					self
						expectNextAll: 'NDATA';
						skipSeparatorsReplacingParameterEntities.
					notation := self nextName].
			driver
				handleGeneralEntityDeclaration: aName
				publicID: externalId key
				systemID: externalId value
				ndata: notation].
	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/11/2013 01:47'!
nextGeneralEntityReference
	| name entity replacement startedInContent |

	name := self nextName.
	self expectNext: $;.
	(self class predefinedEntities includesKey: name)
		ifTrue: [^ self class predefinedEntities at: name].

	entity := driver handleGeneralEntityReference: name.
	replacement := self replacementFromGeneralEntity: entity.

	replacement
		ifNotEmpty: [
			startedInContent := context isInContent.
			self
				pushBackReplacement: replacement
				forEntity: entity
				onClose: [
					self
						endReplacementForGeneralEntity: entity
						isInContent: startedInContent].
			startedInContent
				ifTrue: [driver handleStartContentEntityReplacement: name]].
	^ nil.! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 06:50'!
nextIgnoreSection
	| openSections |

	self expectNextAll: 'GNORE'.
	streamReader skipSeparators.
	self expectNext: $[.

	openSections := 1.
	[openSections > 0 and: [streamReader atEnd not]]
		whileTrue: [
			(streamReader nextMatchAll: ']]>')
				ifTrue: [openSections := openSections - 1]
				ifFalse: [
					(streamReader nextMatchAll: '<!![')
						ifTrue: [openSections := openSections + 1]
						ifFalse: [streamReader next]]].

	openSections > 0
		ifTrue: [self errorExpected: 'terminating ]]>'].

	^ ''.! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/28/2013 22:28'!
nextIncludeSectionOn: aWriteStream
	| isOpen  |

	self expectNextAll: 'NCLUDE'.
	streamReader skipSeparators.
	self expectNext: $[.

	isOpen := true.
	[isOpen and: [streamReader atEnd not]]
		whileTrue: [
			(streamReader nextMatchAll: '<!![')
				ifTrue: [self nextConditionalSectionOn: aWriteStream]
				ifFalse: [
					(streamReader nextMatchAll: ']]>')
						ifTrue: [isOpen := false]
						ifFalse: [aWriteStream nextPut: streamReader next]]].

	isOpen
		ifTrue: [self errorExpected: 'terminating ]]>'].! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2013 01:44'!
nextInternalEntityDeclaration: aName isParameterEntity: aBoolean
	| value |

	value := self nextEntityValue.
	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.

	aBoolean
		ifTrue: [driver handleParameterEntityDeclaration: aName replacement: value]
		ifFalse: [driver handleGeneralEntityDeclaration: aName replacement: value].! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/28/2013 21:09'!
nextName
	^ streamWriter writeWith: [:writeStream |
		self nextNameOn: writeStream.
		writeStream contents]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/30/2013 22:37'!
nextNameOn: aWriteStream
	| nextChar |

	((nextChar := streamReader next) notNil
		and: [NameStartChars includes: nextChar])
		ifFalse: [self errorExpected: 'name'].
	aWriteStream nextPut: nextChar.

	[(nextChar := streamReader peek) notNil
		and: [(NameStartChars includes: nextChar)
			or: [AdditionalNameChars includes: nextChar]]]
		whileTrue: [aWriteStream nextPut: streamReader next].! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/30/2013 22:38'!
nextNmtokenOn: aWriteStream
	"matches the nmtoken production"
	|  nextChar oldPosition |

	oldPosition := aWriteStream position.
	[(nextChar := streamReader peek) notNil
		and: [(NameStartChars includes: nextChar)
			or: [AdditionalNameChars includes: nextChar]]]
		whileTrue: [aWriteStream nextPut: streamReader next].

	(aWriteStream position > oldPosition)
		ifFalse: [self errorExpected: 'name token'].! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/12/2013 00:10'!
nextNotationDeclaration
	| name externalId |

	self
		expectNextAll: 'NOTATION';
		skipSeparatorsReplacingParameterEntities.

	name := self nextName.
	self skipSeparatorsReplacingParameterEntities.

	externalId := self nextExternalIDSystemLiteralRequired: false.
	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.

	driver
		handleNotationDeclaration: name
		publicID: externalId key
		systemID: externalId value.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 22:25'!
nextPITarget
	| target |

	((target := self nextName) sameAs: 'xml')
		ifTrue: [
			target = 'xml'
				ifFalse: [self parseError: 'The "xml" target must be lowercase'].
			(context isInProlog or: [context isInTextDeclaration])
				ifFalse: [
					self parseError:
						'XML declaration outside of prolog or text declaration']].
	^ target.! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2013 01:47'!
nextParameterEntityReference
	| name entity replacement |

	name := self nextName.
	self expectNext: $;.

	entity := driver handleParameterEntityReference: name.
	replacement := self replacementFromParameterEntity: entity.

	replacement
		ifNotEmpty: [
			self
				pushBackReplacement: replacement
				forEntity: entity
				onClose: [self endReplacementForParameterEntity: entity]].! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/13/2013 20:09'!
nextPrologToken
	streamReader skipSeparators.
	streamReader atEnd
		ifTrue: [^ self].

	self expectNext: $<.
	streamReader peek == $?
		ifTrue: [^ self nextPIOrXMLDecl].
	streamReader peek == $!!
		ifTrue: [
			streamReader next.
			streamReader peek == $-
				ifTrue: [^ self nextComment].
			^ self nextDocTypeDecl].

	context enterContent.
	self nextTag.! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/9/2013 06:42'!
nextPubidLiteral
	| quote fpi nextChar |

	quote := self expectQuote.
	fpi := streamWriter writeWith: [:writeStream |
		[(nextChar := streamReader peek) notNil
			and: [(PubidChars includes: nextChar)
				and: [nextChar ~~ quote]]]
			whileTrue: [writeStream nextPut: streamReader next].

		writeStream contents].
	self expectNext: quote.

	^ fpi.! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:20'!
nextSubsetDeclaration
	streamReader peek == $E
		ifTrue: [
			streamReader next.
			^ streamReader peek == $N
				ifTrue: [self nextEntityDeclaration]
				ifFalse: [self nextElementDeclaration]].
	streamReader peek == $A
		ifTrue: [^ self nextAttlistDeclaration].
	streamReader peek == $N
		ifTrue: [^ self nextNotationDeclaration].

	self errorExpected: 'declaration'.! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/29/2013 05:52'!
nextSubsetMarkupToken
	self expectNext: $<.
	streamReader peek == $?
		ifTrue: [^ self nextPIOrXMLDecl]
		ifFalse: [
			context isInExternalSubsetTextDeclaration
				ifTrue: [context enterExternalSubset]].

	self expectNext: $!!.
	streamReader peek == $-
		ifTrue: [^ self nextComment].

	(streamReader peek == $[)
		ifTrue: [
			(context isInExternalSubset
				or: [self hasActiveExternalEntity])
				ifTrue: [^ self nextConditionalSection].
			self parseError: 'Cannot have INCLUDE/IGNORE sections in internal subset'].

	self nextSubsetDeclaration.! !

!XMLTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/6/2013 08:36'!
nextSystemLiteralRequired: aBoolean	
	| quote value |

	(aBoolean or: [streamReader atQuote])
		ifTrue: [
			quote := self expectQuote.
			value := streamReader upTo: quote]
		ifFalse: [value := ''].
	^ value.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 22:11'!
nextTag
	| tagName attributes namespaces nextChar attributeName attributeValue |

	(streamReader peek = $/)
		ifTrue: [^ self nextEndTag].
	tagName := self nextName.

	attributes := OrderPreservingDictionary new: 10 withDefaultValue: ''.
	[streamReader skipSeparators.
	((nextChar := streamReader peek) isNil
		or: [nextChar == $>
			or: [nextChar == $/]])]
		whileFalse: [
			attributeName := self nextName.
			streamReader skipSeparators.
			self expectNext: $=.
			streamReader skipSeparators.
			attributeValue := self nextAttributeValue.
	
			((attributeName at: 1) == $x "fast test to skip the others"
				and: [driver usesNamespaces
					and: [attributeName beginsWith: 'xmlns']])
				ifTrue: [
					namespaces
						ifNil: [namespaces := OrderPreservingDictionary new: 5 withDefaultValue: ''].
					attributeName size > 6
						ifTrue: [
							namespaces
								at: (attributeName copyFrom: 7 to: attributeName size)
								put: attributeValue]
						ifFalse: [namespaces at: '' put: attributeValue]]
				ifFalse: [attributes at: attributeName put: attributeValue]].

	driver
		handleStartTag: tagName
		attributes: attributes
		namespaces: namespaces.

	streamReader peek == $/
		ifTrue: [
			streamReader next.
			driver handleEndTag: tagName].
	self expectTerminator: $>.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 07:03'!
nextXMLEncodingAttributeValue
	| quote |

	^ streamWriter writeWith: [:writeStream |
		quote := self expectQuote.

		writeStream nextPut: self expectLetter.
		[streamReader atEnd not
			and: [EncodingNameChars includes: streamReader peek]]
			whileTrue: [writeStream nextPut: streamReader next].

		self expectNext: quote.
		writeStream contents]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:44'!
nextXMLStandaloneAttributeValue
	| quote yesOrNo |

	quote := self expectQuote.
	yesOrNo := 
		streamReader peek == $y
			ifTrue: [self expectNextAll: 'yes']
			ifFalse: [self expectNextAll: 'no'].
	self expectNext: quote.

	^ yesOrNo.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:40'!
nextXMLVersionAttributeValue
	| quote |

	^ streamWriter writeWith: [:writeStream |
		quote := self expectQuote.

		writeStream
			nextPut: (self expectNext: $1);
			nextPut: (self expectNext: $.);
			nextPut: self expectDigit.

		[streamReader atEnd
			or: [streamReader peek == quote]]
			whileFalse: [writeStream nextPut: self expectDigit].

		self expectNext: quote.
		writeStream contents]! !

!XMLTokenizer methodsFor: 'errors' stamp: 'monty 12/7/2013 01:15'!
parseError: anErrorString
	driver handleParseError: anErrorString! !

!XMLTokenizer methodsFor: 'entity replacement' stamp: 'monty 1/4/2014 07:43'!
pushBackReplacement: aReplacement forEntity: anEntity onClose: aBlock
	(self hasActiveEntityNamed: anEntity name)
		ifTrue: [self parseError: 'Illegal self-referential entity ', anEntity asReference].
	(self activeEntities size >= self maxEntityReplacementDepth)
		ifTrue: [self entityDepthError: 'Cannot further nest entity replacements'].

	self addActiveEntity: anEntity.
	streamReader
		pushStream: aReplacement readStream
		onClose: aBlock.

	anEntity isExternal
		ifTrue: [
			self
				checkTextDeclarationInReplacement: aReplacement
				forEntity: anEntity].! !

!XMLTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/13/2013 19:54'!
removeActiveEntity: anEntity
	self activeEntities remove: anEntity.

	driver activeEntityURI:
		(self activeEntities
			ifNotEmpty: [self activeEntities last uri]
			ifEmpty: [nil]).! !

!XMLTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/11/2013 01:50'!
replacementFromGeneralEntity: anEntity
	anEntity ifNil: [^ ''].

	(context isInLiteralValue and: [anEntity isExternal])
		ifTrue: [self parseError: 'External general entity ', anEntity asReference, ' in literal context'].

	^ context isInLiteralValue
		ifTrue: [anEntity replacementForLiteralContext]
		ifFalse: [anEntity replacementForContentContext].! !

!XMLTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/11/2013 01:50'!
replacementFromParameterEntity: anEntity
	anEntity ifNil: [^ ''].

	^ context isInLiteralValue
		ifTrue: [anEntity replacementForLiteralContext]
		ifFalse: [anEntity replacementForDTDContext]! !

!XMLTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:36'!
tokenContextClass
	^ XMLTokenContext! !

Object subclass: #XMLElementNester
	instanceVariableNames: 'openElements rootElement hasExternalRoot'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLElementNester commentStamp: '<historical>' prior: 0!
This is a base class for element nesters.!

XMLElementNester subclass: #HTMLElementNester
	instanceVariableNames: 'openElementsByName elementsToStart elementsToEnd'
	classVariableNames: 'BodySectionElements HeadSectionElements MutuallyExcludingTags NestableTagGroups NonNestableElements VoidElements'
	poolDictionaries: ''
	category: 'XML-Parser-HTML'!

!HTMLElementNester commentStamp: '<historical>' prior: 0!
This class can handle the nesting of HTML elements missing closing tags or with opening and closing tags in the wrong order.!

!HTMLElementNester class methodsFor: 'class initialization' stamp: 'monty 12/19/2013 07:27'!
initialize
	"self initialize"

	super initialize.
	self
		initializeHeadSectionElements;
		initializeBodySectionElements;
		initializeNonNestableElements;
		initializeNestableTagGroups;
		initializeMutuallyExcludingTags;
		initializeVoidElements! !

!HTMLElementNester class methodsFor: 'class initialization' stamp: 'monty 11/30/2013 07:08'!
initializeBodySectionElements
	BodySectionElements := Set withAll:
		#('a' 'abbr' 'acronym' 'address' 'applet' 'area' 'article' 'aside' 'audio' 'b' 'basefont'
			'bdi' 'bdo' 'big' 'blockquote' 'br' 'button' 'canvas' 'caption' 'center' 'cite' 'code'
			'col' 'colgroup' 'command' 'data' 'datagrid' 'datalist' 'dd' 'del' 'details' 'dfn' 'dir'
			'div' 'dl' 'dt' 'em' 'embed' 'eventsource' 'fieldset' 'figcaption' 'figure' 'font'
			'footer' 'form' 'frame' 'frameset' 'h1' 'h2' 'h3' 'h4' 'h5' 'h6' 'header' 'hgroup' 'hr'
			'i' 'iframe' 'img' 'input' 'ins' 'kbd' 'keygen' 'label' 'legend' 'li' 'map' 'mark' 'menu'
			'meter' 'nav' 'noframes' 'noscript' 'object' 'ol' 'optgroup' 'option' 'output' 'p'
			'param' 'pre' 'progress' 'q' 'rp' 'rt' 'ruby' 's' 'samp' 'script' 'section' 'select' 'small'
			'source' 'span' 'strike' 'strong' 'style' 'sub' 'summary' 'sup' 'table' 'tbody' 'td'
			'textarea' 'tfoot' 'th' 'thead' 'time' 'tr' 'track' 'tt' 'u' 'ul' 'var' 'video' 'wbr')! !

!HTMLElementNester class methodsFor: 'class initialization' stamp: 'monty 11/30/2013 07:53'!
initializeHeadSectionElements
	HeadSectionElements :=  Set withAll:
		#('base' 'isindex' 'link' 'meta' 'noscript' 'object' 'script' 'style' 'template' 'title')! !

!HTMLElementNester class methodsFor: 'class initialization' stamp: 'monty 11/30/2013 06:35'!
initializeMutuallyExcludingTags
	MutuallyExcludingTags := Dictionary new.
	#(#('dd' 'dt') #('thead' 'tbody') #('th' 'td') #('rp' 'rt'))
		do: [:tagList |
			tagList do: [:tag |
				tagList do: [:each |
					(each = tag)
						ifFalse: [MutuallyExcludingTags at: each put: tag]]]]
! !

!HTMLElementNester class methodsFor: 'class initialization' stamp: 'monty 11/30/2013 07:26'!
initializeNestableTagGroups
	NestableTagGroups := Dictionary new.

	#('dd' 'dt') do: [:each |
		NestableTagGroups at: each put: #('dl')].
	#('tr' 'th' 'td' 'thead' 'tfoot') do: [:each |
		NestableTagGroups at: each put: #('table')].
	#('li') do: [:each |
		NestableTagGroups at: each put: #('ul' 'ol' 'menu' 'dir')].
	#('rp' 'rt') do: [:each |
		NestableTagGroups at: each put: #('ruby')].! !

!HTMLElementNester class methodsFor: 'class initialization' stamp: 'monty 11/30/2013 07:12'!
initializeNonNestableElements
	NonNestableElements := Set withAll:
		#('a' 'applet' 'audio' 'button' 'footer' 'form' 'header' 'label' 'map'
			'meter' 'optgroup' 'option' 'ruby' 'select' 'video')! !

!HTMLElementNester class methodsFor: 'class initialization' stamp: 'monty 11/30/2013 06:48'!
initializeVoidElements
	VoidElements :=
		Set withAll:
			#('area' 'base' 'br' 'col' 'command' 'embed' 'hr' 'img' 'input' 'keygen'
				'link' 'menuitem' 'meta' 'param' 'source' 'track' 'wbr')! !

!HTMLElementNester methodsFor: 'testing' stamp: 'monty 11/30/2013 07:14'!
canNestWithinSelf: anElementName
	^ (NonNestableElements includes: anElementName) not! !

!HTMLElementNester methodsFor: 'private' stamp: 'monty 11/30/2013 04:21'!
elementsToEnd
	^ elementsToEnd ifNil: [elementsToEnd := OrderedCollection new]! !

!HTMLElementNester methodsFor: 'checking' stamp: 'monty 11/30/2013 04:37'!
elementsToEndBeforeEnding: anElementName
	self elementsToEnd reset.

	self openElements reverseDo: [:each |
		each = anElementName
			ifTrue: [^ self elementsToEnd]
			ifFalse: [self elementsToEnd addLast: each]].

	^ self elementsToEnd.! !

!HTMLElementNester methodsFor: 'checking' stamp: 'monty 11/30/2013 04:21'!
elementsToEndBeforePCData
	(self hasOpenElement: 'head')
		ifFalse: [^ #()].

	^ (self elementsToEndBeforeEnding: 'head')
		addLast: 'head';
		yourself.! !

!HTMLElementNester methodsFor: 'checking' stamp: 'monty 11/30/2013 19:33'!
elementsToEndBeforeStarting: anElementName
	| excludingElement |

	((anElementName = 'body' or: [anElementName = 'head'])
		or: [self headOrBodyCannotContain: anElementName])
		ifTrue: [^ self elementsToEndBeforeEnding: 'html'].

	self elementsToEnd reset.
	(self canNestWithinSelf: anElementName)
		ifTrue: [^ self elementsToEnd].

	excludingElement := self excludingElementFor: anElementName.
	((self hasOpenElement: anElementName)
		or: [excludingElement notEmpty
			and: [self hasOpenElement: excludingElement]])
		ifFalse: [^ self elementsToEnd].

	self openElements reverseDo: [:each |
		((NestableTagGroups includesKey: anElementName)
			and: [(NestableTagGroups at: anElementName) includes: each])
			ifTrue: [^ #()].

		self elementsToEnd addLast: each.
		((each = anElementName)
			or: [excludingElement notEmpty
				and: [excludingElement = each]])
			ifTrue: [^ self elementsToEnd]].

	^ self elementsToEnd.
	! !

!HTMLElementNester methodsFor: 'private' stamp: 'monty 11/30/2013 04:21'!
elementsToStart
	^ elementsToStart ifNil: [elementsToStart := OrderedCollection new]! !

!HTMLElementNester methodsFor: 'checking' stamp: 'monty 11/30/2013 04:37'!
elementsToStartBeforePCData
	self elementsToStart reset.

	self hasOpenElements
		ifFalse: [self elementsToStart addLast: 'html'].
	(self hasOpenElement: 'body')
		ifFalse: [self elementsToStart addLast: 'body'].

	^ self elementsToStart.! !

!HTMLElementNester methodsFor: 'checking' stamp: 'monty 12/1/2013 01:53'!
elementsToStartBeforeStarting: anElementName
	self elementsToStart reset.

	self hasOpenElements
		ifFalse: [
			(anElementName = 'html')
				ifTrue: [^ self elementsToStart].
			self elementsToStart addLast: 'html'].

	(anElementName = 'body' or: [anElementName = 'head'])
		ifTrue: [^ self elementsToStart]. 

	((self isHeadSectionOnlyElement: anElementName)
		and: [(self hasOpenElement: 'head') not])
			ifTrue: [self elementsToStart addLast: 'head']
		ifFalse: [
			((self isBodySectionOnlyElement: anElementName)
				or: [(self hasOpenElement: 'head') not])
				ifTrue: [
					(self hasOpenElement: 'body')
						ifFalse: [self elementsToStart addLast: 'body']]].

	^ self elementsToStart.! !

!HTMLElementNester methodsFor: 'checking' stamp: 'monty 12/11/2013 21:43'!
endElement: anElementName
	self openElementsByName
		at: anElementName
		put:
			((self openElementsByName
				at: anElementName
				ifAbsent: [1]) - 1 max: 0).

	super endElement: anElementName.! !

!HTMLElementNester methodsFor: 'checking' stamp: 'monty 12/11/2013 21:43'!
ensureNoElementsRemainOpen
	self openElements copy reverseDo: [:each | self endElement: each].! !

!HTMLElementNester methodsFor: 'accessing' stamp: 'monty 11/22/2013 21:37'!
excludingElementFor: anElementName
	^ MutuallyExcludingTags at: anElementName ifAbsent: ['']

	! !

!HTMLElementNester methodsFor: 'testing' stamp: 'monty 11/22/2013 22:36'!
hasOpenElement: anElementName
	^ (self openElementsByName at: anElementName ifAbsent: [0]) > 0! !

!HTMLElementNester methodsFor: 'testing' stamp: 'monty 12/1/2013 01:31'!
headOrBodyCannotContain: anElementName
	^ ((self hasOpenElement: 'head')
		and: [self isBodySectionOnlyElement: anElementName])
			or: [(self hasOpenElement: 'body')
				and: [self isHeadSectionOnlyElement: anElementName]]! !

!HTMLElementNester methodsFor: 'testing' stamp: 'monty 12/1/2013 01:31'!
isBodySectionOnlyElement: anElementName
	^ (BodySectionElements includes: anElementName)
		and: [(HeadSectionElements includes: anElementName) not]! !

!HTMLElementNester methodsFor: 'testing' stamp: 'monty 12/1/2013 01:30'!
isHeadSectionOnlyElement: anElementName
	^ (HeadSectionElements includes: anElementName)
		and: [(BodySectionElements includes: anElementName) not]! !

!HTMLElementNester methodsFor: 'testing' stamp: 'monty 11/30/2013 18:14'!
isOpenUniqueElement: anElementName
	^ (#('html' 'head' 'body') includes: anElementName)
		and: [self hasOpenElement: anElementName]! !

!HTMLElementNester methodsFor: 'testing' stamp: 'monty 11/30/2013 07:36'!
isVoidElement: anElementName
	^ VoidElements includes: anElementName! !

!HTMLElementNester methodsFor: 'private' stamp: 'monty 11/22/2013 18:53'!
openElementsByName
	^ openElementsByName ifNil: [openElementsByName := Dictionary new].! !

!HTMLElementNester methodsFor: 'checking' stamp: 'monty 12/11/2013 21:43'!
startElement: anElementName
	self openElementsByName
		at: anElementName
		put:
			(self openElementsByName
				at: anElementName
				ifAbsent: [0]) + 1.

	super startElement: anElementName.! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 12/6/2013 17:25'!
currentElement
	^ self openElements
		at: self totalOpen
		ifAbsent: ['']! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 12/6/2013 17:25'!
enclosedElementsAsString: aTagName
	| enclosedTags |

	enclosedTags := String new writeStream.
	self
		printOn: enclosedTags
		startingAt: (self openElements lastIndexOf: aTagName) + 1.
	^ enclosedTags contents.! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 12/11/2013 21:43'!
endElement: anElementName
	self openElements removeLast! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 12/6/2013 17:49'!
ensureNoElementsRemainOpen
	self subclassResponsibility! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 12/6/2013 17:30'!
hasExternalRoot
	^ hasExternalRoot ifNil: [hasExternalRoot := false]! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 12/6/2013 17:29'!
hasOpenElement: anElementName
	^ self openElements includes: anElementName! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 12/6/2013 17:29'!
hasOpenElements
	^ self openElements notEmpty! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 12/6/2013 17:29'!
openElements
	^ openElements ifNil: [openElements := OrderedCollection new: 15]! !

!XMLElementNester methodsFor: 'private' stamp: 'monty 12/6/2013 17:29'!
printLimit
	^ 5! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 12/6/2013 17:29'!
printOn: aStream
	self printOn: aStream startingAt: 1! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 12/6/2013 17:29'!
printOn: aStream startingAt: aPosition
	| endPosition |

	endPosition :=  self totalOpen min: (aPosition + self printLimit - 1).
	(self openElements copyFrom: aPosition to: endPosition)
		do: [:each |
			aStream
				nextPut: $<;
				nextPutAll: each;
				nextPut: $>]
		separatedBy: [aStream nextPutAll: ', '].
	(endPosition < self totalOpen)
		ifTrue: [aStream nextPutAll: '...'].! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 12/6/2013 17:28'!
rootElement
	^ rootElement ifNil: ['']! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 12/6/2013 17:28'!
rootElement: anElement
	rootElement := anElement.
	hasExternalRoot := true.! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 12/11/2013 21:43'!
startElement: anElementName
	self openElements addLast: anElementName! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 12/6/2013 17:29'!
totalOpen
	^ self openElements size
	! !

XMLElementNester subclass: #XMLWellFormedElementNester
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLWellFormedElementNester commentStamp: '<historical>' prior: 0!
The class keeps track of elements as they start and end and ensures that they nest properly.!

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 12/11/2013 21:43'!
endElement: anElementName
	(self currentElement = anElementName)
		ifFalse: [
			^ (self hasOpenElement: anElementName)
				ifTrue: [self errorImproperlyNestedElements: anElementName]
				ifFalse: [self errorUnmatchedEndTag: anElementName]].

	super endElement: anElementName.! !

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'JAAyer 11/28/2010 07:03'!
ensureNoElementsRemainOpen
	self hasOpenElements
		ifTrue: [XMLWellFormednessException signal: 'Unterminated elements: ', self printString]! !

!XMLWellFormedElementNester methodsFor: 'errors' stamp: 'JAAyer 11/28/2010 07:03'!
errorImproperlyNestedElements: anElementName
	XMLWellFormednessException signal:
		('<{1}> element ended before its child {2} element(s)'
			format: (Array with: anElementName with: (self enclosedElementsAsString: anElementName)))! !

!XMLWellFormedElementNester methodsFor: 'errors' stamp: 'JAAyer 11/28/2010 07:03'!
errorUnmatchedEndTag: anElementName
	XMLWellFormednessException signal:
		('End </{1}> tag with no corresponding start <{1}> tag'
			format: (Array with: anElementName))! !

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 12/11/2013 21:43'!
startElement: anElementName
	rootElement
		ifNil: [rootElement := anElementName]
		ifNotNil: [
			(self hasOpenElements or: [self hasExternalRoot])
				ifFalse: [
					XMLWellFormednessException signal:
						'Document cannot have multiple root elements']].

	super startElement: anElementName.! !

Object subclass: #XMLElementValidator
	instanceVariableNames: 'element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLElementValidator commentStamp: '<historical>' prior: 0!
This is a base class for element content validators.!

XMLElementValidator subclass: #XMLAnyElementValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAnyElementValidator commentStamp: '<historical>' prior: 0!
This class validates elements that can contain any content.!

!XMLAnyElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:42'!
atEnd
	^ true! !

XMLElementValidator subclass: #XMLDFAElementValidator
	instanceVariableNames: 'dfa currentState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDFAElementValidator commentStamp: '<historical>' prior: 0!
This class can use a DFA to validate element content.
!

!XMLDFAElementValidator class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:47'!
element: aName dfa: aDfa
	^ self new
		setElement: aName
		dfa: aDfa! !

!XMLDFAElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 05:43'!
atEnd
	^ currentState isAccepting! !

!XMLDFAElementValidator methodsFor: 'copying' stamp: 'monty 11/18/2013 05:41'!
postCopy
	currentState := dfa! !

!XMLDFAElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:47'!
setElement: aName dfa: aDfa
	element := aName.
	dfa := aDfa.
	currentState := dfa.! !

!XMLDFAElementValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 00:03'!
validatePCData: aString
	self errorUnexpectedPCData: aString! !

!XMLDFAElementValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 00:21'!
validateStartTag: aName
	currentState :=
		currentState
			detectNextStateMatching: aName
			ifNone: [self errorUnexpectedElement: aName]! !

!XMLElementValidator class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:48'!
element: aName
	^ self new element: aName! !

!XMLElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:41'!
atEnd
	self subclassResponsibility! !

!XMLElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:48'!
element
	^ element! !

!XMLElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:48'!
element: aName
	element := aName! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 12/7/2013 19:44'!
errorUnexpectedElement: aName
	XMLValidationException signal:
		'Unexpected <', aName, '> element in <', self element, '> element'.! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 12/7/2013 19:44'!
errorUnexpectedPCData: aString
	XMLValidationException signal:
		'Unexpected PCDATA in <', self element, '> element'.! !

!XMLElementValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 05:24'!
validatePCData: aString! !

!XMLElementValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 05:24'!
validateStartTag: aName! !

XMLElementValidator subclass: #XMLEmptyElementValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEmptyElementValidator commentStamp: '<historical>' prior: 0!
This class validates empty elements.
!

!XMLEmptyElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:41'!
atEnd
	^ true! !

!XMLEmptyElementValidator methodsFor: 'private' stamp: 'monty 12/7/2013 19:44'!
errorMustBeEmpty
	XMLValidationException signal: 'Element <', self element, '> must be empty'! !

!XMLEmptyElementValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 04:32'!
validatePCData: aString
	self errorMustBeEmpty! !

!XMLEmptyElementValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 04:34'!
validateStartTag: aName
	self errorMustBeEmpty! !

XMLElementValidator subclass: #XMLMixedContentElementValidator
	instanceVariableNames: 'allowedElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLMixedContentElementValidator commentStamp: '<historical>' prior: 0!
This class validates elements that can contain a mix of PCDATA and certain elements.!

!XMLMixedContentElementValidator class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:50'!
element: aName allowedElements: aNameCollection
	^ self new
		setElement: aName
		allowedElements: aNameCollection! !

!XMLMixedContentElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:51'!
allowedElements
	^ allowedElements! !

!XMLMixedContentElementValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 01:14'!
atEnd
	^ true! !

!XMLMixedContentElementValidator methodsFor: 'initialization' stamp: 'monty 12/7/2013 19:50'!
setElement: aName allowedElements: aNameCollection
	element := aName.
	allowedElements := aNameCollection.! !

!XMLMixedContentElementValidator methodsFor: 'validation' stamp: 'monty 12/7/2013 19:51'!
validateStartTag: aName
	(self allowedElements includes: aName)
		ifFalse: [self errorUnexpectedElement: aName]
	! !

Object subclass: #XMLFAState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

XMLFAState subclass: #XMLDFAState
	instanceVariableNames: 'nfaStates transitions dfaStateCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLDFAState commentStamp: 'monty 12/28/2013 22:34' prior: 0!
This class is a DFA state made by grouping one or more NFA states.!

!XMLDFAState class methodsFor: 'instance creation' stamp: 'monty 12/29/2013 01:11'!
nfaStates: anNfaStateSet
	^ self
		nfaStates: anNfaStateSet
		dfaStateCache: nil! !

!XMLDFAState class methodsFor: 'instance creation' stamp: 'monty 12/29/2013 01:09'!
nfaStates: anNfaStateSet dfaStateCache: aDfaStateCache
	^ self new
		setNFAStates: anNfaStateSet
		dfaStateCache: aDfaStateCache! !

!XMLDFAState methodsFor: 'enumerating' stamp: 'monty 12/28/2013 00:21'!
detectNextStateMatching: aMatcher
	^ self detectNextStateMatching: aMatcher ifNone: [nil]! !

!XMLDFAState methodsFor: 'enumerating' stamp: 'monty 12/28/2013 19:54'!
detectNextStateMatching: aMatcher ifNone: aNoneBlock
	^ (self transitions
		at: aMatcher
		ifAbsentPut: [self dfaStateMatching: aMatcher])
			ifNil: [aNoneBlock value]! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/29/2013 03:33'!
dfaStateMatching: aMatcher
	| matchingNfaStates |

	matchingNfaStates := XMLNFAStateSet new.
	nfaStates do: [:state |
		(state matches: aMatcher)
			ifTrue: [state nextState addTo: matchingNfaStates]].

	^ matchingNfaStates
		ifNotEmpty: [dfaStateCache dfaStateFor: matchingNfaStates]
		ifEmpty: [nil].! !

!XMLDFAState methodsFor: 'testing' stamp: 'monty 12/28/2013 18:54'!
isAccepting
	^ nfaStates isAccepting! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/29/2013 01:10'!
nfaStates
	^ nfaStates! !

!XMLDFAState methodsFor: 'initialization' stamp: 'monty 12/29/2013 03:25'!
setNFAStates: anNfaStateSet dfaStateCache: aDfaStateCache
	nfaStates := anNfaStateSet.
	dfaStateCache := (aDfaStateCache ifNil: [XMLDFAStateCache firstState: self]).! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/28/2013 01:09'!
transitions
	^ transitions ifNil: [transitions := Dictionary new]! !

XMLFAState subclass: #XMLDFAStateCache
	instanceVariableNames: 'firstState cachedStates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLDFAStateCache commentStamp: 'monty 12/29/2013 03:32' prior: 0!
This class uses a dictionary with NFAStateSet keys to cache DFA states.!

!XMLDFAStateCache class methodsFor: 'instance creation' stamp: 'monty 12/29/2013 01:02'!
firstState: aFirstState
	^ self new firstState: aFirstState! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 12/29/2013 03:29'!
clearCache
	cachedStates
		removeAll;
		at: firstState nfaStates
		put: firstState! !

!XMLDFAStateCache methodsFor: 'accessing' stamp: 'monty 12/29/2013 06:33'!
dfaStateFor: aStateSet
	^ cachedStates
		at: aStateSet
		ifAbsent: [
			cachedStates size >= self maxCachedStates
				ifTrue: [self clearCache].

			cachedStates
				at: aStateSet
				put: (self newDFAStateFor: aStateSet)].! !

!XMLDFAStateCache methodsFor: 'accessing' stamp: 'monty 12/29/2013 03:24'!
firstState: aFirstState
	firstState := aFirstState.

	(cachedStates := Dictionary new)
		at: firstState nfaStates
		put: firstState.! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 12/29/2013 01:07'!
maxCachedStates
	^ 32! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 12/29/2013 03:29'!
newDFAStateFor: aStateSet
	^ XMLDFAState
		nfaStates: aStateSet
		dfaStateCache: self.! !

!XMLFAState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:22'!
isAccepting
	^ false! !

XMLFAState subclass: #XMLNFAState
	instanceVariableNames: 'lastAddedTo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAState commentStamp: 'monty 12/28/2013 22:34' prior: 0!
This is a base class for NFA states.!

XMLNFAState subclass: #XMLNFAAcceptingState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAAcceptingState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:23'!
isAccepting
	^ true! !

XMLNFAState subclass: #XMLNFABranchingState
	instanceVariableNames: 'leftBranch rightBranch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFABranchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:43'!
branch: aLeftBranch
	^ self new
		setLeftBranch: aLeftBranch
		rightBranch: nil! !

!XMLNFABranchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 22:04'!
leftBranch: aLeftBranch rightBranch: aRightBranch
	^ self new
		setLeftBranch: aLeftBranch
		rightBranch: aRightBranch! !

!XMLNFABranchingState methodsFor: 'adding' stamp: 'monty 12/29/2013 01:10'!
addTo: aStateSet
	lastAddedTo == aStateSet
		ifFalse: [
			"This avoids getting caught in a loop"
			lastAddedTo := aStateSet.

			leftBranch addTo: aStateSet.
			rightBranch addTo: aStateSet]! !

!XMLNFABranchingState methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:45'!
connectToState: aState
	leftBranch ifNil: [leftBranch := aState].
	rightBranch ifNil: [rightBranch := aState].! !

!XMLNFABranchingState methodsFor: 'initialization' stamp: 'monty 12/28/2013 00:45'!
setLeftBranch: aLeftBranch rightBranch: aRightBranch
	leftBranch := aLeftBranch.
	rightBranch := aRightBranch.! !

XMLNFAState subclass: #XMLNFAMatchingState
	instanceVariableNames: 'matcher nextState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAMatchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 21:15'!
matcher: aMatcher
	^ self new matcher: aMatcher! !

!XMLNFAMatchingState methodsFor: 'connecting' stamp: 'monty 12/28/2013 21:52'!
connectToState: aState
	nextState := aState! !

!XMLNFAMatchingState methodsFor: 'accessing' stamp: 'monty 12/27/2013 21:16'!
matcher: aMatcher
	matcher := aMatcher! !

!XMLNFAMatchingState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:15'!
matches: anObject
	^ matcher = anObject! !

!XMLNFAMatchingState methodsFor: 'accessing' stamp: 'monty 12/27/2013 21:13'!
nextState
	^ nextState! !

!XMLNFAState methodsFor: 'adding' stamp: 'monty 12/29/2013 01:10'!
addTo: aStateSet
	lastAddedTo == aStateSet
		ifFalse: [
			"This avoids getting caught in a loop"
			lastAddedTo := aStateSet.

			aStateSet add: self]! !

!XMLNFAState methodsFor: 'converting' stamp: 'monty 12/28/2013 22:33'!
asDFAState
	| nfaStates |

	nfaStates := XMLNFAStateSet new.
	self addTo: nfaStates.

	^ XMLDFAState nfaStates: nfaStates.! !

!XMLNFAState methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:45'!
connectToState: aState! !

!XMLNFAState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:10'!
matches: anObject
	^ false! !

Object subclass: #XMLIOClient
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Clients'!

!XMLIOClient commentStamp: '<historical>' prior: 0!
A base class for IO clients used to retrieve external entities. The get: methods return readable streams.!

XMLIOClient subclass: #XMLFileClient
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Clients'!

!XMLFileClient commentStamp: '<historical>' prior: 0!
A base class for file IO clients. Messages expect and return paths as strings. See the FileSystem implementation for an example.!

!XMLFileClient class methodsFor: 'accessing' stamp: 'monty 11/16/2013 03:18'!
defaultImplementation
	^ XMLFileSystemClient! !

!XMLFileClient methodsFor: 'paths' stamp: 'monty 11/13/2013 23:26'!
currentWorkingDirectory
	"should return cwd as a string"
	self subclassResponsibility! !

!XMLFileClient methodsFor: 'retrieving' stamp: 'monty 11/13/2013 23:22'!
get: aFullPath
	^ self
		get: aFullPath
		onError: [:error | XMLFileIOException signal: error messageText]! !

!XMLFileClient methodsFor: 'retrieving' stamp: 'monty 11/13/2013 23:13'!
get: aFullPath onError: aBlock
	self subclassResponsibility! !

!XMLFileClient methodsFor: 'paths' stamp: 'monty 12/13/2013 04:10'!
pathToFile: aFileName inDirectory: aDirectoryPath
	^ (aDirectoryPath copyWith: self pathSeparator), aFileName! !

XMLFileClient subclass: #XMLFileSystemClient
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Clients'!

!XMLFileSystemClient commentStamp: '<historical>' prior: 0!
This uses the FileSystem API to implement file IO, but does not refer to FileSystem classes directly to avoid compile errors on platforms without FileSystem.
!

!XMLFileSystemClient methodsFor: 'paths' stamp: 'monty 12/7/2013 06:43'!
combineRelative: aRelativePath withBase: aBasePath
	| basePathObject fullPathString |

	(self isNonRelative: aRelativePath)
		ifTrue: [^ aRelativePath].

	basePathObject := self pathObjectFrom: aBasePath.
	fullPathString :=
		(basePathObject isRoot or: [aBasePath last == self pathSeparator])
			ifTrue: [self pathToString: (basePathObject / (self trimRelativePath: aRelativePath))]
			ifFalse: [
				self pathSeparator join:
					(basePathObject segments allButLast
						copyWith: (self trimRelativePath: aRelativePath))].
	^ (self isNonRelative: aBasePath)
		ifTrue: [self makeNonRelative: fullPathString]
		ifFalse: [fullPathString].! !

!XMLFileSystemClient methodsFor: 'paths' stamp: 'monty 12/7/2013 06:42'!
currentWorkingDirectory
	^ self makeNonRelative:
		(self pathToString: (Smalltalk at: #FileSystem) workingDirectory path)! !

!XMLFileSystemClient methodsFor: 'retrieving' stamp: 'monty 12/7/2013 06:39'!
get: aFullPath onError: aBlock
	^ [(self pathObjectFrom: aFullPath) asFileReference readStream]
		on: (Smalltalk at: #FileSystemError)
		do: aBlock! !

!XMLFileSystemClient methodsFor: 'testing' stamp: 'monty 12/7/2013 06:39'!
isNonRelative: aPathString
	^ (self pathObjectFrom: aPathString) isRelative not! !

!XMLFileSystemClient methodsFor: 'private' stamp: 'monty 12/7/2013 06:43'!
makeNonRelative: aPathString
	^ aPathString copyWithFirst: self pathSeparator! !

!XMLFileSystemClient methodsFor: 'private' stamp: 'monty 12/7/2013 06:38'!
pathObjectFrom: aPathString
	^ (Smalltalk at: #Path) from: aPathString! !

!XMLFileSystemClient methodsFor: 'private' stamp: 'monty 12/7/2013 21:46'!
pathToString: aPathObject
	^ self pathSeparator join: aPathObject segments! !

!XMLFileSystemClient methodsFor: 'paths' stamp: 'monty 12/7/2013 06:37'!
trimRelativePath: aPath
	^ (aPath beginsWith: (String with: $. with: self pathSeparator))
		ifTrue: [aPath allButFirst: 2]
		ifFalse: [aPath]! !

XMLIOClient subclass: #XMLHTTPClient
	instanceVariableNames: 'timeout'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Clients'!

!XMLHTTPClient commentStamp: '<historical>' prior: 0!
A base class for HTTP clients. See the Zinc implementation for an example.!

!XMLHTTPClient class methodsFor: 'accessing' stamp: 'monty 11/16/2013 03:17'!
defaultImplementation
	^ XMLZincClient! !

!XMLHTTPClient methodsFor: 'testing' stamp: 'monty 11/19/2013 00:52'!
beginsWithSupportedURLProtocol: aUrl
	^ self supportedURLProtocols anySatisfy: [:each |
		aUrl asLowercase beginsWith: each, '://']! !

!XMLHTTPClient methodsFor: 'retrieving' stamp: 'monty 11/12/2013 23:17'!
get: aUrl
	^ self
		get: aUrl
		timeout: self timeout! !

!XMLHTTPClient methodsFor: 'retrieving' stamp: 'monty 11/12/2013 23:19'!
get: aUrl timeout: aTimeout
	^ self
		get: aUrl
		timeout: aTimeout
		onError: [:error | XMLHTTPException signal: error messageText]! !

!XMLHTTPClient methodsFor: 'retrieving' stamp: 'monty 11/12/2013 23:20'!
get: aUrl timeout: aTimeout onError: aBlock
	self subclassResponsibility! !

!XMLHTTPClient methodsFor: 'accessing' stamp: 'monty 11/19/2013 00:48'!
supportedURLProtocols
	^ #('http' 'file')! !

!XMLHTTPClient methodsFor: 'accessing' stamp: 'monty 11/12/2013 23:20'!
timeout
	^ timeout ifNil: [timeout := 30]! !

!XMLHTTPClient methodsFor: 'accessing' stamp: 'monty 11/12/2013 23:21'!
timeout: aNumber
	timeout := aNumber! !

XMLHTTPClient subclass: #XMLZincClient
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Clients'!

!XMLZincClient commentStamp: '<historical>' prior: 0!
This client uses Zinc-HTTP, but does not refer to Zinc classes directly to avoid compile errors on platforms without Zinc.!

!XMLZincClient methodsFor: 'paths' stamp: 'monty 11/19/2013 00:52'!
combineRelative: aRelativeUrl withBase: aBaseUrl
	| url urlClass |

	urlClass := Smalltalk at: #ZnUrl.
	(self beginsWithSupportedURLProtocol: aRelativeUrl)
		ifTrue: [^ (urlClass fromString: aRelativeUrl) printString].

	url := urlClass fromString: aBaseUrl.
	url isDirectoryPath
		ifFalse: [url queryRemoveAll].
	(self isNonRelative: aRelativeUrl)
		ifTrue: [url clearPath]
		ifFalse: [url removeLastPathSegment].
	^ (url / aRelativeUrl) printString.! !

!XMLZincClient methodsFor: 'retrieving' stamp: 'monty 11/30/2013 23:53'!
get: aUrl timeout: aTimeout onError: aBlock
	^ ((Smalltalk at: #ZnClient) new
		timeout: self timeout;
		ifFail: aBlock;
		url: aUrl;
		get) readStream! !

!XMLIOClient class methodsFor: 'accessing' stamp: 'monty 11/16/2013 03:16'!
defaultImplementation
	self subclassResponsibility! !

!XMLIOClient class methodsFor: 'retrieving' stamp: 'monty 11/16/2013 00:07'!
get: aResource
	^ self new get: aResource! !

!XMLIOClient methodsFor: 'paths' stamp: 'monty 11/13/2013 19:09'!
combineRelative: aRelativeResource withBase: aBaseResource
	self subclassResponsibility! !

!XMLIOClient methodsFor: 'retrieving' stamp: 'monty 11/13/2013 19:06'!
get: aResource
	self subclassResponsibility! !

!XMLIOClient methodsFor: 'testing' stamp: 'monty 11/16/2013 00:17'!
isNonRelative: aResource
	^ aResource notEmpty and: [aResource first == self pathSeparator]! !

!XMLIOClient methodsFor: 'paths' stamp: 'monty 11/16/2013 00:14'!
pathSeparator
	^ $/! !

Object subclass: #XMLNFABuilder
	instanceVariableNames: 'fragmentStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFABuilder commentStamp: '<historical>' prior: 0!
This class builds an NFA using a stack machine.!

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 01:26'!
applyBranch
	| branchState leftFragment  rightFragment |

	leftFragment := self popFragment.
	rightFragment := self popFragment.
	branchState :=
		XMLNFABranchingState
			leftBranch: leftFragment firstState
			rightBranch: rightFragment firstState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchState
			terminalStates: (leftFragment combinedTerminalStatesFrom: rightFragment)).! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 23:46'!
applyConcatenation
	| firstFragment secondFragment |

	secondFragment := self popFragment.
	firstFragment := self popFragment.

	firstFragment
		connectToFragment: secondFragment;
		terminalStates: secondFragment terminalStates.

	self pushFragment: firstFragment.! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 23:43'!
applyOptional
	| optionalFragment branchingState |

	optionalFragment := self popFragment.
	branchingState :=
		XMLNFABranchingState branch: optionalFragment firstState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchingState
			terminalStates: (optionalFragment terminalStatesWith: branchingState))! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 01:19'!
applyPlus
	| plusFragment branchState |

	plusFragment := self topFragment.
	branchState := XMLNFABranchingState branch: plusFragment firstState.
	plusFragment
		connectToState: branchState;
		terminalState: branchState.! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 23:55'!
applyStar
	| branchState starFragment |

	starFragment := self popFragment.
	branchState := XMLNFABranchingState branch: starFragment firstState.
	starFragment connectToState: branchState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchState
			terminalState: branchState)! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 22:51'!
firstStateOfAcceptingNFA
	| firstFragment |

	[self totalFragments > 1]
		whileTrue: [self applyConcatenation].

	firstFragment := self popFragment.
	firstFragment connectToState: XMLNFAAcceptingState new.

	^ firstFragment firstState.! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 14:16'!
fragmentStack
	^ fragmentStack ifNil: [fragmentStack := OrderedCollection new]! !

!XMLNFABuilder methodsFor: 'testing' stamp: 'monty 12/6/2013 06:40'!
hasFragments
	^ self fragmentStack notEmpty! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 08:01'!
popFragment
	^ self fragmentStack removeLast! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 08:01'!
pushFragment: aFragment
	self fragmentStack addLast: aFragment! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 21:41'!
pushMatcher: aMatcher
	| firstState |

	firstState := XMLNFAMatchingState matcher: aMatcher.
	self pushFragment:
		(XMLNFAFragment
			firstState: firstState
			terminalState: firstState).! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 11/17/2013 12:01'!
topFragment
	^ self fragmentStack last! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 15:04'!
totalFragments
	^ self fragmentStack size! !

Object subclass: #XMLNFAFragment
	instanceVariableNames: 'firstState terminalStates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAFragment commentStamp: 'monty 12/28/2013 21:51' prior: 0!
This is a fragment used to build an NFA by the NFABuilder.!

!XMLNFAFragment class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:50'!
firstState: aFirstState terminalState: aTerminalState
	^ self new
		firstState: aFirstState;
		terminalState: aTerminalState! !

!XMLNFAFragment class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:50'!
firstState: aFirstState terminalStates: aTerminalStateCollection
	^ self new
		firstState: aFirstState;
		terminalStates: aTerminalStateCollection! !

!XMLNFAFragment methodsFor: 'converting' stamp: 'monty 12/27/2013 23:14'!
combinedTerminalStatesFrom: aFragment
	^ terminalStates isArray
		ifTrue: [
			aFragment terminalStates asOrderedCollection
				addAll: terminalStates;
				yourself]
		ifFalse: [
			terminalStates
				addAll: aFragment terminalStates;
				yourself]! !

!XMLNFAFragment methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:46'!
connectToFragment: aFragment
	self connectToState: aFragment firstState! !

!XMLNFAFragment methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:46'!
connectToState: aState
	self terminalStates do: [:each | each connectToState: aState]! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 11/17/2013 10:48'!
firstState
	^ firstState! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 11/17/2013 11:51'!
firstState: aState
	firstState := aState! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 12/27/2013 23:50'!
terminalState: aTerminalState
	terminalStates := Array with: aTerminalState! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 12/27/2013 21:46'!
terminalStates
	^ terminalStates! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 12/27/2013 23:50'!
terminalStates: aTerminalStateCollection
	terminalStates := aTerminalStateCollection! !

!XMLNFAFragment methodsFor: 'converting' stamp: 'monty 12/27/2013 23:40'!
terminalStatesWith: aState
	^ terminalStates isArray
		ifTrue: [OrderedCollection with: terminalStates first with: aState]
		ifFalse: [
			terminalStates
				add: aState;
				yourself]! !

Object subclass: #XMLNamespaceScope
	instanceVariableNames: 'defaultNamespace prefixMappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScope commentStamp: '<historical>' prior: 0!
A namespace scope is a possibly empty set of mappings from namespace prefixes to URIs and an optional default namespace URI to be used for unprefixed elements. Scopes can be created from other scopes by sending #enclosingScope: to the class with the other scope as the argument; the new scope will inherit the mappings from the parent.!

!XMLNamespaceScope class methodsFor: 'instance creation' stamp: 'JAAyer 3/5/2010 09:50'!
enclosingScope: aScope
	^ self new inheritMappingsFrom: aScope! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 12:19'!
defaultNamespace
	"Rather than lazily initializing this to an empty string, we leave it as nil, so we can
	distinguish between no default namespace being set and an empty default namespace
	intended to override a non-empty declaration in a parent."

	^ defaultNamespace ifNil: ['']! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 07:41'!
defaultNamespace: aUri
	aUri = self xmlNamespaceURI
		ifTrue: [XMLNamespaceException signal: 'Cannot use "xml" namespace URI as default namespace'].
	aUri = self xmlnsNamespaceURI
		ifTrue: [XMLNamespaceException signal: 'Cannot use "xmlns" namespace URI as default namespace'].

	defaultNamespace := aUri.! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'JAAyer 11/5/2010 22:30'!
ensureAttributePrefix: aPrefix localName: aLocalName hasNoAliasIn: aDictionary
	| namespaceUri aliasingAttributeName |

	namespaceUri := self resolvePrefix: aPrefix.

	(self prefixesAliasing: aPrefix) do: [:aliasingPrefix |
		aliasingAttributeName := aliasingPrefix, ':', aLocalName.
		(aDictionary includesKey: aliasingAttributeName)
			ifTrue: [
				self
					errorAttribute: aliasingAttributeName
					aliases: aPrefix, ':', aLocalName
					with: namespaceUri]].! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'JAAyer 6/20/2010 23:17'!
errorAttribute: anAttribute aliases: anAliasedAttribute with: aUri
	XMLNamespaceException signal:
		('Attribute "{1}" aliases attribute "{2}"; both prefixes map to {3}'
			format: (Array with: anAttribute with: anAliasedAttribute with: aUri))! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 11/23/2010 07:48'!
hasDefaultNamespace
	^ defaultNamespace notNil! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 8/5/2010 01:24'!
hasNamespaces
	^ self hasDefaultNamespace or: [self hasPrefixMappings]! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 7/26/2010 18:17'!
hasPrefixMappings
	^ prefixMappings notNil and: [prefixMappings notEmpty]! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 11/6/2010 01:43'!
includesPrefix: aPrefix
	^ self prefixMappings includesKey: aPrefix! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 8/8/2010 14:43'!
includesPrefix: aPrefix mappedTo: aUri
	^ (self includesPrefix: aPrefix)
		and: [(self resolvePrefix: aPrefix) = aUri]! !

!XMLNamespaceScope methodsFor: 'initializing' stamp: 'JAAyer 11/24/2010 12:18'!
inheritMappingsFrom: anEnclosingScope
	anEnclosingScope hasNamespaces
		ifFalse: [^ self].

	(anEnclosingScope defaultNamespace notEmpty and: [self hasDefaultNamespace not])
		ifTrue: [self defaultNamespace: anEnclosingScope defaultNamespace].

	anEnclosingScope hasPrefixMappings
		ifTrue: [
			anEnclosingScope prefixesAndURIsDo: [:prefix :uri |
				(self includesPrefix: prefix)
					ifFalse: [self mapPrefix: prefix to: uri]]].! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 06:59'!
mapPrefix: aPrefix to: aUri
	aPrefix isEmptyOrNil
		ifTrue: [^ self defaultNamespace: aUri].
	aUri isEmptyOrNil
		ifTrue: [XMLNamespaceException signal: 'Cannot clear prefix by mapping it to nil or an empty string.'].

	aPrefix = 'xml'
		ifTrue: [
			aUri = self xmlNamespaceURI
				ifFalse: [
					XMLNamespaceException signal:
						'Cannot rebind the "xml" prefix to a different namespace URI'].
			^ self].

	aPrefix = 'xmlns'
		ifTrue: [XMLNamespaceException signal: 'Cannot bind "xmlns" prefix at all'].

	(aUri = self xmlNamespaceURI or: [aUri = self xmlnsNamespaceURI])
		ifTrue: [XMLNamespaceException signal: 'Cannot alias the "xml" or "xmlns" prefixes'].

	self prefixMappings at: aPrefix put: aUri.! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 12/6/2010 22:55'!
prefixMappings
	^ prefixMappings ifNil: [prefixMappings := OrderPreservingDictionary defaultValue: '']! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 8/5/2010 01:18'!
prefixes
	^ self prefixMappings keys! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 20:11'!
prefixesAliasing: aPrefix
	"Locate all prefixes that map to the same URI the given prefix does."
	| uri |
	
	uri := self resolvePrefix: aPrefix ifUnmapped: [^ #()].

	^ (self prefixesMappedTo: uri) copyWithout: aPrefix! !

!XMLNamespaceScope methodsFor: 'enumerating' stamp: 'JAAyer 8/5/2010 01:00'!
prefixesAndURIsDo: aTwoArgumentBlock
	self prefixMappings keysAndValuesDo: aTwoArgumentBlock! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 8/8/2010 14:43'!
prefixesMappedTo: aUri
	^ self prefixes select: [:each | self includesPrefix: each mappedTo: aUri]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 07:04'!
resolvePrefix: aPrefix
	^ self resolvePrefix: aPrefix ifUnmapped: ['']! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 22:27'!
resolvePrefix: aPrefix ifUnmapped: aBlock
	aPrefix isEmptyOrNil
		ifTrue: [^ self defaultNamespace].
	aPrefix = 'xml'
		ifTrue: [^ self xmlNamespaceURI].
	aPrefix = 'xmlns'
		ifTrue: [^ self xmlnsNamespaceURI].

	^ self prefixMappings at: aPrefix ifAbsent: aBlock! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 13:36'!
unmapPrefix: aPrefix
	(self includesPrefix: aPrefix)
		ifFalse: [XMLNamespaceException signal: 'Prefix ', aPrefix, ' cannot be unmapped'].

	self prefixMappings removeKey: aPrefix.! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'JAAyer 11/6/2010 02:22'!
validatePrefix: aPrefix of: aMarkupEntityType named: aName
	(aPrefix isEmptyOrNil or: [aPrefix = 'xml'])
		ifTrue: [^ self].

	aPrefix = 'xmlns'
		ifTrue: [
			XMLNamespaceException signal:
				('Cannot use reserved "xmlns" prefix for {1} {2}' format:
					(Array with: aMarkupEntityType with: aName))].

	(self includesPrefix: aPrefix)
		ifFalse: [
			XMLNamespaceException signal:
				('Unmapped prefix "{1}" in {2} {3}' format:
					(Array with: aPrefix with: aMarkupEntityType with: aName))].! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 11/6/2010 02:00'!
validatePrefix: aPrefix ofAttributeName: anAttributeName
	self validatePrefix: aPrefix of: 'attribute' named: anAttributeName! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 11/6/2010 02:00'!
validatePrefix: aPrefix ofElementName: anElementName
	self validatePrefix: aPrefix of: 'element' named: anElementName! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 11/5/2010 22:30'!
validatePrefixedAttributeName: anAttributeName in: aDictionary
	anAttributeName splitQualifiedNameInto: [:prefix :localName |
		prefix ifEmpty: [^ self].

		self
			validatePrefix: prefix
			ofAttributeName: anAttributeName.
		self
			ensureAttributePrefix: prefix
			localName: localName
			hasNoAliasIn: aDictionary]! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 11/23/2010 07:05'!
validatePrefixedAttributeNames: aDictionary
	aDictionary keysDo: [:each | self validatePrefixedAttributeName: each in: aDictionary]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 20:25'!
xmlNamespaceURI
	^ 'http://www.w3.org/XML/1998/namespace'! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 20:25'!
xmlnsNamespaceURI
	^ 'http://www.w3.org/2000/xmlns/'! !

Object subclass: #XMLNamespaceScopeAllocator
	instanceVariableNames: 'scopes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScopeAllocator commentStamp: '<historical>' prior: 0!
This class handles the allocation of namespace scopes using a stack. Sending #enterScope to an instance will cause it to push a new scope onto the stack that inherits all mappings from the previous, enclosing scope and makes it the current scope. Sending #exitScop causes that scope to be removed and the parent scope to again be the current scope.!

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 14:27'!
currentScope
	^ self scopes
		at: self scopes size
		ifAbsent: [self emptyScope]! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'monty 12/7/2013 20:09'!
currentScope: aScope
	self pushScope: aScope! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'JAAyer 8/8/2010 14:45'!
declarePrefix: aPrefix uri: aUri
	"Declare the given name space prefix with the given URL"

	self currentScope mapPrefix: aPrefix to: aUri! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 22:37'!
defaultNamespace
	^ self currentScope defaultNamespace! !

!XMLNamespaceScopeAllocator methodsFor: 'private' stamp: 'JAAyer 3/13/2010 13:44'!
emptyScope
	^ XMLNamespaceScope new! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 22:14'!
enclosingScope
	^ self scopes
		at: self scopes size - 1
		ifAbsent: [self emptyScope]! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'JAAyer 3/13/2010 13:42'!
enterScope
	self pushScope: (XMLNamespaceScope enclosingScope: self currentScope)! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'JAAyer 7/27/2010 14:31'!
exitScope
	self popScope! !

!XMLNamespaceScopeAllocator methodsFor: 'testing' stamp: 'JAAyer 8/6/2010 12:07'!
includesPrefix: aPrefix
	^ self currentScope includesPrefix: aPrefix! !

!XMLNamespaceScopeAllocator methodsFor: 'testing' stamp: 'JAAyer 8/8/2010 14:43'!
includesPrefix: aPrefix mappedTo: aUri
	^ self currentScope includesPrefix: aPrefix mappedTo: aUri! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 3/4/2010 00:12'!
popScope
	self scopes removeLast! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 08:39'!
pushScope: aScope
	self scopes addLast: aScope! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 06:00'!
resolvePrefix: aPrefix
	^ self currentScope resolvePrefix: aPrefix! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 09:50'!
resolvePrefix: aPrefix ifUnmapped: aBlock
	^ self currentScope resolvePrefix: aPrefix ifUnmapped: aBlock! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'monty 12/6/2013 06:47'!
scopes
	^ scopes ifNil: [scopes := OrderedCollection new: 10]! !

!XMLNamespaceScopeAllocator methodsFor: 'validation' stamp: 'JAAyer 8/7/2010 13:42'!
validatePrefix: aPrefix ofElementName: anElementName
	self currentScope
		validatePrefix: aPrefix
		ofElementName: anElementName! !

!XMLNamespaceScopeAllocator methodsFor: 'validation' stamp: 'JAAyer 8/6/2010 23:14'!
validatePrefixedAttributeNames: aDictionary
	self currentScope validatePrefixedAttributeNames: aDictionary.! !

Object subclass: #XMLNestedStreamReader
	instanceVariableNames: 'stream nestedStreams peekChar writeBuffer normalizedLineEndingChar currentLineNumber readLimit'
	classVariableNames: 'CarriageReturn HexDigitCharacterMap LineFeed SeparatorCharacters'
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLNestedStreamReader commentStamp: '<historical>' prior: 0!
This class represents a stream reader that implements pushBack: using nested streams. This enables subsitution/replacement to be performed without modifying the underlying collections streamed-over or having to copy them. It also performs line-ending normalization, transforming CR and CRLF sequences into a single LF character.

(The code in this class is optimized; refactor with care.)!

!XMLNestedStreamReader class methodsFor: 'class initialization' stamp: 'monty 12/19/2013 08:12'!
initialize
	"self initialize"

	super initialize.

	SeparatorCharacters := CharacterSet newFrom: Character separators.
	CarriageReturn := Character cr.
	LineFeed := Character lf.

	HexDigitCharacterMap := Dictionary new.
	($0 to: $9) do: [:each | HexDigitCharacterMap at: each put: each asciiValue - $0 asciiValue].
	($a to: $f) do: [:each | | digitValue |
		digitValue := each asciiValue - $a asciiValue + 10.
		HexDigitCharacterMap
			at: each put: digitValue;
			at: each asUppercase put: digitValue].! !

!XMLNestedStreamReader class methodsFor: 'instance creation' stamp: 'monty 1/4/2014 08:05'!
on: anInitialStream readLimit: aReadLimit
	^ self new
		setStream: anInitialStream
		readLimit: aReadLimit! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 1/4/2014 08:07'!
atEnd
	^ peekChar isNil
		and: [stream atEnd
			and: [self hasNestedStreams not]]! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 12/6/2013 08:36'!
atQuote
	^ self peek == $"
		or: [self peek == $']! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 1/4/2014 08:08'!
basicNext
	"Returns next character in the stream after performing line-ending normalization."

	| nextChar |

	(nextChar := stream next)
		ifNil: [
			[self hasNestedStreams and: [stream atEnd]]
				whileTrue: [self popCurrentStream].
			nextChar := stream next].

	nextChar == CarriageReturn
		ifTrue: [
			nextChar := normalizedLineEndingChar.
			[self hasNestedStreams and: [stream atEnd]]
				whileTrue: [self popCurrentStream].
			(stream peek == LineFeed)
				ifTrue: [stream next]]
		ifFalse: [
			(nextChar == LineFeed)
				ifTrue: [nextChar := normalizedLineEndingChar]].

	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 7/14/2010 00:39'!
currentLineNumber
	^ currentLineNumber! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 1/4/2014 08:07'!
hasNestedStreams
	^ nestedStreams notNil and: [nestedStreams notEmpty]! !

!XMLNestedStreamReader methodsFor: 'initialization' stamp: 'monty 1/4/2014 08:10'!
initialize
	super initialize.

	nestedStreams := nil.
	peekChar := nil.
	writeBuffer := WriteStream on: (String new: 128).
	currentLineNumber := 1.
	normalizedLineEndingChar := LineFeed.! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'JAA 3/23/2011 10:54'!
maxIntegerValue
	"SmallInteger maxValue"
	^ 1073741823! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 12/6/2013 06:47'!
maxNestedStreams
	^ 10! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 1/4/2014 08:08'!
next
	"Return the next character from the current input stream. If the current
	stream is at end, pop to the next nesting level if there is one."

	| nextChar |

	peekChar
		ifNil: [nextChar := self basicNext]
		ifNotNil: [
			nextChar := peekChar.
			peekChar := nil].

	nextChar == normalizedLineEndingChar
		ifTrue: [currentLineNumber := currentLineNumber + 1].
	readLimit ifNotNil: [
		(readLimit := readLimit - 1) > 0
			ifFalse: [XMLLimitException signal: 'Read limit on input exceeded']].

	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAA 3/23/2011 10:48'!
nextDigitWithBase: aBase
	| digit |

	aBase = 16
		ifTrue: [digit := HexDigitCharacterMap at: self peek ifAbsent: [nil]]
		ifFalse: [
			digit := self peek asciiValue - $0 asciiValue.
			(digit >= 0 and: [digit <= 10])
				ifFalse: [digit := nil]].
	digit ifNotNil: [self next].

	^ digit.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAA 3/23/2011 11:07'!
nextInteger
	"We re-implement integer reading here because the impelmentation of Integer>>readFrom: expects
	the underlying stream to be skip:able."
	| base |

	self peek == $x
		ifTrue: [
			self next.
			base := 16]
		ifFalse: [base := 10].

	^ self nextIntegerWithBase: base.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAA 3/23/2011 11:14'!
nextIntegerWithBase: aBase
	| integer maxValue maxBaseValue digitValue |

	integer := 0.
	maxValue := self maxIntegerValue.
	maxBaseValue := (maxValue / aBase) asInteger.
	[self atEnd not
		and: [(digitValue := self nextDigitWithBase: aBase) notNil]]
		whileTrue: [
			integer > maxBaseValue
				ifTrue: [^ integer].
			integer := integer * aBase.

			integer > (maxValue - digitValue)
				ifTrue: [^ integer].
			integer := integer + digitValue].
	^ integer.! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:40'!
nextMatchAll: aString
	| i nextChar pushBackString |

	self peek == (aString at: 1)
		ifFalse: [^ false].

	"we unrolled the first iteration with the above #peek"
	i := 2.
	self next.
	[(i <= aString size)
		and: [(aString at: i) == (nextChar := self next)]]
		whileTrue: [i := i + 1].
	(i > aString size)
		ifTrue: [^ true].

	pushBackString := aString copyFrom: 1 to: i - 1.
	self pushBack:
		(nextChar
			ifNotNil: [pushBackString copyWith: nextChar]
			ifNil: [pushBackString]).
	^ false.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 12/18/2013 17:25'!
normalizedLineEndingChar: aCharacter
	normalizedLineEndingChar := aCharacter! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 10/18/2010 16:17'!
peek
	"Return the next character from the current input stream."

	^ peekChar ifNil: [peekChar := self basicNext]! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 1/4/2014 08:10'!
popCurrentStream
	stream close.
	stream := nestedStreams removeLast.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'monty 12/6/2013 06:44'!
pushBack: aString
	self pushStream: (ReadStream on: aString)! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'monty 12/6/2013 06:45'!
pushBackPeekChar
	| peekCharString |

	peekCharString := String with: peekChar.
	peekChar := nil.
	self pushBack: peekCharString.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'monty 12/13/2013 05:05'!
pushStream: aStream
	self pushStream: aStream onClose: nil! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'monty 1/4/2014 08:09'!
pushStream: aStream onClose: aBlock
	peekChar
		ifNotNil: [self pushBackPeekChar].

	nestedStreams
		ifNil: [nestedStreams := OrderedCollection with: stream]
		ifNotNil: [
			(nestedStreams size >= self maxNestedStreams)
				ifTrue: [XMLParserException signal: 'cannot further nest input streams'].
			nestedStreams addLast: stream].

	self setStream: aStream onClose: aBlock! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 12/13/2013 05:13'!
setStream: aNewStream onClose: aBlock
	stream :=
		XMLPeekableStreamAdapter
			on: aNewStream
			onClose: aBlock! !

!XMLNestedStreamReader methodsFor: 'initialization' stamp: 'monty 1/4/2014 08:18'!
setStream: anInitialStream readLimit: aReadLimit
	self
		setStream: anInitialStream
		onClose: nil.

	readLimit := aReadLimit.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 7/21/2010 11:39'!
skipSeparators
	| nextChar |

	[(nextChar := self peek) notNil
		and: [SeparatorCharacters includes: nextChar]]
		whileTrue: [self next].! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'monty 1/4/2014 08:09'!
skipTo: aDelimiter
	[self atEnd] whileFalse: [
		self next == aDelimiter
			ifTrue: [^ true]].
	^ false.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 08:01'!
stream
	^ stream! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAA 3/22/2011 11:49'!
upTo: aDelimiter
	| nextChar |

	writeBuffer reset.
	[self atEnd or: [(nextChar := self next) == aDelimiter]]
		whileFalse: [writeBuffer nextPut: nextChar].

	^ writeBuffer contents.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAA 3/22/2011 11:49'!
upToAll: aDelimitingString
	"Answer a subcollection from the current access position to the occurrence
	(if any, but not inclusive) of delimitingString. If delimitingString is not
	in the stream, answer the entire rest of the stream."

	writeBuffer reset.
	[self atEnd or: [self nextMatchAll: aDelimitingString]]
		whileFalse: [writeBuffer nextPut: self next].
	^ writeBuffer contents.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'monty 11/13/2013 02:59'!
upToEnd
	writeBuffer reset.
	[self atEnd] whileFalse: [writeBuffer nextPut: self next].
	^ writeBuffer contents.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 12/14/2010 02:39'!
useConverterForEncoding: anEncodingName
	| converterClass |

	converterClass :=
		(Smalltalk
			at: #TextConverter
			ifAbsent: [^ self])
				defaultConverterClassForEncoding: anEncodingName asLowercase.
		
	converterClass
		ifNotNil: [stream converter: converterClass new].! !

Object subclass: #XMLNestedStreamWriter
	instanceVariableNames: 'streams nextStreamIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLNestedStreamWriter commentStamp: 'JAAyer 7/12/2010 01:22' prior: 0!
This class handles the allocation and reuse of multiple WriteStreams. It uses a single execute-around method, #writeWith:, which takes a block that it evaluates with a ready-to-use WriteStream.
!

!XMLNestedStreamWriter class methodsFor: 'instance creation' stamp: 'JAAyer 9/19/2010 23:48'!
new
	^ self new: 5! !

!XMLNestedStreamWriter class methodsFor: 'instance creation' stamp: 'JAAyer 9/19/2010 23:51'!
new: aNumberOfStreams
	^ self basicNew initialize: aNumberOfStreams! !

!XMLNestedStreamWriter methodsFor: 'private' stamp: 'monty 11/26/2013 00:06'!
grow
	streams :=
		(Array new: streams size + 5)
			replaceFrom: 1
			to: streams size
			with: streams
			startingAt: 1.
	nextStreamIndex to: streams size do: [:i | streams at: i put: self newStream].! !

!XMLNestedStreamWriter methodsFor: 'initialization' stamp: 'JAAyer 9/19/2010 23:50'!
initialize: aNumberOfStreams
	streams := Array new: aNumberOfStreams.
	1 to: aNumberOfStreams do: [:i | streams at: i put: self newStream].
	nextStreamIndex := 1.! !

!XMLNestedStreamWriter methodsFor: 'private' stamp: 'monty 11/26/2013 01:18'!
newStream
	^ WriteStream on: (String new: 64)! !

!XMLNestedStreamWriter methodsFor: 'writing' stamp: 'monty 11/26/2013 00:05'!
writeWith: aBlock
	| result stream |

	(nextStreamIndex <= streams size)
		 ifFalse: [self grow].

	(stream := streams at: nextStreamIndex) reset.
	nextStreamIndex := nextStreamIndex + 1.
	"Avoid ensure: for performance"
	result := aBlock value: stream.
	nextStreamIndex := nextStreamIndex - 1.
	^ result.! !

Object subclass: #XMLNode
	instanceVariableNames: 'parent notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNode commentStamp: '<historical>' prior: 0!
This is a base class for XML nodes. It contains testing messages and methods to set and access the parent node and sibling nodes. It also provides a number messages to control printing.!

XMLNode subclass: #XMLAttribute
	instanceVariableNames: 'element name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttribute commentStamp: '<historical>' prior: 0!
The class represents an attribute node, to be stored in an attribute node list. You generally won't need to interact with these objects directly; the various #attribute- messages element nodes respond to will normally suffice. The name of an attribute node can be namespaced using prefixes; however, the prefix must be mapped to a namespace URI in the containing element node. Unprefixed attributes have no namespace, not even a default namespace URI if one is in scope, per the XML namespace standard.

Note that attribute nodes are not children of the elements that contain them; rather, they are considered to be "part" of the element, and this relationship is modeled using a separate instance variable ("element") with different accessor messages.!

!XMLAttribute class methodsFor: 'instance creation' stamp: 'monty 12/9/2013 04:03'!
name: aName value: aValue
	^ self new setName: aName value: aValue! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 18:04'!
element
	^ element! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 18:04'!
element: anElement
	element := anElement! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 4/12/2010 00:37'!
expandedName
	^ self nodeName expandWith: self namespaceURI! !

!XMLAttribute methodsFor: 'testing' stamp: 'JAAyer 6/26/2010 07:31'!
hasPrefix
	^ self nodeName isPrefixed! !

!XMLAttribute methodsFor: 'testing' stamp: 'JAAyer 7/22/2010 08:29'!
isNamed: aName
	^ self nodeName isSameAs: aName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:16'!
localName
	^ self nodeName localPart! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 00:14'!
name
	^ self nodeName qualified! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:19'!
name: aName
	self nodeName qualified: aName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 22:02'!
namespaceURI
	(self element notNil and: [self hasPrefix])
		ifFalse: [^ ''].
	
	^ self element resolvePrefix: self prefix.! !

!XMLAttribute methodsFor: 'private' stamp: 'monty 12/9/2013 04:03'!
nodeName
	^ name
		ifNil: [ 
			(name := XMLNodeName new)
				addObserver: self;
				yourself]! !

!XMLAttribute methodsFor: 'notifications' stamp: 'JAAyer 7/11/2010 02:25'!
nodeNameChangedFrom: anOldName to: aNewName
	self
		notifyOf: #renamedAttribute:from:to:
		with: self
		with: anOldName
		with: aNewName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 3/17/2010 03:48'!
prefix
	^ self nodeName prefix! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 6/15/2010 15:15'!
prefix: aPrefix
	self nodeName prefix: aPrefix! !

!XMLAttribute methodsFor: 'initialization' stamp: 'monty 12/9/2013 05:15'!
setName: aName value: aValue
	"This is used over 'self nodeName qualified:' to avoid sending a name-changed notification"
	(name := XMLNodeName new)
		setQualified: aName;
		addObserver: self.
	value := aValue.! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
value
	^ value ifNil: [value := '']! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 08:18'!
value: aString
	value := aString! !

!XMLAttribute methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter writeName: self name value: self value! !

XMLNode subclass: #XMLComment
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLComment commentStamp: '<historical>' prior: 0!
This class represents comments preserved as nodes. Comments are ignored during parsing by default. If you want them preserved as comment nodes, use #preservesCommentNodes: with the DOM parser before parsing.!

!XMLComment class methodsFor: 'instance creation' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	^ self new string: aString! !

!XMLComment methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ true! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string
	^ string ifNil: [string := '']! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	string := aString! !

!XMLComment methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter comment: self string! !

XMLNode subclass: #XMLDeclarationNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDeclarationNode commentStamp: 'JAAyer 8/11/2010 21:03' prior: 0!
This is a generic class to represent markup declarations in the internal subset. Each subclass represents a specific kind of declaration, and each is immutable.!

XMLDeclarationNode subclass: #XMLAttributeDeclaration
	instanceVariableNames: 'element attribute type defaultPragma defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttributeDeclaration class methodsFor: 'instance creation' stamp: 'monty 12/30/2013 23:59'!
element: anElementName attribute: anAttributeName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	^ self new
		setElement: anElementName
		attribute: anAttributeName
		type: aType
		defaultPragma: aDefaultPragma
		defaultValue: aDefaultValue! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:53'!
attribute
	^ attribute ifNil: [attribute := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:14'!
defaultPragma
	^ defaultPragma ifNil: [defaultPragma := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:11'!
defaultValue
	^ defaultValue ifNil: [defaultValue := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:21'!
element
	^ element ifNil: [element := '']! !

!XMLAttributeDeclaration methodsFor: 'initialization' stamp: 'monty 12/31/2013 00:02'!
setElement: anElementName attribute: anAttributeName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	element := anElementName.
	attribute := anAttributeName.
	type := aType.
	defaultPragma := aDefaultPragma.
	defaultValue := aDefaultValue.! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:42'!
type
	^ type ifNil: [type := '']! !

!XMLAttributeDeclaration methodsFor: 'printing' stamp: 'monty 12/31/2013 00:16'!
writeXMLOn: aWriter
	aWriter attributeDeclaration
		element: self element;
		attribute: self attribute;
		type: self type;
		defaultPragma: self defaultPragma;
		defaultValue: self defaultValue! !

!XMLDeclarationNode methodsFor: 'testing' stamp: 'monty 12/22/2013 06:10'!
isDeclaration
	^ true! !

XMLDeclarationNode subclass: #XMLElementDeclaration
	instanceVariableNames: 'contentModel element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLElementDeclaration class methodsFor: 'instance creation' stamp: 'monty 12/21/2013 03:14'!
element: aName contentModel: aContentModelString
	^ self new setElement: aName contentModel: aContentModelString! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
contentModel
	^ contentModel ifNil: [contentModel := '']! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:21'!
element
	^ element ifNil: [element := '']! !

!XMLElementDeclaration methodsFor: 'initialization' stamp: 'monty 12/21/2013 03:15'!
setElement: aName contentModel: aContentModelString
	element := aName.
	contentModel := aContentModelString.! !

!XMLElementDeclaration methodsFor: 'printing' stamp: 'monty 12/21/2013 03:25'!
writeXMLOn: aWriter
	aWriter elementDeclaration
		element: self element;
		contentModel: self contentModel! !

XMLDeclarationNode subclass: #XMLEntityDeclaration
	instanceVariableNames: 'name replacement publicId systemId ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/20/2010 10:53'!
name: aName publicID: aPublicId systemID: aSystemId
	^ self new
		setName: aName
		replacement: nil
		publicID: aPublicId
		systemID: aSystemId
		ndata: nil! !

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/20/2010 10:52'!
name: aName replacement: aReplacement
	^ self new
		setName: aName
		replacement: aReplacement
		publicID: nil
		systemID: nil
		ndata: nil! !

!XMLEntityDeclaration methodsFor: 'testing' stamp: 'monty 11/30/2013 21:53'!
isParameter
	^ false! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:22'!
name
	^ name ifNil: [name := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
ndata
	^ ndata ifNil: [ndata := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
replacement
	^ replacement ifNil: [replacement := '']! !

!XMLEntityDeclaration methodsFor: 'initialization' stamp: 'JAAyer 7/20/2010 10:51'!
setName: aName replacement: aReplacement publicID: aPublicId systemID: aSystemId ndata: aNotationName
	name := aName.
	replacement := aReplacement.
	publicId := aPublicId.
	systemId := aSystemId.
	ndata := aNotationName.! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
systemID
	^ systemId ifNil: [systemId := '']! !

!XMLEntityDeclaration methodsFor: 'printing' stamp: 'monty 11/13/2013 19:35'!
writeXMLOn: aWriter
	aWriter entityDeclaration
		name: self name;
		isParameter: self isParameter;
		replacement: self replacement;
		publicID: self publicID;
		systemID: self systemID;
		ndata: self ndata! !

XMLEntityDeclaration subclass: #XMLGeneralEntityDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLGeneralEntityDeclaration class methodsFor: 'instance creation' stamp: 'monty 11/14/2013 19:41'!
name: aName publicID: aPublicId systemID: aSystemId ndata: aNotationName
	^ self new
		setName: aName
		replacement: nil
		publicID: aPublicId
		systemID: aSystemId
		ndata: aNotationName! !

XMLEntityDeclaration subclass: #XMLParameterEntityDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLParameterEntityDeclaration methodsFor: 'testing' stamp: 'monty 11/13/2013 19:34'!
isParameter
	^ true! !

XMLDeclarationNode subclass: #XMLNotationDeclaration
	instanceVariableNames: 'name publicId systemId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNotationDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/21/2010 00:55'!
name: aName publicID: aPublicId systemID: aSystemId
	^ self new setName: aName publicID: aPublicId systemID: aSystemId! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:22'!
name
	^ name ifNil: [name := '']! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLNotationDeclaration methodsFor: 'initialization' stamp: 'JAAyer 7/21/2010 00:54'!
setName: aName publicID: aPublicId systemID: aSystemId
	name := aName.
	publicId := aPublicId.
	systemId := aSystemId.! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
systemID
	^ systemId ifNil: [systemId := '']! !

!XMLNotationDeclaration methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter notationDeclaration
		name: self name;
		publicID: self publicID;
		systemID: self systemID! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:15'!
addContent: aStringOrStringNode
	XMLDOMException signal: 'Node cannot contain content'! !

!XMLNode methodsFor: 'observing' stamp: 'OttoBehrens 2/15/2011 08:33'!
addObserver: anObserver
	self notifier addObserver: anObserver! !

!XMLNode methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:59'!
allNodesDo: aBlock
	aBlock value: self! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:13'!
canonicallyPrintOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [ :writer | writer
										isCanonical: true;
										writeWith: [ :wr | self writeXMLOn: wr ] ]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 11/30/2013 19:07'!
canonicallyPrinted
	^ String streamContents: [:stream | self canonicallyPrintOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:16'!
contentString
 	"This returns the combined content string of all descendants with spaces inserted wher necessary."
	^ String streamContents: [:stream | self printContentOn: stream]! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:49'!
hasChildren
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 14:37'!
hasParent
	^ self parent notNil! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/30/2010 19:19'!
isContentNode
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 12/22/2013 06:10'!
isDeclaration
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:50'!
isDocument
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:33'!
isStringNode
	^ false! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:56'!
nextNode
	^ self parent ifNotNil: [self parent nodeAfter: self]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 6/11/2010 10:17'!
notifier
	^ notifier ifNil: [notifier := XMLNotifier new]! !

!XMLNode methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:33'!
notifyOf: aSelector with: anArgument
	self notifier notifyOf: aSelector with: anArgument! !

!XMLNode methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:34'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument
	self notifier notifyOf: aSelector with: aFirstArgument with: aSecondArgument! !

!XMLNode methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:34'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument with: aThirdArgument
	self notifier
		notifyOf: aSelector
		with: aFirstArgument
		with: aSecondArgument
		with: aThirdArgument! !

!XMLNode methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:34'!
notifyOf: aSelector withArguments: anArray
	self notifier notifyOf: aSelector withArguments: anArray! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent
	^ parent! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent: aParentNode
	parent := aParentNode! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:07'!
prettyPrintOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [ :writer | writer
									enablePrettyPrinting;
									writeWith: [ :wr | self writeXMLOn: wr ] ]
		whenAbsent: [ self printOn: aStream ]! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/6/2010 01:18'!
prettyPrinted
	^ String streamContents: [:stream | self prettyPrintOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:56'!
previousNode
	^ self parent ifNotNil: [self parent nodeBefore: self]! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 10/20/2010 19:11'!
printContentOn: aStream! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:41'!
printOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [ :writer | writer writeWith: [ :wr | self writeXMLOn: wr] ]
		whenAbsent: [ super printOn: aStream ]! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:27'!
raiseXMLWriterRequired
	Error signal: 'The XMLWriter package is required to do that'! !

!XMLNode methodsFor: 'observing' stamp: 'OttoBehrens 2/15/2011 08:34'!
removeObserver: anObserver
	self notifier removeObserver: anObserver! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:26'!
withNewXMLWriterDo: aOneArgBlock
	^ self withNewXMLWriterDo: aOneArgBlock whenAbsent: [ self raiseXMLWriterRequired ]
	! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:24'!
withNewXMLWriterDo: aOneArgBlock whenAbsent: aZeroArgBlock
	^ self
			withXMLWriterClassDo: [ :xmlWriterClass | aOneArgBlock value: xmlWriterClass new ]
			whenAbsent: aZeroArgBlock! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:27'!
withNewXMLWriterOn: aStream do: aOneArgBlock
	^ self withNewXMLWriterOn: aStream do: aOneArgBlock whenAbsent: [ self raiseXMLWriterRequired ]! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:25'!
withNewXMLWriterOn: aStream do: aOneArgBlock whenAbsent: aZeroArgBlock
	^ self
			withXMLWriterClassDo: [ :xmlWriterClass | aOneArgBlock value: (xmlWriterClass on: aStream) ]
			whenAbsent: aZeroArgBlock! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:22'!
withXMLWriterClassDo: aOneArgBlock whenAbsent: aZeroArgBlock
	^ (self class environment hasClassNamed: #XMLWriter)
			ifTrue: [ aOneArgBlock value: (self class environment classNamed: #XMLWriter) ]
			ifFalse: [ aZeroArgBlock ]! !

!XMLNode methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	self subclassResponsibility! !

XMLNode subclass: #XMLNodeWithChildren
	instanceVariableNames: 'nodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeWithChildren commentStamp: '<historical>' prior: 0!
This class represents a node that can contain child nodes. You probably will only use its API through one its subclasses, specifically the document, element and doctype declaration node classes.

This class provides message to access, add and remove child nodes under the "accessing" category. The nodes themselves are stored in some type of XMLObservableList, the exact type of which varies depending on the subclass and can be accessed directly using #nodes. You can modify the #nodes list directly if you like, but adding nodes to it or removing nodes from it will have the effect of adding them to removing them from the parent node. If this is not what you want, then make sure you copy the return value of #nodes before manipulating it.

The "enumerating" category provides two different modes of enumeration: the #nodes* enumerating messages enumerate child nodes of the receiver only, while the #allNode* forms recursively enumerate all descendent nodes, including the receiver itself, using depth-first traversal.

The inner XML of a node with children can be accessed as a string using #innerXML and set using #innerXML:.!

XMLNodeWithChildren subclass: #XMLDoctypeDeclaration
	instanceVariableNames: 'root publicId systemId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDoctypeDeclaration commentStamp: '<historical>' prior: 0!
This class represents the <!!DOCTYPE ...> declaration one often finds at the top of an XML document. It stores the root element name, formal public identifier (FPI) if any, system identifier (a URI) and then any nodes comprising the internal subset.!

!XMLDoctypeDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/21/2010 07:47'!
root: aRootElementName publicID: aPublicId systemID: aSystemId
	^ self new
		setRoot: aRootElementName
		publicID: aPublicId
		systemID: aSystemId! !

!XMLDoctypeDeclaration methodsFor: 'private' stamp: 'monty 11/26/2013 21:51'!
nodeListClass
	^ XMLInternalSubsetList! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 6/17/2010 21:35'!
publicID
	^ publicId! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 7/21/2010 07:46'!
root
	^ root! !

!XMLDoctypeDeclaration methodsFor: 'initialization' stamp: 'JAAyer 7/21/2010 07:45'!
setRoot: aRootElementName publicID: aPublicId systemID: aSystemId
	root := aRootElementName.
	publicId := aPublicId.
	systemId := aSystemId.! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 6/17/2010 21:35'!
systemID
	^ systemId! !

!XMLDoctypeDeclaration methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	aWriter doctype
		root: self root;
		publicID: self publicID;
		systemID: self systemID;
		withInternalSubset: [self writeInnerXMLOn: aWriter]! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:22'!
addNode: aNode
	^ self nodes addLast: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 8/14/2010 21:05'!
addNode: aNode after: aPriorNode
	^ self nodes add: aNode after: aPriorNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 8/14/2010 21:05'!
addNode: aNode before: aSubsequentNode
	^ self nodes add: aNode before: aSubsequentNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:22'!
addNodeFirst: aNode
	^ self nodes addFirst: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 05:12'!
addNodes: aNodeCollection
	^ self nodes addAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifications' stamp: 'JAAyer 10/24/2010 18:09'!
addedNode: aNode
	aNode
		addObserver: self;
		parent: self! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 12/4/2010 12:32'!
allNodes
	^ self allNodesCollect: [:each | each]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:11'!
allNodesCollect: aBlock
	| collectedNodes |

	collectedNodes := self nodes copyEmpty.
	self allNodesDo: [:each | collectedNodes add: (aBlock value: each)].
	^ collectedNodes.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:17'!
allNodesDetect: aBlock
	^ self allNodesDetect: aBlock ifNone: [nil]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:21'!
allNodesDetect: aBlock ifNone: anExceptionBlock
	self allNodesDo: [:each | (aBlock value: each) ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:59'!
allNodesDo: aBlock
	super allNodesDo: aBlock.

	self hasChildren
		ifTrue: [self nodesDo: [:each | each allNodesDo: aBlock]]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:11'!
allNodesSelect: aBlock
	| selectedNodes |

	selectedNodes := self nodes copyEmpty.
	self allNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedNodes add: each]].
	^ selectedNodes.! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 11/26/2013 21:52'!
firstNode
	^ self nodes first! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:48'!
hasChildren
	"Direct variable access avoids possibly unnecessary lazy initialization of the node list"

	^ nodes notNil and: [nodes notEmpty]! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:23'!
includesNode: aNode
	^ self nodes includes: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'DamienCassou 4/30/2013 17:14'!
innerXML
	^ self
			withNewXMLWriterDo: [ :writer | self writeInnerXMLOn: writer.
																writer contents ]
			! !

!XMLNodeWithChildren methodsFor: 'parsing' stamp: 'JAAyer 11/26/2010 19:23'!
innerXML: aStringOrStream
	self removeNodes.

	(self innerXMLParserOn: aStringOrStream) parseDocument.! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'JAAyer 11/26/2010 19:23'!
innerXMLParserOn: aStringOrStream
	^ XMLDOMParser on: aStringOrStream withRootNode: self! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 11/26/2013 21:53'!
lastNode
	^ self nodes last! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:26'!
nodeAfter: aNode
	^ self nodes after: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
nodeBefore: aNode
	^ self nodes before: aNode! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 11/26/2013 21:51'!
nodeListClass
	self subclassResponsibility! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 11/26/2013 21:51'!
nodes
	^ nodes ifNil: [
		(nodes := self nodeListClass new)
			addObserver: self;
			yourself]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:13'!
nodesCollect: aBlock
	^ self nodes collect: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:45'!
nodesDetect: aBlock
	^ self nodesDetect: aBlock ifNone: [nil]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:14'!
nodesDetect: aBlock ifNone: anExceptionBlock
	^ self nodes detect: aBlock ifNone: anExceptionBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 7/20/2010 10:27'!
nodesDo: aBlock
	self nodes do: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:13'!
nodesSelect: aBlock
	^ self nodes select: aBlock! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNode: aNode
	^ self nodes remove: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes
	self nodes removeAll! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes: aNodeCollection
	^ self nodes removeAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifications' stamp: 'JAAyer 10/24/2010 18:08'!
removedNode: aNode
	aNode
		removeObserver: self;
		parent: nil.! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 22:39'!
replaceNode: aNode with: aReplacementNode
	^ self nodes
		at: (self nodes indexOf: aNode)
		put: aReplacementNode! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeInnerXMLOn: aWriter
	self nodesDo: [:each | each writeXMLOn: aWriter]! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	self writeInnerXMLOn: aWriter! !

XMLNodeWithChildren subclass: #XMLNodeWithElements
	instanceVariableNames: 'configuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeWithElements commentStamp: '<historical>' prior: 0!
This class represents a node with elements. You will likely use its API through its subclasses instead.

Instances provide "accessing" messages to retrieve child elements by their name and namespace information. The #elementAt: forms return the first matching element, while the #elementsAt: forms return all matching child elements.

As with node enumeration in the superclass, there are two different modes of enumeration: the #elements* enumerating messages that enumerate child elements only, and the #allElements* forms that enumerate all descendent elements, including the receiver if it is an element (but not if it is a document), using depth-first traversal.

The #findElementNamed:* forms can be used to search using depth-first traversal for a specific element. Searching will start with the receiver if it is an element.

All element name matching is done using both the qualified and local name of elements, so 'prefix:element-name' will only match 'prefix:element-name' while 'element-name' will match 'element-name', 'prefix:element-name' or 'different-prefix:element-name' and so on.!

XMLNodeWithElements subclass: #XMLDocument
	instanceVariableNames: 'version encoding doctypeDeclaration isStandalone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDocument commentStamp: '<historical>' prior: 0!
This class represents a document node, which is often the root of a node tree. It can contain XML version and encoding information, a DOCTYPE declaration with a possible internal subset and a root element child node.!

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration
	^ doctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration: aDoctypeDeclaration
	doctypeDeclaration := aDoctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 13:34'!
document
	^ self! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 09:26'!
document: aDocument
	self shouldNotImplement! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:03'!
encoding	
	^ encoding ifNil: [encoding := 'UTF-8']! !

!XMLDocument methodsFor: 'accessing' stamp: 'mir 1/17/2002 12:57'!
encoding: aString	
	encoding := aString! !

!XMLDocument methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:51'!
isDocument
	^ true! !

!XMLDocument methodsFor: 'testing' stamp: 'monty 11/19/2013 05:38'!
isStandalone
	^ isStandalone ifNil: [isStandalone := false]! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 11/19/2013 05:40'!
isStandalone: aBoolean
	isStandalone := aBoolean! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/25/2010 15:28'!
root
	^ self firstElement! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 12/6/2013 06:33'!
root: anElement
	self
		removeNodes: self elements;
		addNode: anElement! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
version	
	^ version! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 19:39'!
version: aVersionNumber
	version := aVersionNumber! !

!XMLDocument methodsFor: 'printing' stamp: 'monty 11/19/2013 05:47'!
writeXMLOn: aWriter
	self version ifNotNil: [
		aWriter xml
			version: self version;
			encoding: self encoding;
			standalone: self isStandalone].
	self doctypeDeclaration
		ifNotNil: [self doctypeDeclaration writeXMLOn: aWriter].
	
	self writeInnerXMLOn: aWriter.! !

XMLNodeWithElements subclass: #XMLElement
	instanceVariableNames: 'name attributes namespaceScope document'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLElement commentStamp: '<historical>' prior: 0!
The class represents an element node. An element consists of a name and optionally attributes, namespace declarations and any number child nodes and elements.

The name can consist of a local name only or a qualified name, which is a namespace prefix and local name separated by a colon. If the name is qualfied and namespace support is enabled (the default), then the prefix must be mapped to a namespace URI in the element or its parent at the time the name is set. The class-side instance creation #name:namespaceURI:* messages and #name:namespaces:* messages allow you set both simulaniously, as does the instance-side #name:namespaceURI: message. If namespace support is disabled, prefixes are ignored (not validated). You can test the name of an element using #isNamed: and #isNamedAny:, which test the qualified and local names of an element for a match.

Namespaces can be declared using #declareNamespace:uri:, using #declareNamespaces: with a collection of associations (where an empty prefix represents the default namespace) or simultaniously while changing the element's name using #name:namespaceURI:. The default namespace can be accessed through #defaultNamespace and #defaultNamespace:. When you print an element, the minimum number of xmlns declarations necessary for it to be valid will be printed. You can query an element's namespace information using #namespaceURI, #prefix, #expandedName and other messages under "namespaces."" For an element to be namespaced, it must have a qualified name with a valid prefix or have a default namespace declaration in scope for it.

The #attribute- messages provide a Dictionary-like prototcol for manipulating attribute nodes. Attribute value accessors return empty strings when the specified value is not present. The underlying attribute node list can be accessed using #attributeNodes, and an order-preserving dictionary of attribute names/values can be obtained using #attributes.

See the comment for the superclass for how to access child nodes and elements.!

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 10/30/2010 22:09'!
name: aLocalName
	^ self new name: aLocalName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:05'!
name: aLocalName attributes: aDictionaryOrArray
	^ (self named: aLocalName)
		setAttributes: aDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaceURI: aUri
	^ self new name: aQualifiedName namespaceURI: aUri! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaceURI: aUri attributes: aDictionaryOrArray
	^ (self name: aQualifiedName namespaceURI: aUri)
		setAttributes: aDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaces: aNamespaceDictionaryOrArray
	^ self new
		declareNamespaces: aNamespaceDictionaryOrArray;
		name: aQualifiedName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaces: aNamespaceDictionaryOrArray attributes: anAttributeDictionaryOrArray
	^ (self
		name: aQualifiedName
		namespaces: aNamespaceDictionaryOrArray)
			setAttributes: anAttributeDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 3/11/2010 23:10'!
named: aLocalName
	^ self new name: aLocalName! !

!XMLElement class methodsFor: 'private' stamp: 'monty 12/27/2013 03:43'!
qualifiedName: aQualifiedName prefix: aPrefix localName: aLocalName attributes: anAttributeDictionary
	^ self new
		setQualifiedName: aQualifiedName
		prefix: aPrefix
		localName: aLocalName
		attributes: anAttributeDictionary! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 19:07'!
addContent: aStringOrStringNode
	"Adds a string or string node. If the last child node is already a string node,
	the characters being added are concatenated to it."

	(self hasChildren and: [self lastNode isStringNode])
		ifTrue: [self lastNode addContent: aStringOrStringNode]
		ifFalse: [self addNode: aStringOrStringNode asXMLString]! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 10/24/2010 18:07'!
addedAttribute: anAttribute
	anAttribute
		addObserver: self;
		element: self.
	self validateAttributeName: anAttribute name.! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 11/24/2010 09:47'!
addedElement: anElement
	super addedElement: anElement.
	self hasNamespaces
		ifTrue: [anElement inheritNamespacesFrom: self]! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:37'!
allElementsDo: aBlock
	"See superclass version."

	aBlock value: self.

	super allElementsDo: aBlock.! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:15'!
attributeAssociations
	^ self attributeNodes asNameValueAssociations! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName
	^ self attributeAt: aName ifAbsent: ['']! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName ifAbsent: aBlock
	^ self attributeNodes attributeValueAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName ifAbsentPut: aBlock
	^ self attributeNodes attributeValueAt: aName ifAbsentPut: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:34'!
attributeAt: aName put: aValue
	^ self attributeNodes attributeValueAt: aName put: aValue! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:08'!
attributeNames
	^ self attributeNodes attributeNames! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:14'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self attributeNodes attributeNamesAndValuesDo: aTwoArgumentBlock! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:09'!
attributeNamesDo: aBlock
	self attributeNodes attributeNamesDo: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName
	^ self attributeNodes attributeNodeAt: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName ifAbsent: aBlock
	^ self attributeNodes attributeNodeAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri
	^ self attributeNodes attributeNodeAt: aLocalName namespaceURI: aUri! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock
	^ self attributeNodes attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 12:41'!
attributeNodes
	^ attributes ifNil: [
		(attributes := XMLAttributeList new)
			addObserver: self;
			yourself]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:16'!
attributes
	^ self attributeNodes asDictionary! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
contentString: aString
	self
		removeNodes;
		addContent: aString! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/5/2010 20:44'!
declareNamespaces: aDictionaryOrArray
	aDictionaryOrArray associationsDo: [:each |
		each key isEmptyOrNil
			ifTrue: [self defaultNamespace: each value]
			ifFalse: [self declarePrefix: each key uri: each value]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:23'!
declarePrefix: aPrefix uri: aUri
	| oldMapping |

	oldMapping := self namespaceScope resolvePrefix: aPrefix.
	oldMapping = aUri
		ifTrue: [^ self].

	self
		updateEnclosedScopesWith: [:scope | scope mapPrefix: aPrefix to: aUri]
		whileTrue: [:scope |
			(scope includesPrefix: aPrefix) not
				or: [scope includesPrefix: aPrefix mappedTo: oldMapping]].! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:55'!
declaresDefaultNamespace
	"Did the receiver declare a new default namespace rather than inheriting one?"

	^ self namespaceScope hasDefaultNamespace
		and: [self defaultNamespace isEmpty
			or: [self isRoot
				or: [self defaultNamespace ~= self parent defaultNamespace]]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:54'!
declaresPrefix: aPrefix uri: aUri
	"Did the receiver declare a new prefix/uri pair rather than inheriting one?"

	^ (self isDeclaredPrefix: aPrefix uri: aUri)
		and: [self isRoot
			or: [(self parent resolvePrefix: aPrefix) ~= aUri]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/20/2010 23:08'!
defaultNamespace
	^ self namespaceScope defaultNamespace! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:23'!
defaultNamespace: aUri
	| oldNamespace |

	oldNamespace := self defaultNamespace.
	self
		updateEnclosedScopesWith: [:scope |
			scope defaultNamespace:
				(aUri
					ifEmpty: [scope == self namespaceScope ifTrue: [''] ifFalse: [nil]]
					ifNotEmpty: [aUri])]
		whileTrue: [:scope |
			scope hasDefaultNamespace not
				or: [scope defaultNamespace = oldNamespace]]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/12/2010 10:46'!
document
	^ document! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 18:19'!
document: aDocument
	document := aDocument.

	self hasElements
		ifTrue: [self elementsDo: [:each | each document: aDocument]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 4/12/2010 00:37'!
expandedName
	^ self nodeName expandWith: self namespaceURI! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 3/11/2010 23:19'!
hasAttributes
	^ self attributeNodes notEmpty! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 10/16/2010 06:07'!
hasNamespaces
	^ namespaceScope notNil and: [namespaceScope hasNamespaces]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/26/2010 07:31'!
hasPrefix
	^ self nodeName isPrefixed! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 8/10/2010 04:11'!
hasStringNodes
	^ self nodes anySatisfy: [:each | each isStringNode]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:23'!
includesAttribute: aName
	^ self attributeNodes includesAttribute: aName.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 11/25/2010 10:39'!
inheritNamespacesFrom: anElement
	self allElementsDo: [:each |
		each namespaceScope inheritMappingsFrom: anElement namespaceScope]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:54'!
isDeclaredPrefix: aPrefix
	^ self namespaceScope includesPrefix: aPrefix! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:54'!
isDeclaredPrefix: aPrefix uri: aUri
	^ self namespaceScope includesPrefix: aPrefix mappedTo: aUri! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ true! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 11/26/2010 12:04'!
isNamed: aName
	"Does the receiver's qualified or local name match the argument?'"
	^ self nodeName isSameAs: aName! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 11/26/2010 12:04'!
isNamedAny: aNameCollection
	"Does the receiver's qualified or local name match any of the names supplied as the argument?'"
	^ aNameCollection anySatisfy: [:each | self isNamed: each]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 11/26/2010 12:03'!
isRoot
	"Is the receiver the root element?"
	^ (self hasParent and: [self parent isElement]) not! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:16'!
localName
	^ self nodeName localPart! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 00:14'!
name
	^ self nodeName qualified! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:19'!
name: aName
	self nodeName qualified: aName! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/23/2010 13:40'!
name: aQualifiedName namespaceURI: aUri
	aQualifiedName splitQualifiedNameInto: [:prefix :localPart | self declarePrefix: prefix uri: aUri].
	self name: aQualifiedName.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 6/20/2010 23:10'!
namespaceScope
	^ namespaceScope ifNil: [namespaceScope := XMLNamespaceScope new]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 3/14/2010 12:52'!
namespaceURI
	^ self resolvePrefix: self prefix! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:56'!
nextElement
	^ self parent ifNotNil: [self parent elementAfter: self]! !

!XMLElement methodsFor: 'private' stamp: 'monty 12/9/2013 04:02'!
nodeName
	^ name
		ifNil: [ 
			(name := XMLNodeName new)
				addObserver: self;
				yourself]! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 7/11/2010 02:25'!
nodeNameChangedFrom: anOldName to: aNewName
	self validateName.
	self
		notifyOf: #renamedElement:from:to:
		with: self
		with: anOldName
		with: aNewName! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 3/17/2010 03:48'!
prefix
	^ self nodeName prefix! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/15/2010 15:15'!
prefix: aPrefix
	self nodeName prefix: aPrefix! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:57'!
previousElement
	^ self parent ifNotNil: [self parent elementBefore: self]! !

!XMLElement methodsFor: 'printing' stamp: 'JAAyer 11/24/2010 14:30'!
printNamespaceDeclarationsOn: aTagWriter
	(self declaresDefaultNamespace
		or: [self defaultNamespace notEmpty and: [aTagWriter isEmbedded not]])
		ifTrue: [aTagWriter xmlns: self defaultNamespace].

	self namespaceScope prefixesAndURIsDo: [:prefix :uri |
		((self declaresPrefix: prefix uri: uri) or: [aTagWriter isEmbedded not])
			ifTrue: [aTagWriter xmlnsAt: prefix put: uri]]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 12/6/2010 21:24'!
removeAttribute: aName
	^ (self attributeNodes removeAttributeNodeAt: aName) value.! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:54'!
removeAttributeNode: aNode
	^ self attributeNodes remove: aNode! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 08:42'!
removeAttributes
	^ self attributeNodes removeAll! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 10/24/2010 18:08'!
removedAttribute: anAttribute
	anAttribute
		removeObserver: self;
		element: nil! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 8/6/2010 20:16'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self validateAttributeName: aNewName! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/20/2010 23:08'!
resolvePrefix: aPrefix
	^ self namespaceScope resolvePrefix: aPrefix! !

!XMLElement methodsFor: 'initialization' stamp: 'JAAyer 7/12/2010 14:40'!
setAttributes: aDictionaryOrArray
	self attributeNodes removeAll.
	aDictionaryOrArray associationsDo: [:each |
		self attributeNodes
			addName: each key
			value: each value].! !

!XMLElement methodsFor: 'initialization' stamp: 'monty 12/27/2013 03:42'!
setNamespaceScope: aNamespaceScope
	namespaceScope := aNamespaceScope! !

!XMLElement methodsFor: 'initialization' stamp: 'monty 12/27/2013 03:43'!
setQualifiedName: aQualifiedName prefix: aPrefix localName: aLocalName attributes: anAttributeDictionary
	"This is used instead of using #nodeName's ifNil: initialization to avoid
	sending a name-changed notification"
	(name := XMLNodeName new)
		setQualified: aQualifiedName prefix: aPrefix localPart: aLocalName;
		addObserver: self.

	anAttributeDictionary
		ifNotEmpty: [
			"By not observing 'attributes' until after it is filled, we avoid
			re-validating the attributes, which the parser should have already done"
			(attributes := XMLAttributeList new: anAttributeDictionary size)
				setFromDictionary: anAttributeDictionary andElement: self;
				addObserver: self].! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:08'!
stringNodes
	^ self nodes select: [:each | each isStringNode]! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 7/22/2010 08:08'!
stringNodesDo: aBlock
	self nodes do: [:each | each isStringNode ifTrue: [aBlock value: each]]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 01:44'!
strings
	^ self stringNodes collect: [:each | each string]! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 8/9/2010 01:44'!
stringsDo: aBlock
	self stringNodesDo: [:each | aBlock value: each string]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/23/2010 13:36'!
undeclarePrefix: aPrefix
	self namespaceScope unmapPrefix: aPrefix! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 11/24/2010 12:23'!
updateEnclosedScopesWith: aBlock whileTrue: aConditionalBlock
	"Updates the receiver's scope and the scopes of all descendants with aBlock
	where aConditional is true. It stops descending down a path of a subtree
	as soon as aConditional is false, but continues down other paths."

	(aConditionalBlock value: self namespaceScope)
		ifTrue: [aBlock value: self namespaceScope]
		ifFalse: [^ self].

	self elementsDo: [:each |
		each
			updateEnclosedScopesWith: aBlock
			whileTrue: aConditionalBlock]! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 8/6/2010 23:46'!
validateAttributeName: anAttributeName
	self usesNamespaces ifFalse: [^ self].

	self namespaceScope
		validatePrefixedAttributeName: anAttributeName
		in: self attributeNodes attributesByName.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 8/7/2010 13:42'!
validateName
	(self usesNamespaces and: [self hasPrefix])
		ifTrue: [
			self namespaceScope
				validatePrefix: self prefix
				ofElementName: self name]! !

!XMLElement methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	| tagWriter |

	(tagWriter := aWriter tag)
		name: self name.

	self hasNamespaces
		ifTrue: [self printNamespaceDeclarationsOn: tagWriter].

	tagWriter
		attributes: self attributes;
		with: [self writeInnerXMLOn: aWriter].! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
addElement: anElement
	self addNode: anElement! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'JAAyer 11/24/2010 07:16'!
addedElement: anElement
	anElement
		configuration: self configuration;
		document: self document! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 12/4/2010 12:32'!
allElements
	^ self allElementsCollect: [:each | each]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 12/9/2013 02:51'!
allElementsCollect: aBlock
	| collectedElements |

	collectedElements := self nodes copyEmpty.
	self allElementsDo: [:each | collectedElements add: (aBlock value: each)].
	^ collectedElements.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:47'!
allElementsDetect: aBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."

	^ self allElementsDetect: aBlock ifNone: [nil]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 14:20'!
allElementsDetect: aBlock ifNone: anExceptionBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."

	self allElementsDo: [:each | (aBlock value: each) ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:37'!
allElementsDo: aBlock
	"Descend depth-first visiting each element with aBlock."

	self hasElements
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/25/2010 13:58'!
allElementsNamed: aName
	^ self allElementsSelect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 13:17'!
allElementsNamed: aName do: aBlock
	self allElementsDo: [:each |
		(each isNamed: aName)
			ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/25/2010 13:30'!
allElementsNamed: aName namespaceURI: aUri
	^ (self allElementsNamed: aName) select: [:each | each namespaceURI = aUri]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 12/9/2013 02:51'!
allElementsSelect: aBlock
	| selectedElements |

	selectedElements := self nodes copyEmpty.
	self allElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedElements add: each]].
	^ selectedElements.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:49'!
configuration
	^ configuration ifNil: [
		self configuration: XMLDOMConfiguration new.
		configuration]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 18:19'!
configuration: aConfiguration
	configuration := aConfiguration.

	self hasElements
		ifTrue: [self elementsDo: [:each | each configuration: aConfiguration]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:09'!
contentNodes
	^ self nodes select: [:each | each isContentNode]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 7/22/2010 08:09'!
contentNodesDo: aBlock
	self nodes do: [:each | each isContentNode ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 16:14'!
contentStringAt: aName
	^ (self elementAt: aName ifAbsent: [^ '']) contentString! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 03:19'!
document
	self subclassResponsibility! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 09:26'!
document: aDocument 
	self subclassResponsibility! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 3/12/2010 11:45'!
elementAfter: anElement
	^ self elements after: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:31'!
elementAt: aName
	^ self nodes elementAt: aName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName ifAbsent: aBlock
	^ self nodes elementAt: aName ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri
	^ self nodes elementAt: aName namespaceURI: aUri! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri ifAbsent: aBlock
	^ self nodes elementAt: aName namespaceURI: aUri ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 3/12/2010 11:45'!
elementBefore: anElement
	^ self elements before: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 5/4/2010 13:18'!
elements
	^ self nodes elements! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementsAt: aName
	^ self nodes elementsAt: aName! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:32'!
elementsAt: aName do: aBlock
	(self nodes elementsAt: aName) do: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:33'!
elementsAt: aName namespaceURI: aUri
	^ self nodes elementsAt: aName namespaceURI: aUri! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsCollect: aBlock
	^ self nodes elementsCollect: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock
	^ self nodes elementsDetect: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self nodes elementsDetect: aBlock ifNone: anExceptionBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 6/26/2010 00:55'!
elementsDo: aBlock
	self nodes elementsDo: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:36'!
elementsSelect: aBlock
	^ self nodes elementsSelect: aBlock! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 17:24'!
findElementNamed: aName
	^ self allElementsDetect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 17:24'!
findElementNamed: aName namespaceURI: aUri
	^ self allElementsDetect: [:each | (each isNamed: aName) and: [each namespaceURI = aUri]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 17:24'!
findElementNamed: aName namespaceURI: aUri with: aBlock
	^ self allElementsDetect: [:each |
			(each isNamed: aName)
				and: [each namespaceURI = aUri
					and: [aBlock value: each]]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 13:48'!
findElementNamed: aName with: aBlock
	^ self allElementsDetect: [:each | (each isNamed: aName) and: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/26/2013 21:53'!
firstElement
	^ self elements first! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 10/19/2010 11:13'!
hasContentNodes
	^ self nodes anySatisfy: [:each | each isContentNode]! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 11/24/2010 09:27'!
hasElements
	"Faster version used when traversing to avoid initializing the node list
	in leaf elements"
	^ nodes notNil and: [nodes hasElements]! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:21'!
includesElement: aName
	^ self nodes includesElementAt: aName! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 12/27/2013 03:37'!
innerXMLParserOn: aStringOrStream
	^ self configuration parserHandlerClass
		on: aStringOrStream
		withRootNode: self! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:15'!
isContentNode
	^ true! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/26/2013 21:53'!
lastElement
	^ self elements last! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 8/7/2010 20:11'!
namespaceScope
	^ XMLNamespaceScope new! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 11/26/2013 22:23'!
nodeListClass
	^ self configuration nodeListClass! !

!XMLNodeWithElements methodsFor: 'printing' stamp: 'JAAyer 5/5/2010 15:38'!
printContentOn: aStream
	self contentNodesDo: [:each | each printContentOn: aStream]! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'JAAyer 7/21/2010 12:01'!
removedElement: anElement
	anElement document: nil! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'JAAyer 6/22/2010 22:41'!
renamedElement: anElement from: anOldName to: aNewName! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 11/23/2010 14:48'!
usesNamespaces
	"Uses direct access and isNil test for performance"
	^ configuration isNil or: [configuration usesNamespaces]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 12:59'!
usesNamespaces: aBoolean
	self configuration usesNamespaces: aBoolean! !

XMLNode subclass: #XMLPI
	instanceVariableNames: 'target data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPI commentStamp: 'JAAyer 6/25/2010 14:06' prior: 0!
This class represents a processing instruction.
!

!XMLPI class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
target: targetName data: aString
	^ self new
		target: targetName;
		data: aString! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
data
	^ data ifNil: [data := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
data: aString
	data := aString! !

!XMLPI methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ true! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/21/2010 12:42'!
target
	^ target ifNil: [target := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
target: aString
	target := aString! !

!XMLPI methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter pi
		target: self target;
		data: self data! !

XMLNode subclass: #XMLString
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLString commentStamp: '<historical>' prior: 0!
This class represents string nodes, or nodes that contain no markup. String nodes can be created directly using the class-side #string: message or by sending a String object #asXMLString. When printing, they escape special characters in their contents.!

XMLString subclass: #XMLCData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLCData commentStamp: '<historical>' prior: 0!
This class represents a preserved CDATA section from a parsed document. It behaves like its superclass, except that its printed form differs in that it contains a "<!![CDATA[" prefix and a "]]>" suffix and its contents are NOT escaped. By default, CDATA sections are converted to regular string nodes during parsing. To preserve them as CDATA nodes, use #preservesCDataNodes: with the DOM parser before parsing.!

!XMLCData methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ true! !

!XMLCData methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter cdata: self string! !

!XMLString class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
string: aString
	^ self new string: aString! !

!XMLString methodsFor: 'accessing' stamp: 'monty 1/1/2014 07:23'!
addContent: aStringOrStringNode
	string :=
		string
			ifNil: [aStringOrStringNode asString]
			ifNotNil: [string, aStringOrStringNode asString]! !

!XMLString methodsFor: 'private' stamp: 'monty 1/1/2014 07:14'!
asString
	^ self string! !

!XMLString methodsFor: 'converting' stamp: 'JAAyer 7/22/2010 05:19'!
asXMLString
	^ self! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:26'!
isContentNode
	^ true! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:35'!
isStringNode
	^ true! !

!XMLString methodsFor: 'printing' stamp: 'JAAyer 9/2/2010 13:25'!
printContentOn: aStream
	| lastChar |

	self string ifEmpty: [^ self].

	((lastChar := aStream peekLast) isNil
		or: [lastChar isSeparator
			or: [self string first isSeparator]])
		ifFalse: [aStream space].

	aStream nextPutAll: self string.! !

!XMLString methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 05:19'!
string
	^ string ifNil: [string := '']! !

!XMLString methodsFor: 'accessing' stamp: 'monty 1/1/2014 07:23'!
string: aString
	string := aString asString! !

!XMLString methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter string: self string! !

Object subclass: #XMLNodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeFactory commentStamp: '<historical>' prior: 0!
This class is the default node factory. Its accessor messages return the class objects used by the DOM parser to build a node tree. You can override these messages to return different class objects and inject an instance of your factory using the DOM parser's #nodeFactory: accessor before parsing.!

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:58'!
attributeDeclarationClass
	^ XMLAttributeDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
cdataClass
	^ XMLCData! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:20'!
classForElement: aQualifiedName prefix: aPrefix uri: aNamespaceUri localName: aLocalName
	^ self elementClass! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
commentClass
	^ XMLComment! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
doctypeDeclarationClass
	^ XMLDoctypeDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
documentClass
	^ XMLDocument! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
elementClass
	^ XMLElement! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
elementDeclarationClass
	^ XMLElementDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
generalEntityDeclarationClass
	^ XMLGeneralEntityDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
notationDeclarationClass
	^ XMLNotationDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
parameterEntityDeclarationClass
	^ XMLParameterEntityDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
piClass
	^ XMLPI! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
stringClass
	^ XMLString! !

XMLNodeFactory subclass: #XMLPluggableElementFactory
	instanceVariableNames: 'elementClass elementClassesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPluggableElementFactory commentStamp: '<historical>' prior: 0!
This class is a pluggable factory that can map elements to different XMLElement subclasses based on the name and namespace information of those elements. You have to create an instance, configure it to handle certain elements with certain classes, and then inject your instance into a DOM parser using #nodeFactory: before parsing. Here is an example of its use:

	doc := (XMLDOMParser on: someXML)
		nodeFactory:
			(XMLPluggableElementFactory new
				elementClass: GenericElement;
				handleElement: 'user' withClass: UserElement;
				handleElement: 'report' withClass: ReportElement;
				handleElement: 'report' namespaceURI: 'urn:specialreprot' withClass: SpecialReportElement)
		parseDocument.
		
The #handleElementsMatchingClass*: forms try to match elements to the specified classes based on the names of those classes when stripped of any prefix (like XML) and "Element" suffix and converted to both camel case and a hyphenated form. So this:
	XMLPluggableElementFactory new
		handleElementsMatchingClass: MYRootUserElement
		....
will handle 'rootUser' and 'root-user' elements with the MYRootUserElement class.!

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'JAAyer 11/14/2010 21:57'!
classForElement: aName prefix: aPrefix uri: aUri localName: aLocalName
	^ (self classForElement: aName uri: aUri)
		ifNil: [
			aPrefix
				ifEmpty: [self elementClass]
				ifNotEmpty: [
					(self classForElement: aLocalName uri: aUri)
						ifNil: [self elementClass]]]! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/14/2010 21:55'!
classForElement: aName uri: aUri
	| classesForElement |

	classesForElement :=
		self elementClassesByName
			at: aName
			ifAbsent: [^ nil].

	^ classesForElement
		at: aUri
		ifAbsent: [
			classesForElement
				at: ''
				ifAbsent: [nil]].! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'JAAyer 11/15/2010 05:05'!
elementClass
	^ elementClass ifNil: [elementClass := super elementClass]! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'JAAyer 11/15/2010 05:04'!
elementClass: anElementClass
	elementClass := anElementClass! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/13/2010 08:36'!
elementClassesByName
	^ elementClassesByName ifNil: [elementClassesByName := Dictionary new]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/13/2010 19:37'!
handleElement: anElementName namespaceURI: aUri withClass: aClass
	(self elementClassesByName
		at: anElementName
		ifAbsentPut: [Dictionary new])
			at: aUri put: aClass! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/13/2010 19:37'!
handleElement: anElementName withClass: aClass
	self
		handleElement: anElementName
		namespaceURI: ''
		withClass: aClass! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:39'!
handleElements: anElementNameCollection namespaceURI: aNamespaceURI withClass: aClass
	anElementNameCollection do: [:each |
		self
			handleElement: each
			namespaceURI: aNamespaceURI
			withClass: aClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:36'!
handleElements: anElementNameCollection withClass: aClass
	anElementNameCollection do: [:each | self handleElement: each withClass: aClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/22/2010 11:10'!
handleElementsMatchingClass: aClass
	| className |

	(className := self toNameWithoutPrefixAndElementSuffix: aClass name asString)
		ifEmpty: [^ self].

	self
		handleElement: (self toCamelCasedName: className)
		withClass: aClass.
	self
		handleElement: (self toHyphenatedName: className)
		withClass: aClass.! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:26'!
handleElementsMatchingClasses: aClassCollection
	aClassCollection do: [:each | self handleElementsMatchingClass: each]! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:29'!
toCamelCasedName: aString
	^ aString allButFirst copyWithFirst: aString first asLowercase! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:34'!
toHyphenatedName: aString
	| nameStream hyphenatedStream previousChar nextChar |

	nameStream := aString readStream.
	hyphenatedStream := (String new: nameStream size) writeStream.
	[nameStream atEnd]
		whileFalse: [
			((nextChar := nameStream next) isUppercase
				and: [previousChar notNil and: [previousChar isLowercase]])
				ifTrue: [hyphenatedStream nextPut: $-].
			hyphenatedStream nextPut: nextChar asLowercase.
			previousChar := nextChar].
	^ hyphenatedStream contents.! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/22/2010 11:10'!
toNameWithoutPrefixAndElementSuffix: aString
	| nameStream |

	nameStream := aString readStream.
	[nameStream atEnd not and: [nameStream peek isUppercase]]
		whileTrue: [nameStream next].
	nameStream atEnd
		ifTrue: [^ ''].
	nameStream back.

	^ nameStream upToEnd copyReplaceAll: 'Element' with: ''.! !

Object subclass: #XMLNodeName
	instanceVariableNames: 'qualified prefix localPart notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeName commentStamp: '<historical>' prior: 0!
The class represents a name for an element or attribute. It stores the prefix, local part and qualified name. It also notifies any observers whenever any part of the name it models changes using #nodeNameChangedFrom:to:. You should not need to use this class directly.!

!XMLNodeName methodsFor: 'observing' stamp: 'OttoBehrens 2/15/2011 08:18'!
addObserver: anObserver
	self notifier addObserver: anObserver! !

!XMLNodeName methodsFor: 'private' stamp: 'JAAyer 11/1/2010 04:36'!
changeWith: aBlock
	| oldName newName |

	oldName := self qualified.
	aBlock value.
	newName := self qualified.
	newName = oldName
		ifFalse: [self notifyOf: #nodeNameChangedFrom:to: with: oldName with: newName].! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 15:01'!
expandWith: aUri
	^ aUri
		ifEmpty: [self localPart]
		ifNotEmpty: ['{', aUri, '}', self localPart]! !

!XMLNodeName methodsFor: 'testing' stamp: 'JAAyer 8/9/2010 14:46'!
isPrefixed
	^ self prefix notEmpty! !

!XMLNodeName methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:00'!
isSameAs: aNameString
	^ self qualified = aNameString or: [self localPart = aNameString]! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:23'!
localPart
	^ localPart ifNil: [localPart := '']! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 11/21/2010 04:57'!
localPart: aLocalPart
	self changeWith: [self setPrefix: self prefix localPart: aLocalPart]! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 6/15/2010 15:18'!
notifier
	^ notifier ifNil: [notifier := XMLNotifier new]! !

!XMLNodeName methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:18'!
notifyOf: aSelector with: anArgument
	self notifier notifyOf: aSelector with: anArgument! !

!XMLNodeName methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:18'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument
	self notifier notifyOf: aSelector with: aFirstArgument with: aSecondArgument! !

!XMLNodeName methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:18'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument with: aThirdArgument
	self notifier
		notifyOf: aSelector
		with: aFirstArgument
		with: aSecondArgument
		with: aThirdArgument! !

!XMLNodeName methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:18'!
notifyOf: aSelector withArguments: anArray
	self notifier notifyOf: aSelector withArguments: anArray! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 14:43'!
prefix
	^ prefix ifNil: [prefix := '']! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:17'!
prefix: aPrefix
	self changeWith: [self setPrefix: aPrefix localPart: self localPart]! !

!XMLNodeName methodsFor: 'printing' stamp: 'JAAyer 6/27/2010 15:15'!
printOn: aStream
	aStream nextPutAll: self qualified! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:23'!
qualified
	^ qualified ifNil: [qualified := '']! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:19'!
qualified: aQualifiedName
	aQualifiedName splitQualifiedNameInto: [:prefixPart :localName |
		self changeWith: [self setPrefix: prefixPart localPart: localName]]! !

!XMLNodeName methodsFor: 'observing' stamp: 'OttoBehrens 2/15/2011 08:18'!
removeObserver: anObserver
	self notifier removeObserver: anObserver! !

!XMLNodeName methodsFor: 'private' stamp: 'JAAyer 8/14/2010 20:50'!
setPrefix: aPrefix localPart: aLocalPart
	aPrefix isEmptyOrNil
		ifTrue: [
			self
				setQualified: aLocalPart
				prefix: ''
				localPart: aLocalPart]
		ifFalse: [
			self
				setQualified: aPrefix, ':', aLocalPart
				prefix: aPrefix
				localPart: aLocalPart]! !

!XMLNodeName methodsFor: 'private' stamp: 'monty 12/9/2013 04:47'!
setQualified: aQualifiedName
	(qualified := aQualifiedName)
		splitQualifiedNameInto: [:aPrefix :aLocalPart |
			prefix := aPrefix.
			localPart := aLocalPart].! !

!XMLNodeName methodsFor: 'private' stamp: 'JAAyer 6/27/2010 15:20'!
setQualified: aQualifiedName prefix: aPrefix localPart: aLocalPart
	qualified := aQualifiedName.
	prefix := aPrefix.
	localPart := aLocalPart.! !

Object subclass: #XMLNotifier
	instanceVariableNames: 'observers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNotifier commentStamp: '<historical>' prior: 0!
This class implements the observer pattern.

Classes using it need to forward its entire protocol to a private instance. Objects will then be able to register themselves as observers of it and receive notifications from it, which are selectors that the observers are expected to implement, typically under the category "notifications."

It stores observers in an Array grown and shrunk with #copyWith: and #copyWithout: rather than an OrderedCollection for better memory usage and faster performance (because each node only has at most two observers).!

!XMLNotifier methodsFor: 'observing' stamp: 'JAAyer 7/20/2010 10:05'!
addObserver: anObject
	observers :=
		observers
			ifNil: [Array with: anObject]
			ifNotNil: [observers copyWith: anObject]! !

!XMLNotifier methodsFor: 'notification' stamp: 'JAAyer 7/20/2010 10:04'!
notifyOf: aSelector with: anArgument
	observers ifNotNil: [
		observers do: [:each |
			each perform: aSelector with: anArgument]]! !

!XMLNotifier methodsFor: 'notification' stamp: 'JAAyer 7/20/2010 10:04'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument
	observers ifNotNil: [
		observers do: [:each |
			each
				perform: aSelector
				with: aFirstArgument
				with: aSecondArgument]]! !

!XMLNotifier methodsFor: 'notification' stamp: 'JAAyer 7/20/2010 10:04'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument with: aThirdArgument
	observers ifNotNil: [
		observers do: [:each |
			each
				perform: aSelector
				with: aFirstArgument
				with: aSecondArgument
				with: aThirdArgument]]! !

!XMLNotifier methodsFor: 'notification' stamp: 'JAAyer 7/20/2010 10:05'!
notifyOf: aSelector withArguments: anArray
	observers ifNotNil: [
		observers do: [:each |
			each perform: aSelector withArguments: anArray]]! !

!XMLNotifier methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:02'!
observers
	^ observers ifNil: [#()]! !

!XMLNotifier methodsFor: 'observing' stamp: 'JAAyer 7/20/2010 10:03'!
removeObserver: anObject
	(observers := observers copyWithout: anObject)
		ifEmpty: [observers := nil]! !

Object subclass: #XMLParser
	instanceVariableNames: 'tokenizer driver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParser commentStamp: '<historical>' prior: 0!
This class is a facade that parses XML using a driver and tokenizer. See SAXHandler and XMLDOMParser for ready-made front ends to it.!

!XMLParser class methodsFor: 'instance creation' stamp: 'JAAyer 11/28/2010 07:09'!
driver: aDriver on: aStringOrStream
	^ self new setDriver: aDriver tokenizer: (aDriver tokenizerOn: aStringOrStream)! !

!XMLParser methodsFor: 'testing' stamp: 'JAAyer 11/27/2010 07:17'!
atEnd
	^ self tokenizer atEnd! !

!XMLParser methodsFor: 'accessing' stamp: 'monty 11/14/2013 23:34'!
currentLineNumber
	^ self tokenizer currentLineNumber! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 07:07'!
driver
	^ driver! !

!XMLParser methodsFor: 'parsing' stamp: 'JAAyer 11/27/2010 07:15'!
parseDocument
	[self atEnd] whileFalse: [self parseToken]! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 12/19/2013 07:56'!
parseDocumentWhile: aBlock
	[self atEnd not and: [aBlock value]] whileTrue: [self parseToken]! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 11/16/2013 22:08'!
parseToken
	^ [self tokenizer nextToken]
		on: XMLParserException
		do: [:error |
			error signal:
				error messageText,
				' (line ', self currentLineNumber printString, ')'].! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 07:09'!
setDriver: aDriver tokenizer: aTokenizer
	driver := aDriver.
	tokenizer := aTokenizer.! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 07:16'!
tokenizer
	^ tokenizer! !

Object subclass: #XMLParserDriver
	instanceVariableNames: 'activeEntityUri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParserDriver commentStamp: '<historical>' prior: 0!
This is a base class for parser drivers. Drivers are sent messages by a tokenizer to handle various XML tokens as they are read. To create a driver, subclass this class and override the handler messages and testing messages that control tokenization.!

XMLParserDriver subclass: #DTDExternalSubsetDriver
	instanceVariableNames: 'saxDriver documentUri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!DTDExternalSubsetDriver commentStamp: '<historical>' prior: 0!
This class is a driver for parsing the external subset of a DTD.!

!DTDExternalSubsetDriver class methodsFor: 'instance creation' stamp: 'monty 11/19/2013 06:10'!
saxDriver: aSaxDriver documentURI: anExternalSubsetUri
	^ self new
		setSAXDriver: aSaxDriver
		documentURI: anExternalSubsetUri ! !

!DTDExternalSubsetDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:11'!
documentURI
	^ documentUri! !

!DTDExternalSubsetDriver methodsFor: 'accessing' stamp: 'monty 1/4/2014 03:50'!
dtd
	"direct acceess for speed"
	^ saxDriver dtd! !

!DTDExternalSubsetDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:04'!
externalEntityResolver
	^ self saxDriver externalEntityResolver! !

!DTDExternalSubsetDriver methodsFor: 'handling tokens' stamp: 'monty 12/30/2013 23:55'!
handleAttributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self isValidating
		ifTrue: [
			self dtd
				addValidatorForAttributeIn: anElement
				named: aName
				type: aType
				defaultPragma: aDefaultPragma
				defaultValue: aDefaultValue]! !

!DTDExternalSubsetDriver methodsFor: 'handling tokens' stamp: 'monty 12/10/2013 06:32'!
handleElementDeclaration: aName contentModel: aContentModel
	self isValidating
		ifTrue: [
			self dtd
				addValidatorForElement: aName
				contentModel: aContentModel]! !

!DTDExternalSubsetDriver methodsFor: 'handling tokens' stamp: 'monty 12/14/2013 02:15'!
handleGeneralEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation
	aNotation ifNotEmpty: [
		self dtd
			addUnparsedEntity: aName
			ndata: aNotation.
		^ self].

	self dtd
		addExternalGeneralEntity: aName
		resolver: self externalEntityResolver
		uri: aSystemId
		from: self currentURI.! !

!DTDExternalSubsetDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:52'!
handleGeneralEntityDeclaration: aName replacement: aReplacement
	self dtd
		addGeneralEntity: aName
		replacement: aReplacement
		from: self currentURI! !

!DTDExternalSubsetDriver methodsFor: 'handling tokens' stamp: 'monty 11/19/2013 02:20'!
handleNotationDeclaration: aName publicID: aPublicId systemID: aSystemId
	self dtd addNotation: aName! !

!DTDExternalSubsetDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:55'!
handleParameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId
	self dtd
		addExternalParameterEntity: aName
		resolver: self externalEntityResolver
		uri: aSystemId
		from: self currentURI! !

!DTDExternalSubsetDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:53'!
handleParameterEntityDeclaration: aName replacement: aReplacement
	self dtd
		addParameterEntity: aName
		replacement:aReplacement
		from: self currentURI! !

!DTDExternalSubsetDriver methodsFor: 'handling tokens' stamp: 'monty 11/19/2013 06:41'!
handleParameterEntityReference: aName
	| entity |

	(entity := self dtd parameterEntityAt: aName)
		ifNil: [
			self isValidating
				ifTrue: [
					XMLValidationException signal:
						'Undeclared parameter entity reference %', aName, ';']].
	^ entity.! !

!DTDExternalSubsetDriver methodsFor: 'testing' stamp: 'monty 1/4/2014 03:49'!
isValidating
	"direct acceess for speed"
	^ saxDriver isValidating! !

!DTDExternalSubsetDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:04'!
saxDriver
	^ saxDriver! !

!DTDExternalSubsetDriver methodsFor: 'initialization' stamp: 'monty 11/19/2013 06:10'!
setSAXDriver: aSaxdriver documentURI: aUri
	saxDriver := aSaxdriver.
	documentUri := aUri.! !

!DTDExternalSubsetDriver methodsFor: 'initialization' stamp: 'monty 11/19/2013 06:51'!
tokenizerOn: aStringOrStream
	| tokenizer |

	tokenizer := super tokenizerOn: aStringOrStream.
	tokenizer tokenContext enterExternalSubsetTextDeclaration.

	^ tokenizer.! !

!DTDExternalSubsetDriver methodsFor: 'testing' stamp: 'monty 11/30/2013 21:55'!
usesNamespaces
	^ self saxDriver usesNamespaces! !

XMLParserDriver subclass: #SAXDriver
	instanceVariableNames: 'saxHandler elementNester scopeAllocator dtd validator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXDriver commentStamp: '<historical>' prior: 0!
SAXDriver together with SAXHandler provides a SAX-like API for XML parsing. A SAXDriver processes XML tokens and signals SAX events to a SAXHandler. To use it, create a subclass of SAXHandler and override messages to handle SAX events of interest. See "instance creation" on the class-side of SAXHandler for ways to instantiate such a parser.!

SAXDriver subclass: #HTMLDriver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTML'!

!HTMLDriver methodsFor: 'private' stamp: 'monty 12/27/2013 04:07'!
elementNesterClass
	^ HTMLElementNester! !

!HTMLDriver methodsFor: 'element nesting' stamp: 'monty 12/13/2013 04:45'!
endNestingOfElement: anElementName
	((self elementNester isVoidElement: anElementName)
		or: [(self elementNester hasOpenElement: anElementName) not])
		ifTrue: [^ self].

	self preEndElements:
		(self elementNester elementsToEndBeforeEnding: anElementName).

	super endNestingOfElement: anElementName.! !

!HTMLDriver methodsFor: 'handling tokens' stamp: 'monty 11/22/2013 18:32'!
handleEndDocument
	self elementNester openElements
		reverseDo: [:each | self handleEndTag: each].

	super handleEndDocument.! !

!HTMLDriver methodsFor: 'handling tokens' stamp: 'monty 11/30/2013 19:00'!
handleNonHTMLSection: aString
	self saxHandler characters: aString! !

!HTMLDriver methodsFor: 'handling tokens' stamp: 'monty 11/30/2013 04:01'!
handlePCData: aString
	self
		preEndElements: self elementNester elementsToEndBeforePCData;
		preStartElements: self elementNester elementsToStartBeforePCData.

	super handlePCData: aString.! !

!HTMLDriver methodsFor: 'handling tokens' stamp: 'monty 11/22/2013 19:35'!
handleParseError: anError! !

!HTMLDriver methodsFor: 'handling tokens' stamp: 'monty 12/11/2013 22:45'!
handleStartTag: anElementName attributes: anAttributeDictionary namespaces: aNamespaceDictionary
	super
		handleStartTag: anElementName
		attributes: anAttributeDictionary
		namespaces: aNamespaceDictionary.

	(self elementNester isVoidElement: anElementName)
		ifTrue: [self preEndElements: (Array with: anElementName)].! !

!HTMLDriver methodsFor: 'handling tokens' stamp: 'monty 12/9/2013 07:00'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo
	super
		handleXMLVersion: (aVersion ifEmpty: '1.0')
		encoding: (anEncoding ifEmpty: 'UTF-8')
		standalone: 'no'! !

!HTMLDriver methodsFor: 'testing' stamp: 'monty 11/30/2013 04:13'!
isValidating
	^ false! !

!HTMLDriver methodsFor: 'element nesting' stamp: 'monty 12/11/2013 23:29'!
preEndElements: anElementCollection
	anElementCollection do: [:each |
		(self elementNester hasOpenElements)
			ifFalse: [^ self].

		self elementNester endElement: each.
	
		each splitQualifiedNameInto: [:prefixPart :localPart |
			self saxHandler 
				endElement: each
				prefix: prefixPart
				uri: ''
				localName: localPart]].! !

!HTMLDriver methodsFor: 'element nesting' stamp: 'monty 12/11/2013 23:27'!
preStartElements: anElementCollection
	anElementCollection do: [:each |
		self elementNester startElement: each.

		each splitQualifiedNameInto: [:prefixPart :localPart |
			self saxHandler 
				startElement: each
				prefix: prefixPart
				uri: ''
				localName: localPart
				attributes: Dictionary new]].! !

!HTMLDriver methodsFor: 'testing' stamp: 'monty 11/30/2013 04:14'!
resolvesExternalEntities
	^ false! !

!HTMLDriver methodsFor: 'element nesting' stamp: 'monty 12/13/2013 04:45'!
startNestingOfElement: anElementName
	(self elementNester isOpenUniqueElement: anElementName)
		ifTrue: [^ self].

	self preEndElements:
		(self elementNester elementsToEndBeforeStarting: anElementName).
	self preStartElements:
		(self elementNester elementsToStartBeforeStarting: anElementName).

	super startNestingOfElement: anElementName.! !

!HTMLDriver methodsFor: 'private' stamp: 'monty 11/22/2013 09:48'!
tokenizerClass
	^ HTMLTokenizer! !

!HTMLDriver methodsFor: 'testing' stamp: 'monty 11/30/2013 04:13'!
usesNamespaces
	^ false! !

!SAXDriver class methodsFor: 'instance creation' stamp: 'monty 12/8/2013 02:28'!
saxHandler: aHandler
	^ self new setSaxHandler: aHandler! !

!SAXDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 04:13'!
currentElement
	^ self elementNester currentElement! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 20:36'!
currentScope
	^ self usesNamespaces
		ifTrue: [self scopeAllocator currentScope]
		ifFalse: [nil]! !

!SAXDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 03:40'!
currentScope: aNamespaceScope
	self usesNamespaces
		ifTrue: [self scopeAllocator currentScope: aNamespaceScope]! !

!SAXDriver methodsFor: 'namespaces' stamp: 'JAAyer 11/27/2010 20:36'!
declareNamespaces: aDictionary
	aDictionary keysAndValuesDo: [:prefix :uri |
		self scopeAllocator declarePrefix: prefix uri: uri.
		self saxHandler startPrefixMapping: prefix uri: uri]! !

!SAXDriver methodsFor: 'accessing' stamp: 'monty 11/16/2013 04:35'!
documentURI
	^ self saxHandler documentURI! !

!SAXDriver methodsFor: 'accessing' stamp: 'monty 11/14/2013 23:22'!
dtd
	^ dtd ifNil: [dtd := DTDDoctypeDefinition new]! !

!SAXDriver methodsFor: 'private' stamp: 'monty 12/27/2013 04:12'!
elementNester
	^ elementNester ifNil: [elementNester := self elementNesterClass new]! !

!SAXDriver methodsFor: 'private' stamp: 'monty 12/27/2013 04:08'!
elementNesterClass
	^ XMLWellFormedElementNester! !

!SAXDriver methodsFor: 'element nesting' stamp: 'monty 12/13/2013 04:46'!
endNestingOfElement: anElementName
	self elementNester endElement: anElementName! !

!SAXDriver methodsFor: 'accessing' stamp: 'monty 12/8/2013 02:33'!
externalEntityResolver
	^ saxHandler externalEntityResolver! !

!SAXDriver methodsFor: 'private' stamp: 'monty 12/12/2013 01:58'!
externalSubsetParserOnURI: aUri
	^ XMLParser
		driver:
			(DTDExternalSubsetDriver
				saxDriver: self
				documentURI: aUri)
		on: (self externalEntityResolver resolveEntity: aUri)
	! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/30/2013 23:56'!
handleAttributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self isValidating
		ifTrue: [
			self dtd
				addValidatorForAttributeIn: anElement
				named: aName
				type: aType
				defaultPragma: aDefaultPragma
				defaultValue: aDefaultValue].
	self saxHandler
		attributeDeclaration: anElement
		name: aName
		type: aType
		defaultPragma: aDefaultPragma
		defaultValue: aDefaultValue! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:22'!
handleCData: aString
	self saxHandler
		startCData;
		characters: aString;
		endCData! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/11/2013 22:26'!
handleComment: aString
	self saxHandler comment: aString! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/10/2013 06:32'!
handleElementDeclaration: aName contentModel: aContentModel
	self isValidating
		ifTrue: [
			self dtd
				addValidatorForElement: aName
				contentModel: aContentModel].

	self saxHandler
		elementDeclaration: aName
		contentModel: aContentModel! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2013 05:10'!
handleEndContentEntityReplacement: aName
	self saxHandler endEntity: aName! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 1/4/2014 03:55'!
handleEndDTD
	(self resolvesExternalEntities and: [self dtd hasExternalSubset])
		ifTrue: [self parseExternalSubset: self dtd externalSubsetURI].

	self saxHandler endDTD.! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/21/2013 02:22'!
handleEndDocument
	self elementNester ensureNoElementsRemainOpen.
	self isValidating
		ifTrue: [self validator validateEndDocument].

	self saxHandler endDocument.! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 04:46'!
handleEndTag: anElementName
	self endNestingOfElement: anElementName.

	anElementName splitQualifiedNameInto: [:prefixPart :localPart |
		self saxHandler 
			endElement: anElementName
			prefix: prefixPart
			uri:
				(self usesNamespaces
					ifTrue: [self scopeAllocator resolvePrefix: prefixPart]
					ifFalse: [''])
			localName: localPart.

		self validateEndTag: anElementName].! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/14/2013 02:15'!
handleGeneralEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation
	aNotation
		ifNotEmpty: [
			self dtd
				addUnparsedEntity: aName
				ndata: aNotation]
		ifEmpty: [
			self resolvesExternalEntities
				ifTrue: [
					self dtd
						addExternalGeneralEntity: aName
						resolver: self externalEntityResolver
						uri: aSystemId
						from: self currentURI]].

	self saxHandler
		generalEntityDeclaration: aName
		publicID: aPublicId
		systemID: aSystemId
		ndata: aNotation.! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:49'!
handleGeneralEntityDeclaration: aName replacement: aReplacement
	self dtd
		addGeneralEntity: aName
		replacement: aReplacement
		from: self currentURI.

	self saxHandler
		generalEntityDeclaration: aName
		replacement: aReplacement.! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/8/2013 02:22'!
handleGeneralEntityReference: aName
	| entity |

	(entity := self dtd generalEntityAt: aName)
		ifNil: [
			self isValidating
				ifTrue: [
					XMLValidationException signal:
						'Undeclared general entity reference &', aName, ';'].
			self preservesUndeclaredEntityReferences
				ifTrue: [entity := DTDBypassedGeneralEntity named: aName].
			self saxHandler skippedEntity: aName].
	^ entity.! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 11/19/2013 02:19'!
handleNotationDeclaration: aName publicID: aPublicId systemID: aSystemId
	self isValidating
		ifTrue: [self dtd addNotation: aName].

	self saxHandler
		notationDeclaration: aName
		publicID: aPublicId
		systemID: aSystemId.! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/18/2013 17:53'!
handlePCData: aString
	self isValidating
		ifTrue: [self  validator validatePCData: aString].

	self saxHandler 
		characters: aString! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:22'!
handlePI: piTarget data: piData
	self saxHandler 
		processingInstruction: piTarget data: piData! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:54'!
handleParameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId
	self resolvesExternalEntities
		ifTrue: [
			self dtd
				addExternalParameterEntity: aName
				resolver: self externalEntityResolver
				uri: aSystemId
				from: self currentURI].

	self saxHandler
		parameterEntityDeclaration: aName
		publicID: aPublicId
		systemID: aSystemId.! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:54'!
handleParameterEntityDeclaration: aName replacement: aReplacement
	self dtd
		addParameterEntity: aName
		replacement:aReplacement
		from: self currentURI.

	self saxHandler
		parameterEntityDeclaration: aName
		replacement: aReplacement.! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/8/2013 02:22'!
handleParameterEntityReference: aName
	| entity |

	(entity := self dtd parameterEntityAt: aName)
		ifNil: [
			self isValidating
				ifTrue: [
					XMLValidationException signal:
						'Undeclared parameter entity reference %', aName, ';'].
			self preservesUndeclaredEntityReferences
				ifTrue: [entity := DTDBypassedParameterEntity named: aName].
			self saxHandler skippedEntity: aName].
	^ entity.! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/18/2013 22:08'!
handleStartContentEntityReplacement: aName
	self saxHandler startEntity: aName! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 1/4/2014 03:46'!
handleStartDTD: aRoot publicID: aPublicId systemID: aSystemId
	self dtd
		rootElement: aRoot
		externalSubsetURI: aSystemId.
	
	self saxHandler
		startDTD: aRoot
		publicID: aPublicId
		systemID: aSystemId.! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 18:29'!
handleStartDocument
	self saxHandler startDocument! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/27/2013 07:51'!
handleStartTag: anElementName attributes: anAttributeDictionary namespaces: aNamespaceDictionary
	self startNestingOfElement: anElementName.

	anElementName splitQualifiedNameInto: [:prefixPart :localPart |
		self
			validateStartTag: anElementName
			prefix: prefixPart
			attributes: anAttributeDictionary
			namespaces: aNamespaceDictionary.

		self saxHandler 
			startElement: anElementName
			prefix: prefixPart
			uri:
				(self usesNamespaces
					ifTrue: [self scopeAllocator resolvePrefix: prefixPart]
					ifFalse: [''])
			localName: localPart
			attributes: anAttributeDictionary].

	self usesNamespaces
		ifTrue: [self saxHandler currentScope: self scopeAllocator currentScope].! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2013 04:01'!
handleUnparsedEntityReference: aName
	| notationName |

	notationName := self dtd unparsedEntityAt: aName.
	self isValidating
		ifTrue: [
			notationName
				ifNil: [XMLValidationException signal: 'Undeclared unparsed entity reference &', aName, ';'].
			(self dtd includesNotation: notationName)
				ifFalse: [XMLValidationException signal: 'Undeclared notation "', notationName, '"']].
	^ notationName.

	! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 12/11/2013 23:00'!
handleWhitespace: aString
	self saxHandler ignorableWhitespace: aString! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'monty 1/5/2014 00:52'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo
	| versionNumber standaloneBoolean |

	aVersion ifNotEmpty: [versionNumber := aVersion asNumber].
	standaloneBoolean := aYesOrNo = 'yes'.

	self dtd isStandalone: standaloneBoolean.

	self saxHandler
		xmlVersion: versionNumber
		encoding: anEncoding
		standalone: standaloneBoolean! !

!SAXDriver methodsFor: 'testing' stamp: 'monty 1/4/2014 03:49'!
isValidating
	"direct acceess for speed"
	^ saxHandler isValidating! !

!SAXDriver methodsFor: 'private' stamp: 'monty 12/12/2013 01:57'!
parseExternalSubset: aUri
	| parser |

	parser :=
		self externalSubsetParserOnURI:
			(self externalEntityResolver
				entityURIFromRelative: aUri
				andBase: self documentURI).
	parser parseDocument.! !

!SAXDriver methodsFor: 'testing' stamp: 'monty 1/4/2014 03:49'!
preservesUndeclaredEntityReferences
	^ self saxHandler preservesUndeclaredEntityReferences! !

!SAXDriver methodsFor: 'testing' stamp: 'monty 1/4/2014 03:49'!
resolvesExternalEntities
	^ self saxHandler resolvesExternalEntities! !

!SAXDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 04:14'!
rootElement
	^ self elementNester rootElement! !

!SAXDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 04:11'!
rootElement: anElementName
	self elementNester rootElement: anElementName! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 08:00'!
saxHandler
	^ saxHandler! !

!SAXDriver methodsFor: 'private' stamp: 'monty 12/27/2013 03:40'!
scopeAllocator
	^ scopeAllocator ifNil: [scopeAllocator := XMLNamespaceScopeAllocator new]! !

!SAXDriver methodsFor: 'initialization' stamp: 'monty 12/11/2013 23:00'!
setSaxHandler: aHandler
	saxHandler := aHandler! !

!SAXDriver methodsFor: 'element nesting' stamp: 'monty 12/13/2013 04:46'!
startNestingOfElement: anElementName
	self elementNester startElement: anElementName! !

!SAXDriver methodsFor: 'namespaces' stamp: 'JAAyer 11/27/2010 20:36'!
undeclareNamespaces
	| currentScope enclosingScope |

	currentScope := self scopeAllocator currentScope.
	currentScope hasNamespaces
		ifFalse: [^ self].
	enclosingScope := self scopeAllocator enclosingScope.

	currentScope prefixesAndURIsDo: [:prefix :uri |
		(enclosingScope includesPrefix: prefix mappedTo: uri)
			ifFalse: [self saxHandler endPrefixMapping: prefix]].

	(currentScope defaultNamespace notNil
		and: [currentScope defaultNamespace ~= enclosingScope defaultNamespace])
		ifTrue: [self saxHandler endPrefixMapping: '']! !

!SAXDriver methodsFor: 'testing' stamp: 'monty 1/4/2014 03:49'!
usesNamespaces
	"direct acceess for speed"
	^ saxHandler usesNamespaces! !

!SAXDriver methodsFor: 'validation' stamp: 'monty 11/19/2013 01:12'!
validateEndTag: anEndTag
	self usesNamespaces
		ifTrue: [
			self undeclareNamespaces.
			self scopeAllocator exitScope].
	self isValidating
		ifTrue: [self validator validateEndTag: anEndTag].! !

!SAXDriver methodsFor: 'validation' stamp: 'monty 12/21/2013 20:26'!
validateRootElement: aStartTag
	(self saxHandler requiresDocumentConstraints
		and: [self validator hasElementValidators not])
		ifTrue: [
			XMLValidationException signal:
				'Document validation constraints required'].

	self validator validateRootElement: aStartTag.! !

!SAXDriver methodsFor: 'validation' stamp: 'monty 12/14/2013 02:18'!
validateStartTag: aStartTag prefix: aPrefix attributes: anAttributeDictionary namespaces: aNamespaceDictionary
	self usesNamespaces
		ifTrue: [
			self scopeAllocator enterScope.

			aNamespaceDictionary
				ifNotNil: [self declareNamespaces: aNamespaceDictionary].

			aPrefix
				ifNotEmpty: [
					self scopeAllocator
						validatePrefix: aPrefix
						ofElementName: aStartTag].
			self scopeAllocator validatePrefixedAttributeNames: anAttributeDictionary].

	self isValidating
		ifTrue: [
			self elementNester totalOpen > 1
				ifFalse: [self validateRootElement: aStartTag].
		
			self validator
				validateStartTag: aStartTag
				attributes: anAttributeDictionary].! !

!SAXDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 01:10'!
validator
	^ validator ifNil: [validator := DTDDocumentValidator dtd: self dtd]! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 12/13/2013 18:19'!
activeEntityURI
	^ activeEntityUri! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 12/13/2013 18:19'!
activeEntityURI: aUri
	activeEntityUri := aUri! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 12/13/2013 18:19'!
currentURI
	^ self activeEntityURI ifNil: [self documentURI]! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 11/16/2013 04:35'!
documentURI
	^ nil! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 11/30/2013 04:17'!
dtd
	^ nil! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/30/2013 23:46'!
handleAttributeDeclaration: anElement name: anName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleCData: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleComment: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:46'!
handleElementDeclaration: aName contentModel: aContentModel! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2013 05:10'!
handleEndContentEntityReplacement: aName
! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/12/2013 17:36'!
handleEndDTD! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleEndDocument! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleEndTag: anElementName! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:08'!
handleGeneralEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:10'!
handleGeneralEntityDeclaration: aName replacement: aReplacement! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 20:44'!
handleGeneralEntityReference: aName
	^ nil! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:56'!
handleNotationDeclaration: aName publicID: aPublicId systemID: aSystemId! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handlePCData: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handlePI: piTarget data: piData! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:09'!
handleParameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:10'!
handleParameterEntityDeclaration: aName replacement: aReplacement! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 20:44'!
handleParameterEntityReference: aName
	^ nil! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 1/4/2014 07:42'!
handleParseError: anErrorString
	XMLWellFormednessException signal: anErrorString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2013 05:10'!
handleStartContentEntityReplacement: aName! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/12/2013 17:15'!
handleStartDTD: aRoot publicID: aPublicId systemID: aSystemId! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleStartDocument! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleStartTag: anElementName attributes: anAttributeDictionary namespaces: aNamespaceDictionary! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2013 03:47'!
handleUnparsedEntityReference: aName
	^ nil! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleWhitespace: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo! !

!XMLParserDriver methodsFor: 'testing' stamp: 'JAAyer 11/27/2010 07:24'!
isValidating
	self subclassResponsibility! !

!XMLParserDriver methodsFor: 'private' stamp: 'JAAyer 11/27/2010 07:42'!
tokenizerClass
	^ XMLTokenizer! !

!XMLParserDriver methodsFor: 'initialization' stamp: 'JAAyer 11/28/2010 07:16'!
tokenizerOn: aStringOrStream
	^ self tokenizerClass driver: self on: aStringOrStream! !

!XMLParserDriver methodsFor: 'testing' stamp: 'JAAyer 11/27/2010 07:24'!
usesNamespaces
	self subclassResponsibility! !

Object subclass: #XMLPeekableStreamAdapter
	instanceVariableNames: 'stream peekChar onClose'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLPeekableStreamAdapter commentStamp: '<historical>' prior: 0!
This class wraps any readable stream that at the very least supports #next and #atEnd and makes it peekable. Additionally, it ensures that #next and #peek can be used at the end of a stream safely without raising an exception (the Squeak and Pharo behavior).!

!XMLPeekableStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 12/13/2013 05:07'!
on: aStream
	^ self
		on: aStream
		onClose: nil! !

!XMLPeekableStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 12/13/2013 05:08'!
on: aStream onClose: aBlock
	^ self new
		setStream: aStream
		onClose: aBlock! !

!XMLPeekableStreamAdapter methodsFor: 'testing' stamp: 'JAAyer 12/14/2010 02:48'!
atEnd
	^ peekChar isNil and: [stream atEnd]! !

!XMLPeekableStreamAdapter methodsFor: 'file open/close' stamp: 'monty 11/16/2013 04:22'!
close
	stream close.
	onClose ifNotNil: [onClose value].! !

!XMLPeekableStreamAdapter methodsFor: 'accessing' stamp: 'JAAyer 12/14/2010 02:34'!
converter: aConverter
	(stream respondsTo: #converter:)
		ifTrue: [stream converter: aConverter]! !

!XMLPeekableStreamAdapter methodsFor: 'accessing' stamp: 'JAAyer 12/14/2010 02:37'!
next
	| nextChar |

	peekChar
		ifNotNil: [
			nextChar := peekChar.
			peekChar := nil]
		ifNil: [
			nextChar := stream atEnd
				ifTrue: [nil]
				ifFalse: [stream next]].
	^ nextChar.! !

!XMLPeekableStreamAdapter methodsFor: 'accessing' stamp: 'JAAyer 12/14/2010 02:18'!
peek
	^ peekChar
		ifNil: [
			stream atEnd
				ifFalse: [peekChar := stream next].
			peekChar]! !

!XMLPeekableStreamAdapter methodsFor: 'initialization' stamp: 'monty 12/13/2013 05:08'!
setStream: aStream onClose: aBlock
	stream := aStream.
	peekChar := nil.
	onClose := aBlock.! !

Object subclass: #XMLTokenContext
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLTokenContext commentStamp: '<historical>' prior: 0!
The context a token occurs in.!

XMLTokenContext subclass: #HTMLTokenContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTML'!

!HTMLTokenContext commentStamp: '<historical>' prior: 0!
This class adds HTML-specific contexts.!

!HTMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/23/2013 03:40'!
enterNonHTMLComment
	context := #inNonHTMLComment! !

!HTMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/23/2013 03:17'!
enterNonHTMLSection
	context := #inNonHTMLSection! !

!HTMLTokenContext methodsFor: 'testing' stamp: 'monty 11/23/2013 03:18'!
isInContent
	^ super isInContent or: [self isInNonHTMLSection]! !

!HTMLTokenContext methodsFor: 'testing' stamp: 'monty 11/23/2013 03:19'!
isInNonHTMLComment
	^ context == #inNonHTMLComment! !

!HTMLTokenContext methodsFor: 'testing' stamp: 'monty 11/23/2013 03:21'!
isInNonHTMLSection
	^ context == #inNonHTMLSection or: [self isInNonHTMLComment]! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 01:50'!
atEnd
	^ context == #atEnd! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 01:52'!
enterAtEnd
	context := #atEnd! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 01:51'!
enterContent
	context := #inContent! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/22/2013 18:59'!
enterDoctype
	context := #inDoctype! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 01:52'!
enterExternalSubset
	context := #inExternalSubset! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/19/2013 05:13'!
enterExternalSubsetTextDeclaration
	context := #inExternalSubsetTextDeclaration! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 01:52'!
enterInternalSubset
	context := #inInternalSubset! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 07:18'!
enterLiteralValue
	context := #inLiteralValue! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 02:25'!
enterProlog
	context := #inProlog! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/16/2013 05:31'!
enterTextDeclaration
	context := #inTextDeclaration! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 01:51'!
isInContent
	^ context == #inContent! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/22/2013 18:59'!
isInDoctype
	^ context == #inDoctype! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/19/2013 05:11'!
isInExternalSubset
	^ context == #inExternalSubset
		or: [self isInExternalSubsetTextDeclaration]! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/19/2013 05:11'!
isInExternalSubsetTextDeclaration
	^ context == #inExternalSubsetTextDeclaration! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 01:50'!
isInInternalSubset
	^ context == #inInternalSubset! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 07:17'!
isInLiteralValue
	^ context == #inLiteralValue! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 02:25'!
isInProlog
	^ context == #inProlog! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 22:52'!
isInSubset
	^ self isInInternalSubset or: [self isInExternalSubset]! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/19/2013 06:13'!
isInTextDeclaration
	^ context == #inTextDeclaration
		or: [self isInExternalSubsetTextDeclaration]! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 01:30'!
isInitialized
	^ context notNil! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 01:36'!
notInitialized
	^ context isNil! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 18:25'!
resetAfter: aBlock
	| oldContext |

	oldContext := context.
	aBlock value.
	context := oldContext.! !
DTDDFAContentModelParser initialize!
XMLDriverTokenizer initialize!
HTMLTokenizer initialize!
XMLNestedStreamReader initialize!
DTDAttributeDefinitionParser initialize!
HTMLElementNester initialize!
DTDValueListParser initialize!
XMLTokenizer initialize!
